@@ -28,6 +28,7 @@ from sqlspec.core import (
     ParameterProcessor,
     ParameterStyle,
     ParameterStyleConfig,
+    ProcessedState,
     SQLResult,
     StackOperation,
     StackResult,
@@ -38,7 +39,27 @@ from sqlspec.core import (
 from sqlspec.core import filters as filters
 from sqlspec.driver import AsyncDriverAdapterBase, ExecutionResult, SyncDriverAdapterBase
 from sqlspec.exceptions import StackExecutionError
+from sqlspec.extensions.events import (
+    AsyncEventChannel,
+    AsyncEventListener,
+    EventMessage,
+    SyncEventChannel,
+    SyncEventListener,
+)
 from sqlspec.loader import SQLFile, SQLFileLoader
+from sqlspec.observability import (
+    LoggingConfig,
+    ObservabilityConfig,
+    ObservabilityRuntime,
+    RedactionConfig,
+    StatementEvent,
+    StatementObserver,
+    TelemetryConfig,
+    create_event,
+    create_statement_observer,
+    default_statement_observer,
+    format_statement_event,
+)
 from sqlspec.typing import ConnectionT, PoolT, SchemaT, StatementParameters, SupportedSchemaModel
 from sqlspec.utils.logging import suppress_erroneous_sqlglot_log_messages
 
@@ -49,6 +70,8 @@ __all__ = (
     "ArrowResult",
     "AsyncDatabaseConfig",
     "AsyncDriverAdapterBase",
+    "AsyncEventChannel",
+    "AsyncEventListener",
     "CacheConfig",
     "CacheStats",
     "Column",
@@ -57,16 +80,22 @@ __all__ = (
     "CreateTable",
     "Delete",
     "DropTable",
+    "EventMessage",
     "ExecutionResult",
     "FunctionColumn",
     "Insert",
+    "LoggingConfig",
     "Merge",
+    "ObservabilityConfig",
+    "ObservabilityRuntime",
     "ParameterConverter",
     "ParameterProcessor",
     "ParameterStyle",
     "ParameterStyleConfig",
     "PoolT",
+    "ProcessedState",
     "QueryBuilder",
+    "RedactionConfig",
     "SQLFactory",
     "SQLFile",
     "SQLFileLoader",
@@ -79,21 +108,30 @@ __all__ = (
     "StackResult",
     "Statement",
     "StatementConfig",
+    "StatementEvent",
+    "StatementObserver",
     "StatementParameters",
     "StatementStack",
     "SupportedSchemaModel",
     "SyncDatabaseConfig",
     "SyncDriverAdapterBase",
+    "SyncEventChannel",
+    "SyncEventListener",
+    "TelemetryConfig",
     "Update",
     "__version__",
     "adapters",
     "base",
     "builder",
     "core",
+    "create_event",
+    "create_statement_observer",
+    "default_statement_observer",
     "driver",
     "exceptions",
     "extensions",
     "filters",
+    "format_statement_event",
     "loader",
     "migrations",
     "sql",
