@@ -16,6 +16,7 @@ from decimal import Decimal
 from typing import Any, Final, Literal, Protocol, overload
 
 from sqlspec._typing import NUMPY_INSTALLED
+from sqlspec.core.filters import OffsetPagination
 from sqlspec.typing import MSGSPEC_INSTALLED, ORJSON_INSTALLED, PYDANTIC_INSTALLED, BaseModel
 
 
@@ -43,6 +44,8 @@ def _type_to_string(value: Any) -> Any:  # pragma: no cover
         return str(value.value)
     if PYDANTIC_INSTALLED and isinstance(value, BaseModel):
         return value.model_dump_json()
+    if isinstance(value, OffsetPagination):
+        return {"items": value.items, "limit": value.limit, "offset": value.offset, "total": value.total}
     if NUMPY_INSTALLED:
         import numpy as np
 
