@@ -1,4 +1,4 @@
-from sqlspec.adapters.adbc._types import AdbcConnection
+from sqlspec.adapters.adbc._typing import AdbcConnection
 from sqlspec.adapters.adbc.config import AdbcConfig, AdbcConnectionParams
 from sqlspec.adapters.adbc.driver import AdbcCursor, AdbcDriver, AdbcExceptionHandler
 
