@@ -1,21 +1,22 @@
 """AsyncMy ADK store for Google Agent Development Kit session/event storage."""
 
-import json
-from typing import TYPE_CHECKING, Any, Final
+import re
+from typing import TYPE_CHECKING, Any, Final, cast
 
 import asyncmy
 
 from sqlspec.extensions.adk import BaseAsyncADKStore, EventRecord, SessionRecord
-from sqlspec.utils.logging import get_logger
+from sqlspec.extensions.adk.memory.store import BaseAsyncADKMemoryStore
+from sqlspec.utils.serializers import from_json, to_json
 
 if TYPE_CHECKING:
     from datetime import datetime
 
     from sqlspec.adapters.asyncmy.config import AsyncmyConfig
+    from sqlspec.extensions.adk import MemoryRecord
 
-logger = get_logger("adapters.asyncmy.adk.store")
 
-__all__ = ("AsyncmyADKStore",)
+__all__ = ("AsyncmyADKMemoryStore", "AsyncmyADKStore")
 
 MYSQL_TABLE_NOT_FOUND_ERROR: Final = 1146
 
@@ -39,7 +40,7 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
         from sqlspec.adapters.asyncmy.adk import AsyncmyADKStore
 
         config = AsyncmyConfig(
-            pool_config={"host": "localhost", ...},
+            connection_config={"host": "localhost", ...},
             extension_config={
                 "adk": {
                     "session_table": "my_sessions",
@@ -49,7 +50,7 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
             }
         )
         store = AsyncmyADKStore(config)
-        await store.create_tables()
+        await store.ensure_tables()
 
     Notes:
         - MySQL JSON type used (not JSONB) - requires MySQL 5.7.8+
@@ -92,8 +93,6 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
             Input: "tenant_id BIGINT NOT NULL REFERENCES tenants(id) ON DELETE CASCADE"
             Output: ("tenant_id BIGINT NOT NULL", "FOREIGN KEY (tenant_id) REFERENCES tenants(id) ON DELETE CASCADE")
         """
-        import re
-
         references_match = re.search(r"\s+REFERENCES\s+(.+)", column_ddl, re.IGNORECASE)
 
         if not references_match:
@@ -202,7 +201,6 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
         async with self._config.provide_session() as driver:
             await driver.execute_script(await self._get_create_sessions_table_sql())
             await driver.execute_script(await self._get_create_events_table_sql())
-        logger.debug("Created ADK tables: %s, %s", self._session_table, self._events_table)
 
     async def create_session(
         self, session_id: str, app_name: str, user_id: str, state: "dict[str, Any]", owner_id: "Any | None" = None
@@ -224,7 +222,7 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
             State is JSON-serialized before insertion.
             If owner_id_column is configured, owner_id must be provided.
         """
-        state_json = json.dumps(state)
+        state_json = to_json(state)
 
         params: tuple[Any, ...]
         if self._owner_id_column_name:
@@ -279,7 +277,7 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
                     id=session_id_val,
                     app_name=app_name,
                     user_id=user_id,
-                    state=json.loads(state_json) if isinstance(state_json, str) else state_json,
+                    state=from_json(state_json) if isinstance(state_json, str) else state_json,
                     create_time=create_time,
                     update_time=update_time,
                 )
@@ -299,7 +297,7 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
             This replaces the entire state dictionary.
             Uses update_time auto-update trigger.
         """
-        state_json = json.dumps(state)
+        state_json = to_json(state)
 
         sql = f"""
         UPDATE {self._session_table}
@@ -366,7 +364,7 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
                         id=row[0],
                         app_name=row[1],
                         user_id=row[2],
-                        state=json.loads(row[3]) if isinstance(row[3], str) else row[3],
+                        state=from_json(row[3]) if isinstance(row[3], str) else row[3],
                         create_time=row[4],
                         update_time=row[5],
                     )
@@ -387,12 +385,12 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
             Uses UTC_TIMESTAMP(6) for timestamp if not provided.
             JSON fields are serialized before insertion.
         """
-        content_json = json.dumps(event_record.get("content")) if event_record.get("content") else None
+        content_json = to_json(event_record.get("content")) if event_record.get("content") else None
         grounding_metadata_json = (
-            json.dumps(event_record.get("grounding_metadata")) if event_record.get("grounding_metadata") else None
+            to_json(event_record.get("grounding_metadata")) if event_record.get("grounding_metadata") else None
         )
         custom_metadata_json = (
-            json.dumps(event_record.get("custom_metadata")) if event_record.get("custom_metadata") else None
+            to_json(event_record.get("custom_metadata")) if event_record.get("custom_metadata") else None
         )
 
         sql = f"""
@@ -486,9 +484,9 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
                         long_running_tool_ids_json=row[7],
                         branch=row[8],
                         timestamp=row[9],
-                        content=json.loads(row[10]) if row[10] and isinstance(row[10], str) else row[10],
-                        grounding_metadata=json.loads(row[11]) if row[11] and isinstance(row[11], str) else row[11],
-                        custom_metadata=json.loads(row[12]) if row[12] and isinstance(row[12], str) else row[12],
+                        content=from_json(row[10]) if row[10] and isinstance(row[10], str) else row[10],
+                        grounding_metadata=from_json(row[11]) if row[11] and isinstance(row[11], str) else row[11],
+                        custom_metadata=from_json(row[12]) if row[12] and isinstance(row[12], str) else row[12],
                         partial=row[13],
                         turn_complete=row[14],
                         interrupted=row[15],
@@ -501,3 +499,205 @@ class AsyncmyADKStore(BaseAsyncADKStore["AsyncmyConfig"]):
             if "doesn't exist" in str(e) or e.args[0] == MYSQL_TABLE_NOT_FOUND_ERROR:
                 return []
             raise
+
+
+def _parse_owner_id_column_for_mysql(column_ddl: str) -> "tuple[str, str]":
+    """Parse owner ID column DDL for MySQL FOREIGN KEY syntax.
+
+    Args:
+        column_ddl: Column DDL like "tenant_id BIGINT NOT NULL REFERENCES tenants(id) ON DELETE CASCADE".
+
+    Returns:
+        Tuple of (column_definition, foreign_key_constraint).
+    """
+    references_match = re.search(r"\s+REFERENCES\s+(.+)", column_ddl, re.IGNORECASE)
+    if not references_match:
+        return (column_ddl.strip(), "")
+
+    col_def = column_ddl[: references_match.start()].strip()
+    fk_clause = references_match.group(1).strip()
+    col_name = col_def.split()[0]
+    fk_constraint = f"FOREIGN KEY ({col_name}) REFERENCES {fk_clause}"
+    return (col_def, fk_constraint)
+
+
+class AsyncmyADKMemoryStore(BaseAsyncADKMemoryStore["AsyncmyConfig"]):
+    """MySQL/MariaDB ADK memory store using AsyncMy driver."""
+
+    __slots__ = ()
+
+    def __init__(self, config: "AsyncmyConfig") -> None:
+        """Initialize AsyncMy memory store."""
+        super().__init__(config)
+
+    async def _get_create_memory_table_sql(self) -> str:
+        """Get MySQL CREATE TABLE SQL for memory entries."""
+        owner_id_line = ""
+        fk_constraint = ""
+        if self._owner_id_column_ddl:
+            col_def, fk_def = _parse_owner_id_column_for_mysql(self._owner_id_column_ddl)
+            owner_id_line = f",\n            {col_def}"
+            if fk_def:
+                fk_constraint = f",\n            {fk_def}"
+
+        fts_index = ""
+        if self._use_fts:
+            fts_index = f",\n            FULLTEXT INDEX idx_{self._memory_table}_fts (content_text)"
+
+        return f"""
+        CREATE TABLE IF NOT EXISTS {self._memory_table} (
+            id VARCHAR(128) PRIMARY KEY,
+            session_id VARCHAR(128) NOT NULL,
+            app_name VARCHAR(128) NOT NULL,
+            user_id VARCHAR(128) NOT NULL,
+            event_id VARCHAR(128) NOT NULL UNIQUE,
+            author VARCHAR(256){owner_id_line},
+            timestamp TIMESTAMP(6) NOT NULL DEFAULT CURRENT_TIMESTAMP(6),
+            content_json JSON NOT NULL,
+            content_text TEXT NOT NULL,
+            metadata_json JSON,
+            inserted_at TIMESTAMP(6) NOT NULL DEFAULT CURRENT_TIMESTAMP(6),
+            INDEX idx_{self._memory_table}_app_user_time (app_name, user_id, timestamp),
+            INDEX idx_{self._memory_table}_session (session_id){fts_index}{fk_constraint}
+        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci
+        """
+
+    def _get_drop_memory_table_sql(self) -> "list[str]":
+        """Get MySQL DROP TABLE SQL statements."""
+        return [f"DROP TABLE IF EXISTS {self._memory_table}"]
+
+    async def create_tables(self) -> None:
+        """Create the memory table and indexes if they don't exist."""
+        if not self._enabled:
+            return
+
+        async with self._config.provide_session() as driver:
+            await driver.execute_script(await self._get_create_memory_table_sql())
+
+    async def insert_memory_entries(self, entries: "list[MemoryRecord]", owner_id: "object | None" = None) -> int:
+        """Bulk insert memory entries with deduplication."""
+        if not self._enabled:
+            msg = "Memory store is disabled"
+            raise RuntimeError(msg)
+
+        if not entries:
+            return 0
+
+        inserted_count = 0
+        if self._owner_id_column_name:
+            sql = f"""
+            INSERT IGNORE INTO {self._memory_table} (
+                id, session_id, app_name, user_id, event_id, author,
+                {self._owner_id_column_name}, timestamp, content_json,
+                content_text, metadata_json, inserted_at
+            ) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
+            """
+        else:
+            sql = f"""
+            INSERT IGNORE INTO {self._memory_table} (
+                id, session_id, app_name, user_id, event_id, author,
+                timestamp, content_json, content_text, metadata_json, inserted_at
+            ) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
+            """
+
+        async with self._config.provide_connection() as conn:
+            async with conn.cursor() as cursor:
+                for entry in entries:
+                    params: tuple[Any, ...]
+                    if self._owner_id_column_name:
+                        params = (
+                            entry["id"],
+                            entry["session_id"],
+                            entry["app_name"],
+                            entry["user_id"],
+                            entry["event_id"],
+                            entry["author"],
+                            owner_id,
+                            entry["timestamp"],
+                            to_json(entry["content_json"]),
+                            entry["content_text"],
+                            to_json(entry["metadata_json"]),
+                            entry["inserted_at"],
+                        )
+                    else:
+                        params = (
+                            entry["id"],
+                            entry["session_id"],
+                            entry["app_name"],
+                            entry["user_id"],
+                            entry["event_id"],
+                            entry["author"],
+                            entry["timestamp"],
+                            to_json(entry["content_json"]),
+                            entry["content_text"],
+                            to_json(entry["metadata_json"]),
+                            entry["inserted_at"],
+                        )
+                    await cursor.execute(sql, params)
+                    inserted_count += cursor.rowcount
+            await conn.commit()
+        return inserted_count
+
+    async def search_entries(
+        self, query: str, app_name: str, user_id: str, limit: "int | None" = None
+    ) -> "list[MemoryRecord]":
+        """Search memory entries by text query."""
+        if not self._enabled:
+            msg = "Memory store is disabled"
+            raise RuntimeError(msg)
+
+        if not query:
+            return []
+
+        limit_value = limit or self._max_results
+        if self._use_fts:
+            sql = f"""
+            SELECT * FROM {self._memory_table}
+            WHERE app_name = %s AND user_id = %s
+              AND MATCH(content_text) AGAINST (%s IN NATURAL LANGUAGE MODE)
+            ORDER BY timestamp DESC
+            LIMIT %s
+            """
+            params = (app_name, user_id, query, limit_value)
+        else:
+            sql = f"""
+            SELECT * FROM {self._memory_table}
+            WHERE app_name = %s AND user_id = %s AND content_text LIKE %s
+            ORDER BY timestamp DESC
+            LIMIT %s
+            """
+            params = (app_name, user_id, f"%{query}%", limit_value)
+
+        async with self._config.provide_connection() as conn, conn.cursor() as cursor:
+            await cursor.execute(sql, params)
+            rows = await cursor.fetchall()
+            columns = [col[0] for col in cursor.description or []]
+
+        return [cast("MemoryRecord", dict(zip(columns, row, strict=False))) for row in rows]
+
+    async def delete_entries_by_session(self, session_id: str) -> int:
+        """Delete all memory entries for a specific session."""
+        if not self._enabled:
+            msg = "Memory store is disabled"
+            raise RuntimeError(msg)
+
+        sql = f"DELETE FROM {self._memory_table} WHERE session_id = %s"
+        async with self._config.provide_connection() as conn, conn.cursor() as cursor:
+            await cursor.execute(sql, (session_id,))
+            await conn.commit()
+            return cursor.rowcount if cursor.rowcount and cursor.rowcount > 0 else 0
+
+    async def delete_entries_older_than(self, days: int) -> int:
+        """Delete memory entries older than specified days."""
+        if not self._enabled:
+            msg = "Memory store is disabled"
+            raise RuntimeError(msg)
+
+        sql = f"""
+        DELETE FROM {self._memory_table}
+        WHERE inserted_at < (UTC_TIMESTAMP(6) - INTERVAL %s DAY)
+        """
+        async with self._config.provide_connection() as conn, conn.cursor() as cursor:
+            await cursor.execute(sql, (days,))
+            await conn.commit()
+            return cursor.rowcount if cursor.rowcount and cursor.rowcount > 0 else 0
