@@ -1,25 +1,27 @@
 """DuckDB database configuration with connection pooling."""
 
 from collections.abc import Callable, Sequence
-from contextlib import contextmanager
 from typing import TYPE_CHECKING, Any, ClassVar, TypedDict, cast
 
 from typing_extensions import NotRequired
 
-from sqlspec.adapters.duckdb._types import DuckDBConnection
-from sqlspec.adapters.duckdb.driver import (
-    DuckDBCursor,
-    DuckDBDriver,
-    DuckDBExceptionHandler,
-    build_duckdb_statement_config,
+from sqlspec.adapters.duckdb._typing import DuckDBConnection
+from sqlspec.adapters.duckdb.core import (
+    apply_driver_features,
+    build_connection_config,
+    build_statement_config,
+    default_statement_config,
 )
+from sqlspec.adapters.duckdb.driver import DuckDBCursor, DuckDBDriver, DuckDBExceptionHandler, DuckDBSessionContext
 from sqlspec.adapters.duckdb.pool import DuckDBConnectionPool
 from sqlspec.config import ExtensionConfigs, SyncDatabaseConfig
+from sqlspec.extensions.events import EventRuntimeHints
 from sqlspec.observability import ObservabilityConfig
+from sqlspec.utils.config_tools import normalize_connection_config
 from sqlspec.utils.serializers import to_json
 
 if TYPE_CHECKING:
-    from collections.abc import Callable, Generator
+    from collections.abc import Callable
 
     from sqlspec.core import StatementConfig
 __all__ = (
@@ -48,7 +50,7 @@ class DuckDBConnectionParams(TypedDict):
 
     database: NotRequired[str]
     read_only: NotRequired[bool]
-    config: NotRequired[dict[str, Any]]
+    config: NotRequired["dict[str, Any]"]
     memory_limit: NotRequired[str]
     threads: NotRequired[int]
     temp_directory: NotRequired[str]
@@ -79,7 +81,7 @@ class DuckDBConnectionParams(TypedDict):
     binary_as_string: NotRequired[bool]
     arrow_large_buffer_size: NotRequired[bool]
     errors_as_json: NotRequired[bool]
-    extra: NotRequired[dict[str, Any]]
+    extra: NotRequired["dict[str, Any]"]
 
 
 class DuckDBPoolParams(DuckDBConnectionParams):
@@ -93,6 +95,7 @@ class DuckDBPoolParams(DuckDBConnectionParams):
     pool_max_size: NotRequired[int]
     pool_timeout: NotRequired[float]
     pool_recycle_seconds: NotRequired[int]
+    health_check_interval: NotRequired[float]
 
 
 class DuckDBExtensionConfig(TypedDict):
@@ -120,7 +123,7 @@ class DuckDBSecretConfig(TypedDict):
     name: str
     """Name of the secret."""
 
-    value: dict[str, Any]
+    value: "dict[str, Any]"
     """Secret configuration values."""
 
     scope: NotRequired[str]
@@ -141,6 +144,14 @@ class DuckDBDriverFeatures(TypedDict):
             When False, UUID strings are treated as regular strings.
         extension_flags: Connection-level flags (e.g., allow_community_extensions) applied
             via SET statements immediately after connection creation.
+        enable_events: Enable database event channel support.
+            Defaults to True when extension_config["events"] is configured.
+            Provides pub/sub capabilities via table-backed queue (DuckDB has no native pub/sub).
+            Requires extension_config["events"] for migration setup.
+        events_backend: Event channel backend selection.
+            Only option: "table_queue" (durable table-backed queue with retries and exactly-once delivery).
+            DuckDB does not have native pub/sub, so table_queue is the only backend.
+            Defaults to "table_queue".
     """
 
     extensions: NotRequired[Sequence[DuckDBExtensionConfig]]
@@ -148,7 +159,63 @@ class DuckDBDriverFeatures(TypedDict):
     on_connection_create: NotRequired["Callable[[DuckDBConnection], DuckDBConnection | None]"]
     json_serializer: NotRequired["Callable[[Any], str]"]
     enable_uuid_conversion: NotRequired[bool]
-    extension_flags: NotRequired[dict[str, Any]]
+    extension_flags: NotRequired["dict[str, Any]"]
+    enable_events: NotRequired[bool]
+    events_backend: NotRequired[str]
+
+
+class DuckDBConnectionContext:
+    """Context manager for DuckDB connections."""
+
+    __slots__ = ("_config", "_ctx")
+
+    def __init__(self, config: "DuckDBConfig") -> None:
+        self._config = config
+        self._ctx: Any = None
+
+    def __enter__(self) -> DuckDBConnection:
+        pool = self._config.provide_pool()
+        self._ctx = pool.get_connection()
+        return cast("DuckDBConnection", self._ctx.__enter__())
+
+    def __exit__(
+        self, exc_type: "type[BaseException] | None", exc_val: "BaseException | None", exc_tb: Any
+    ) -> bool | None:
+        if self._ctx:
+            return cast("bool | None", self._ctx.__exit__(exc_type, exc_val, exc_tb))
+        return None
+
+
+class _DuckDBConnectionHook:
+    __slots__ = ("_hook",)
+
+    def __init__(self, hook: "Callable[[Any], None]") -> None:
+        self._hook = hook
+
+    def __call__(self, context: "dict[str, Any]") -> None:
+        connection = context.get("connection")
+        if connection is None:
+            return
+        self._hook(connection)
+
+
+class _DuckDBSessionConnectionHandler:
+    __slots__ = ("_config", "_ctx")
+
+    def __init__(self, config: "DuckDBConfig") -> None:
+        self._config = config
+        self._ctx: Any = None
+
+    def acquire_connection(self) -> "DuckDBConnection":
+        pool = self._config.provide_pool()
+        self._ctx = pool.get_connection()
+        return cast("DuckDBConnection", self._ctx.__enter__())
+
+    def release_connection(self, _conn: "DuckDBConnection") -> None:
+        if self._ctx is None:
+            return
+        self._ctx.__exit__(None, None, None)
+        self._ctx = None
 
 
 class DuckDBConfig(SyncDatabaseConfig[DuckDBConnection, DuckDBConnectionPool, DuckDBDriver]):
@@ -187,7 +254,7 @@ class DuckDBConfig(SyncDatabaseConfig[DuckDBConnection, DuckDBConnectionPool, Du
         ...     return msgspec.json.encode(obj).decode("utf-8")
         >>>
         >>> config = DuckDBConfig(
-        ...     pool_config={"database": ":memory:"},
+        ...     connection_config={"database": ":memory:"},
         ...     driver_features={
         ...         "json_serializer": custom_json,
         ...         "enable_uuid_conversion": False,
@@ -207,20 +274,21 @@ class DuckDBConfig(SyncDatabaseConfig[DuckDBConnection, DuckDBConnectionPool, Du
     def __init__(
         self,
         *,
-        pool_config: "DuckDBPoolParams | dict[str, Any] | None" = None,
-        pool_instance: "DuckDBConnectionPool | None" = None,
-        migration_config: dict[str, Any] | None = None,
+        connection_config: "DuckDBPoolParams | dict[str, Any] | None" = None,
+        connection_instance: "DuckDBConnectionPool | None" = None,
+        migration_config: "dict[str, Any] | None" = None,
         statement_config: "StatementConfig | None" = None,
         driver_features: "DuckDBDriverFeatures | dict[str, Any] | None" = None,
         bind_key: "str | None" = None,
         extension_config: "ExtensionConfigs | None" = None,
         observability_config: "ObservabilityConfig | None" = None,
+        **kwargs: Any,
     ) -> None:
         """Initialize DuckDB configuration.
 
         Args:
-            pool_config: Pool configuration parameters
-            pool_instance: Pre-created pool instance
+            connection_config: Connection and pool configuration parameters
+            connection_instance: Pre-created pool instance
             migration_config: Migration configuration
             statement_config: Statement configuration override
             driver_features: DuckDB-specific driver features including json_serializer
@@ -228,70 +296,56 @@ class DuckDBConfig(SyncDatabaseConfig[DuckDBConnection, DuckDBConnectionPool, Du
             bind_key: Optional unique identifier for this configuration
             extension_config: Extension-specific configuration (e.g., Litestar plugin settings)
             observability_config: Adapter-level observability overrides for lifecycle hooks and observers
+            **kwargs: Additional keyword arguments passed to the base configuration.
         """
-        if pool_config is None:
-            pool_config = {}
-        pool_config.setdefault("database", ":memory:shared_db")
+        connection_config = normalize_connection_config(connection_config)
+        connection_config.setdefault("database", ":memory:shared_db")
 
-        if pool_config.get("database") in {":memory:", ""}:
-            pool_config["database"] = ":memory:shared_db"
+        if connection_config.get("database") in {":memory:", ""}:
+            connection_config["database"] = ":memory:shared_db"
 
         extension_flags: dict[str, Any] = {}
-        for key in tuple(pool_config.keys()):
+        for key in tuple(connection_config.keys()):
             if key in EXTENSION_FLAG_KEYS:
-                extension_flags[key] = pool_config.pop(key)  # type: ignore[misc]
+                extension_flags[key] = connection_config.pop(key)
 
-        processed_features: dict[str, Any] = dict(driver_features) if driver_features else {}
-        user_connection_hook = cast(
-            "Callable[[Any], None] | None", processed_features.pop("on_connection_create", None)
-        )
-        processed_features.setdefault("enable_uuid_conversion", True)
-        serializer = processed_features.setdefault("json_serializer", to_json)
+        features: dict[str, Any] = dict(driver_features) if driver_features else {}
+        user_connection_hook = cast("Callable[[Any], None] | None", features.pop("on_connection_create", None))
+        features.setdefault("enable_uuid_conversion", True)
+        serializer = features.setdefault("json_serializer", to_json)
 
         if extension_flags:
-            existing_flags = cast("dict[str, Any]", processed_features.get("extension_flags", {}))
+            existing_flags = cast("dict[str, Any]", features.get("extension_flags", {}))
             merged_flags = {**existing_flags, **extension_flags}
-            processed_features["extension_flags"] = merged_flags
+            features["extension_flags"] = merged_flags
 
         local_observability = observability_config
         if user_connection_hook is not None:
-
-            def _wrap_lifecycle_hook(context: dict[str, Any]) -> None:
-                connection = context.get("connection")
-                if connection is None:
-                    return
-                user_connection_hook(connection)
-
-            lifecycle_override = ObservabilityConfig(lifecycle={"on_connection_create": [_wrap_lifecycle_hook]})
+            lifecycle_override = ObservabilityConfig(
+                lifecycle={"on_connection_create": [_DuckDBConnectionHook(user_connection_hook)]}
+            )
             local_observability = ObservabilityConfig.merge(local_observability, lifecycle_override)
 
-        base_statement_config = statement_config or build_duckdb_statement_config(
+        statement_config = statement_config or build_statement_config(
             json_serializer=cast("Callable[[Any], str]", serializer)
         )
+        statement_config = apply_driver_features(statement_config, features)
 
         super().__init__(
             bind_key=bind_key,
-            pool_config=dict(pool_config),
-            pool_instance=pool_instance,
+            connection_config=connection_config,
+            connection_instance=connection_instance,
             migration_config=migration_config,
-            statement_config=base_statement_config,
-            driver_features=processed_features,
+            statement_config=statement_config,
+            driver_features=features,
             extension_config=extension_config,
             observability_config=local_observability,
+            **kwargs,
         )
 
-    def _get_connection_config_dict(self) -> "dict[str, Any]":
-        """Get connection configuration as plain dict for pool creation."""
-        return {
-            k: v
-            for k, v in self.pool_config.items()
-            if v is not None
-            and k not in {"pool_min_size", "pool_max_size", "pool_timeout", "pool_recycle_seconds", "extra"}
-        }
-
     def _create_pool(self) -> DuckDBConnectionPool:
         """Create connection pool from configuration."""
-        connection_config = self._get_connection_config_dict()
+        connection_config = build_connection_config(self.connection_config)
 
         extensions = self.driver_features.get("extensions", None)
         secrets = self.driver_features.get("secrets", None)
@@ -300,18 +354,26 @@ class DuckDBConfig(SyncDatabaseConfig[DuckDBConnection, DuckDBConnectionPool, Du
         secrets_dicts = [dict(secret) for secret in secrets] if secrets else None
         extension_flags_dict = dict(extension_flags) if extension_flags else None
 
+        pool_recycle_seconds = self.connection_config.get("pool_recycle_seconds")
+        health_check_interval = self.connection_config.get("health_check_interval")
+        pool_kwargs: dict[str, Any] = {}
+        if pool_recycle_seconds is not None:
+            pool_kwargs["pool_recycle_seconds"] = pool_recycle_seconds
+        if health_check_interval is not None:
+            pool_kwargs["health_check_interval"] = health_check_interval
+
         return DuckDBConnectionPool(
             connection_config=connection_config,
             extensions=extensions_dicts,
             extension_flags=extension_flags_dict,
             secrets=secrets_dicts,
-            **self.pool_config,
+            **pool_kwargs,
         )
 
     def _close_pool(self) -> None:
         """Close the connection pool."""
-        if self.pool_instance:
-            self.pool_instance.close()
+        if self.connection_instance:
+            self.connection_instance.close()
 
     def create_connection(self) -> DuckDBConnection:
         """Get a DuckDB connection from the pool.
@@ -333,42 +395,40 @@ class DuckDBConfig(SyncDatabaseConfig[DuckDBConnection, DuckDBConnectionPool, Du
 
         return pool.acquire()
 
-    @contextmanager
-    def provide_connection(self, *args: Any, **kwargs: Any) -> "Generator[DuckDBConnection, None, None]":
+    def provide_connection(self, *args: Any, **kwargs: Any) -> "DuckDBConnectionContext":
         """Provide a pooled DuckDB connection context manager.
 
         Args:
             *args: Additional arguments.
             **kwargs: Additional keyword arguments.
 
-        Yields:
-            A DuckDB connection instance.
+        Returns:
+            A DuckDB connection context manager.
         """
-        pool = self.provide_pool()
-        with pool.get_connection() as connection:
-            yield connection
+        return DuckDBConnectionContext(self)
 
-    @contextmanager
     def provide_session(
-        self, *args: Any, statement_config: "StatementConfig | None" = None, **kwargs: Any
-    ) -> "Generator[DuckDBDriver, None, None]":
+        self, *_args: Any, statement_config: "StatementConfig | None" = None, **_kwargs: Any
+    ) -> "DuckDBSessionContext":
         """Provide a DuckDB driver session context manager.
 
         Args:
-            *args: Additional arguments.
+            *_args: Additional arguments.
             statement_config: Optional statement configuration override.
-            **kwargs: Additional keyword arguments.
+            **_kwargs: Additional keyword arguments.
 
-        Yields:
-            A context manager that yields a DuckDBDriver instance.
+        Returns:
+            A DuckDB driver session context manager.
         """
-        with self.provide_connection(*args, **kwargs) as connection:
-            driver = self.driver_type(
-                connection=connection,
-                statement_config=statement_config or self.statement_config,
-                driver_features=self.driver_features,
-            )
-            yield self._prepare_driver(driver)
+        handler = _DuckDBSessionConnectionHandler(self)
+
+        return DuckDBSessionContext(
+            acquire_connection=handler.acquire_connection,
+            release_connection=handler.release_connection,
+            statement_config=statement_config or self.statement_config or default_statement_config,
+            driver_features=self.driver_features,
+            prepare_driver=self._prepare_driver,
+        )
 
     def get_signature_namespace(self) -> "dict[str, Any]":
         """Get the signature namespace for DuckDB types.
@@ -382,6 +442,7 @@ class DuckDBConfig(SyncDatabaseConfig[DuckDBConnection, DuckDBConnectionPool, Du
 
         namespace = super().get_signature_namespace()
         namespace.update({
+            "DuckDBConnectionContext": DuckDBConnectionContext,
             "DuckDBConnection": DuckDBConnection,
             "DuckDBConnectionParams": DuckDBConnectionParams,
             "DuckDBConnectionPool": DuckDBConnectionPool,
@@ -392,5 +453,11 @@ class DuckDBConfig(SyncDatabaseConfig[DuckDBConnection, DuckDBConnectionPool, Du
             "DuckDBExtensionConfig": DuckDBExtensionConfig,
             "DuckDBPoolParams": DuckDBPoolParams,
             "DuckDBSecretConfig": DuckDBSecretConfig,
+            "DuckDBSessionContext": DuckDBSessionContext,
         })
         return namespace
+
+    def get_event_runtime_hints(self) -> "EventRuntimeHints":
+        """Return polling defaults optimized for DuckDB."""
+
+        return EventRuntimeHints(poll_interval=0.15, lease_seconds=15)
