@@ -1,36 +1,36 @@
 """OracleDB database configuration with direct field-based configuration."""
 
-import contextlib
-import logging
-from contextlib import asynccontextmanager
 from typing import TYPE_CHECKING, Any, ClassVar, TypedDict, cast
 
 import oracledb
+from mypy_extensions import mypyc_attr
 from typing_extensions import NotRequired
 
-from sqlspec.adapters.oracledb._numpy_handlers import register_numpy_handlers
-from sqlspec.adapters.oracledb._types import (
+from sqlspec.adapters.oracledb._numpy_handlers import register_numpy_handlers  # pyright: ignore[reportPrivateUsage]
+from sqlspec.adapters.oracledb._typing import (
     OracleAsyncConnection,
     OracleAsyncConnectionPool,
     OracleSyncConnection,
     OracleSyncConnectionPool,
 )
 from sqlspec.adapters.oracledb._uuid_handlers import register_uuid_handlers
+from sqlspec.adapters.oracledb.core import apply_driver_features, default_statement_config, requires_session_callback
 from sqlspec.adapters.oracledb.driver import (
     OracleAsyncCursor,
     OracleAsyncDriver,
     OracleAsyncExceptionHandler,
+    OracleAsyncSessionContext,
     OracleSyncCursor,
     OracleSyncDriver,
     OracleSyncExceptionHandler,
-    oracledb_statement_config,
+    OracleSyncSessionContext,
 )
 from sqlspec.adapters.oracledb.migrations import OracleAsyncMigrationTracker, OracleSyncMigrationTracker
 from sqlspec.config import AsyncDatabaseConfig, ExtensionConfigs, SyncDatabaseConfig
-from sqlspec.typing import NUMPY_INSTALLED
+from sqlspec.utils.config_tools import normalize_connection_config, reject_pool_aliases
 
 if TYPE_CHECKING:
-    from collections.abc import AsyncGenerator, Callable, Generator
+    from collections.abc import Callable
 
     from oracledb import AuthMode
 
@@ -45,8 +45,6 @@ __all__ = (
     "OracleSyncConfig",
 )
 
-logger = logging.getLogger(__name__)
-
 
 class OracleConnectionParams(TypedDict):
     """OracleDB connection parameters."""
@@ -85,7 +83,7 @@ class OraclePoolParams(OracleConnectionParams):
     max_sessions_per_shard: NotRequired[int]
     soda_metadata_cache: NotRequired[bool]
     ping_interval: NotRequired[int]
-    extra: NotRequired[dict[str, Any]]
+    extra: NotRequired["dict[str, Any]"]
 
 
 class OracleDriverFeatures(TypedDict):
@@ -105,11 +103,68 @@ class OracleDriverFeatures(TypedDict):
         Applies only to RAW(16) columns; other RAW sizes remain unchanged.
         Uses Python's stdlib uuid module (no external dependencies).
         Defaults to True for improved type safety and storage efficiency.
+    enable_events: Enable database event channel support.
+        Defaults to True when extension_config["events"] is configured.
+        Provides pub/sub capabilities via Oracle Advanced Queuing or table-backed fallback.
+        Requires extension_config["events"] for migration setup when using table_queue backend.
+    events_backend: Event channel backend selection.
+        Options: "advanced_queue", "table_queue"
+        - "advanced_queue": Oracle Advanced Queuing (native messaging, requires DBMS_AQADM privileges)
+        - "table_queue": Durable table-backed queue with retries and exactly-once delivery
+        Defaults to "table_queue" (works on all Oracle editions without special privileges).
     """
 
     enable_numpy_vectors: NotRequired[bool]
     enable_lowercase_column_names: NotRequired[bool]
     enable_uuid_binary: NotRequired[bool]
+    enable_events: NotRequired[bool]
+    events_backend: NotRequired[str]
+
+
+class OracleSyncConnectionContext:
+    """Context manager for Oracle sync connections."""
+
+    __slots__ = ("_config", "_conn")
+
+    def __init__(self, config: "OracleSyncConfig") -> None:
+        self._config = config
+        self._conn: OracleSyncConnection | None = None
+
+    def __enter__(self) -> "OracleSyncConnection":
+        if self._config.connection_instance is None:
+            self._config.connection_instance = self._config.create_pool()
+        self._conn = self._config.connection_instance.acquire()
+        return self._conn
+
+    def __exit__(
+        self, exc_type: "type[BaseException] | None", exc_val: "BaseException | None", exc_tb: Any
+    ) -> bool | None:
+        if self._conn:
+            if self._config.connection_instance:
+                self._config.connection_instance.release(self._conn)
+            self._conn = None
+        return None
+
+
+class _OracleSyncSessionConnectionHandler:
+    __slots__ = ("_config", "_conn")
+
+    def __init__(self, config: "OracleSyncConfig") -> None:
+        self._config = config
+        self._conn: OracleSyncConnection | None = None
+
+    def acquire_connection(self) -> "OracleSyncConnection":
+        if self._config.connection_instance is None:
+            self._config.connection_instance = self._config.create_pool()
+        self._conn = self._config.connection_instance.acquire()
+        return self._conn
+
+    def release_connection(self, _conn: "OracleSyncConnection") -> None:
+        if self._conn is None:
+            return
+        if self._config.connection_instance:
+            self._config.connection_instance.release(self._conn)
+        self._conn = None
 
 
 class OracleSyncConfig(SyncDatabaseConfig[OracleSyncConnection, "OracleSyncConnectionPool", OracleSyncDriver]):
@@ -129,55 +184,50 @@ class OracleSyncConfig(SyncDatabaseConfig[OracleSyncConnection, "OracleSyncConne
     def __init__(
         self,
         *,
-        pool_config: "OraclePoolParams | dict[str, Any] | None" = None,
-        pool_instance: "OracleSyncConnectionPool | None" = None,
-        migration_config: dict[str, Any] | None = None,
+        connection_config: "OraclePoolParams | dict[str, Any] | None" = None,
+        connection_instance: "OracleSyncConnectionPool | None" = None,
+        migration_config: "dict[str, Any] | None" = None,
         statement_config: "StatementConfig | None" = None,
         driver_features: "OracleDriverFeatures | dict[str, Any] | None" = None,
         bind_key: "str | None" = None,
         extension_config: "ExtensionConfigs | None" = None,
+        **kwargs: Any,
     ) -> None:
         """Initialize Oracle synchronous configuration.
 
         Args:
-            pool_config: Pool configuration parameters.
-            pool_instance: Existing pool instance to use.
+            connection_config: Connection and pool configuration parameters.
+            connection_instance: Existing pool instance to use.
             migration_config: Migration configuration.
             statement_config: Default SQL statement configuration.
             driver_features: Optional driver feature configuration (TypedDict or dict).
             bind_key: Optional unique identifier for this configuration.
             extension_config: Extension-specific configuration (e.g., Litestar plugin settings).
+            **kwargs: Additional keyword arguments.
         """
+        reject_pool_aliases(kwargs)
 
-        processed_pool_config: dict[str, Any] = dict(pool_config) if pool_config else {}
-        if "extra" in processed_pool_config:
-            extras = processed_pool_config.pop("extra")
-            processed_pool_config.update(extras)
-        statement_config = statement_config or oracledb_statement_config
+        connection_config = normalize_connection_config(connection_config)
+        statement_config = statement_config or default_statement_config
 
-        processed_driver_features: dict[str, Any] = dict(driver_features) if driver_features else {}
-        processed_driver_features.setdefault("enable_numpy_vectors", NUMPY_INSTALLED)
-        processed_driver_features.setdefault("enable_lowercase_column_names", True)
-        processed_driver_features.setdefault("enable_uuid_binary", True)
+        driver_features = apply_driver_features(driver_features)
 
         super().__init__(
-            pool_config=processed_pool_config,
-            pool_instance=pool_instance,
+            connection_config=connection_config,
+            connection_instance=connection_instance,
             migration_config=migration_config,
             statement_config=statement_config,
-            driver_features=processed_driver_features,
+            driver_features=driver_features,
             bind_key=bind_key,
             extension_config=extension_config,
+            **kwargs,
         )
 
     def _create_pool(self) -> "OracleSyncConnectionPool":
         """Create the actual connection pool."""
-        config = dict(self.pool_config)
+        config = dict(self.connection_config)
 
-        needs_session_callback = self.driver_features.get("enable_numpy_vectors", False) or self.driver_features.get(
-            "enable_uuid_binary", False
-        )
-        if needs_session_callback:
+        if requires_session_callback(self.driver_features):
             config["session_callback"] = self._init_connection
 
         return oracledb.create_pool(**config)
@@ -200,8 +250,9 @@ class OracleSyncConfig(SyncDatabaseConfig[OracleSyncConnection, "OracleSyncConne
 
     def _close_pool(self) -> None:
         """Close the actual connection pool."""
-        if self.pool_instance:
-            self.pool_instance.close()
+        if self.connection_instance:
+            self.connection_instance.close()
+            self.connection_instance = None
 
     def create_connection(self) -> "OracleSyncConnection":
         """Create a single connection (not from pool).
@@ -209,47 +260,40 @@ class OracleSyncConfig(SyncDatabaseConfig[OracleSyncConnection, "OracleSyncConne
         Returns:
             An Oracle Connection instance.
         """
-        if self.pool_instance is None:
-            self.pool_instance = self.create_pool()
-        return self.pool_instance.acquire()
+        if self.connection_instance is None:
+            self.connection_instance = self.create_pool()
+        return self.connection_instance.acquire()
 
-    @contextlib.contextmanager
-    def provide_connection(self) -> "Generator[OracleSyncConnection, None, None]":
+    def provide_connection(self) -> "OracleSyncConnectionContext":
         """Provide a connection context manager.
 
-        Yields:
-            An Oracle Connection instance.
+        Returns:
+            An Oracle Connection context manager.
         """
-        if self.pool_instance is None:
-            self.pool_instance = self.create_pool()
-        conn = self.pool_instance.acquire()
-        try:
-            yield conn
-        finally:
-            self.pool_instance.release(conn)
+        return OracleSyncConnectionContext(self)
 
-    @contextlib.contextmanager
     def provide_session(
-        self, *args: Any, statement_config: "StatementConfig | None" = None, **kwargs: Any
-    ) -> "Generator[OracleSyncDriver, None, None]":
+        self, *_args: Any, statement_config: "StatementConfig | None" = None, **_kwargs: Any
+    ) -> "OracleSyncSessionContext":
         """Provide a driver session context manager.
 
         Args:
-            *args: Positional arguments (unused).
+            *_args: Positional arguments (unused).
             statement_config: Optional statement configuration override.
-            **kwargs: Keyword arguments (unused).
+            **_kwargs: Keyword arguments (unused).
 
-        Yields:
-            An OracleSyncDriver instance.
+        Returns:
+            An OracleSyncDriver session context manager.
         """
-        _ = (args, kwargs)  # Mark as intentionally unused
-        with self.provide_connection() as conn:
-            driver = self.driver_type(
-                connection=conn,
-                statement_config=statement_config or self.statement_config,
-                driver_features=self.driver_features,
-            )
-            yield self._prepare_driver(driver)
+        handler = _OracleSyncSessionConnectionHandler(self)
+
+        return OracleSyncSessionContext(
+            acquire_connection=handler.acquire_connection,
+            release_connection=handler.release_connection,
+            statement_config=statement_config or self.statement_config or default_statement_config,
+            driver_features=self.driver_features,
+            prepare_driver=self._prepare_driver,
+        )
 
     def provide_pool(self) -> "OracleSyncConnectionPool":
         """Provide pool instance.
@@ -257,9 +301,9 @@ class OracleSyncConfig(SyncDatabaseConfig[OracleSyncConnection, "OracleSyncConne
         Returns:
             The connection pool.
         """
-        if not self.pool_instance:
-            self.pool_instance = self.create_pool()
-        return self.pool_instance
+        if not self.connection_instance:
+            self.connection_instance = self.create_pool()
+        return self.connection_instance
 
     def get_signature_namespace(self) -> "dict[str, Any]":
         """Get the signature namespace for OracleDB types.
@@ -280,15 +324,64 @@ class OracleSyncConfig(SyncDatabaseConfig[OracleSyncConnection, "OracleSyncConne
             "OracleConnectionParams": OracleConnectionParams,
             "OracleDriverFeatures": OracleDriverFeatures,
             "OraclePoolParams": OraclePoolParams,
+            "OracleSyncConnectionContext": OracleSyncConnectionContext,
             "OracleSyncConnection": OracleSyncConnection,
             "OracleSyncConnectionPool": OracleSyncConnectionPool,
             "OracleSyncCursor": OracleSyncCursor,
             "OracleSyncDriver": OracleSyncDriver,
             "OracleSyncExceptionHandler": OracleSyncExceptionHandler,
+            "OracleSyncSessionContext": OracleSyncSessionContext,
         })
         return namespace
 
 
+class OracleAsyncConnectionContext:
+    """Async context manager for Oracle connections."""
+
+    __slots__ = ("_config", "_conn")
+
+    def __init__(self, config: "OracleAsyncConfig") -> None:
+        self._config = config
+        self._conn: OracleAsyncConnection | None = None
+
+    async def __aenter__(self) -> "OracleAsyncConnection":
+        if self._config.connection_instance is None:
+            self._config.connection_instance = await self._config.create_pool()
+        self._conn = cast("OracleAsyncConnection", await self._config.connection_instance.acquire())
+        return self._conn
+
+    async def __aexit__(
+        self, exc_type: "type[BaseException] | None", exc_val: "BaseException | None", exc_tb: Any
+    ) -> bool | None:
+        if self._conn:
+            if self._config.connection_instance:
+                await self._config.connection_instance.release(self._conn)
+            self._conn = None
+        return None
+
+
+class _OracleAsyncSessionConnectionHandler:
+    __slots__ = ("_config", "_conn")
+
+    def __init__(self, config: "OracleAsyncConfig") -> None:
+        self._config = config
+        self._conn: OracleAsyncConnection | None = None
+
+    async def acquire_connection(self) -> "OracleAsyncConnection":
+        if self._config.connection_instance is None:
+            self._config.connection_instance = await self._config.create_pool()
+        self._conn = cast("OracleAsyncConnection", await self._config.connection_instance.acquire())
+        return self._conn
+
+    async def release_connection(self, _conn: "OracleAsyncConnection") -> None:
+        if self._conn is None:
+            return
+        if self._config.connection_instance:
+            await self._config.connection_instance.release(self._conn)
+        self._conn = None
+
+
+@mypyc_attr(native_class=False)
 class OracleAsyncConfig(AsyncDatabaseConfig[OracleAsyncConnection, "OracleAsyncConnectionPool", OracleAsyncDriver]):
     """Configuration for Oracle asynchronous database connections."""
 
@@ -306,54 +399,49 @@ class OracleAsyncConfig(AsyncDatabaseConfig[OracleAsyncConnection, "OracleAsyncC
     def __init__(
         self,
         *,
-        pool_config: "OraclePoolParams | dict[str, Any] | None" = None,
-        pool_instance: "OracleAsyncConnectionPool | None" = None,
-        migration_config: dict[str, Any] | None = None,
+        connection_config: "OraclePoolParams | dict[str, Any] | None" = None,
+        connection_instance: "OracleAsyncConnectionPool | None" = None,
+        migration_config: "dict[str, Any] | None" = None,
         statement_config: "StatementConfig | None" = None,
         driver_features: "OracleDriverFeatures | dict[str, Any] | None" = None,
         bind_key: "str | None" = None,
         extension_config: "ExtensionConfigs | None" = None,
+        **kwargs: Any,
     ) -> None:
         """Initialize Oracle asynchronous configuration.
 
         Args:
-            pool_config: Pool configuration parameters.
-            pool_instance: Existing pool instance to use.
+            connection_config: Connection and pool configuration parameters.
+            connection_instance: Existing pool instance to use.
             migration_config: Migration configuration.
             statement_config: Default SQL statement configuration.
             driver_features: Optional driver feature configuration (TypedDict or dict).
             bind_key: Optional unique identifier for this configuration.
             extension_config: Extension-specific configuration (e.g., Litestar plugin settings).
+            **kwargs: Additional keyword arguments.
         """
+        reject_pool_aliases(kwargs)
 
-        processed_pool_config: dict[str, Any] = dict(pool_config) if pool_config else {}
-        if "extra" in processed_pool_config:
-            extras = processed_pool_config.pop("extra")
-            processed_pool_config.update(extras)
+        connection_config = normalize_connection_config(connection_config)
 
-        processed_driver_features: dict[str, Any] = dict(driver_features) if driver_features else {}
-        processed_driver_features.setdefault("enable_numpy_vectors", NUMPY_INSTALLED)
-        processed_driver_features.setdefault("enable_lowercase_column_names", True)
-        processed_driver_features.setdefault("enable_uuid_binary", True)
+        driver_features = apply_driver_features(driver_features)
 
         super().__init__(
-            pool_config=processed_pool_config,
-            pool_instance=pool_instance,
+            connection_config=connection_config,
+            connection_instance=connection_instance,
             migration_config=migration_config,
-            statement_config=statement_config or oracledb_statement_config,
-            driver_features=processed_driver_features,
+            statement_config=statement_config or default_statement_config,
+            driver_features=driver_features,
             bind_key=bind_key,
             extension_config=extension_config,
+            **kwargs,
         )
 
     async def _create_pool(self) -> "OracleAsyncConnectionPool":
         """Create the actual async connection pool."""
-        config = dict(self.pool_config)
+        config = dict(self.connection_config)
 
-        needs_session_callback = self.driver_features.get("enable_numpy_vectors", False) or self.driver_features.get(
-            "enable_uuid_binary", False
-        )
-        if needs_session_callback:
+        if requires_session_callback(self.driver_features):
             config["session_callback"] = self._init_connection
 
         return oracledb.create_pool_async(**config)
@@ -376,8 +464,9 @@ class OracleAsyncConfig(AsyncDatabaseConfig[OracleAsyncConnection, "OracleAsyncC
 
     async def _close_pool(self) -> None:
         """Close the actual async connection pool."""
-        if self.pool_instance:
-            await self.pool_instance.close()
+        if self.connection_instance:
+            await self.connection_instance.close()
+            self.connection_instance = None
 
     async def close_pool(self) -> None:
         """Close the connection pool."""
@@ -389,47 +478,40 @@ class OracleAsyncConfig(AsyncDatabaseConfig[OracleAsyncConnection, "OracleAsyncC
         Returns:
             An Oracle AsyncConnection instance.
         """
-        if self.pool_instance is None:
-            self.pool_instance = await self.create_pool()
-        return cast("OracleAsyncConnection", await self.pool_instance.acquire())
+        if self.connection_instance is None:
+            self.connection_instance = await self.create_pool()
+        return cast("OracleAsyncConnection", await self.connection_instance.acquire())
 
-    @asynccontextmanager
-    async def provide_connection(self) -> "AsyncGenerator[OracleAsyncConnection, None]":
+    def provide_connection(self) -> "OracleAsyncConnectionContext":
         """Provide an async connection context manager.
 
-        Yields:
-            An Oracle AsyncConnection instance.
+        Returns:
+            An Oracle AsyncConnection context manager.
         """
-        if self.pool_instance is None:
-            self.pool_instance = await self.create_pool()
-        conn = await self.pool_instance.acquire()
-        try:
-            yield conn
-        finally:
-            await self.pool_instance.release(conn)
+        return OracleAsyncConnectionContext(self)
 
-    @asynccontextmanager
-    async def provide_session(
-        self, *args: Any, statement_config: "StatementConfig | None" = None, **kwargs: Any
-    ) -> "AsyncGenerator[OracleAsyncDriver, None]":
+    def provide_session(
+        self, *_args: Any, statement_config: "StatementConfig | None" = None, **_kwargs: Any
+    ) -> "OracleAsyncSessionContext":
         """Provide an async driver session context manager.
 
         Args:
-            *args: Positional arguments (unused).
+            *_args: Positional arguments (unused).
             statement_config: Optional statement configuration override.
-            **kwargs: Keyword arguments (unused).
+            **_kwargs: Keyword arguments (unused).
 
-        Yields:
-            An OracleAsyncDriver instance.
+        Returns:
+            An OracleAsyncDriver session context manager.
         """
-        _ = (args, kwargs)  # Mark as intentionally unused
-        async with self.provide_connection() as conn:
-            driver = self.driver_type(
-                connection=conn,
-                statement_config=statement_config or self.statement_config,
-                driver_features=self.driver_features,
-            )
-            yield self._prepare_driver(driver)
+        handler = _OracleAsyncSessionConnectionHandler(self)
+
+        return OracleAsyncSessionContext(
+            acquire_connection=handler.acquire_connection,
+            release_connection=handler.release_connection,
+            statement_config=statement_config or self.statement_config or default_statement_config,
+            driver_features=self.driver_features,
+            prepare_driver=self._prepare_driver,
+        )
 
     async def provide_pool(self) -> "OracleAsyncConnectionPool":
         """Provide async pool instance.
@@ -437,26 +519,25 @@ class OracleAsyncConfig(AsyncDatabaseConfig[OracleAsyncConnection, "OracleAsyncC
         Returns:
             The async connection pool.
         """
-        if not self.pool_instance:
-            self.pool_instance = await self.create_pool()
-        return self.pool_instance
+        if not self.connection_instance:
+            self.connection_instance = await self.create_pool()
+        return self.connection_instance
 
     def get_signature_namespace(self) -> "dict[str, Any]":
-        """Get the signature namespace for OracleDB async types.
-
-        Provides OracleDB async-specific types for Litestar framework recognition.
+        """Get the signature namespace for OracleAsyncConfig types.
 
         Returns:
             Dictionary mapping type names to types.
         """
-
         namespace = super().get_signature_namespace()
         namespace.update({
+            "OracleAsyncConnectionContext": OracleAsyncConnectionContext,
             "OracleAsyncConnection": OracleAsyncConnection,
             "OracleAsyncConnectionPool": OracleAsyncConnectionPool,
             "OracleAsyncCursor": OracleAsyncCursor,
             "OracleAsyncDriver": OracleAsyncDriver,
             "OracleAsyncExceptionHandler": OracleAsyncExceptionHandler,
+            "OracleAsyncSessionContext": OracleAsyncSessionContext,
             "OracleConnectionParams": OracleConnectionParams,
             "OracleDriverFeatures": OracleDriverFeatures,
             "OraclePoolParams": OraclePoolParams,
