@@ -11,15 +11,15 @@ from rich.console import Console
 
 from sqlspec.builder import CreateTable, Select, sql
 from sqlspec.migrations.base import BaseMigrationTracker
+from sqlspec.migrations.version import parse_version
 from sqlspec.utils.logging import get_logger
-from sqlspec.utils.version import parse_version
 
 if TYPE_CHECKING:
     from sqlspec.driver import AsyncDriverAdapterBase, SyncDriverAdapterBase
 
 __all__ = ("OracleAsyncMigrationTracker", "OracleSyncMigrationTracker")
 
-logger = get_logger("migrations.oracle")
+logger = get_logger("sqlspec.migrations.oracle")
 console = Console()
 
 
@@ -51,7 +51,8 @@ class OracleMigrationTrackerMixin:
             SQL builder object for Oracle table creation.
         """
         return (
-            sql.create_table(self.version_table)
+            sql
+            .create_table(self.version_table)
             .column("version_num", "VARCHAR2(32)", primary_key=True)
             .column("version_type", "VARCHAR2(16)")
             .column("execution_sequence", "INTEGER")
@@ -73,7 +74,8 @@ class OracleMigrationTrackerMixin:
             SQL builder object for version query.
         """
         return (
-            sql.select('VERSION_NUM AS "version_num"')
+            sql
+            .select('VERSION_NUM AS "version_num"')
             .from_(self.version_table)
             .order_by("EXECUTION_SEQUENCE DESC")
             .limit(1)
@@ -90,7 +92,8 @@ class OracleMigrationTrackerMixin:
             SQL builder object for migrations query.
         """
         return (
-            sql.select(
+            sql
+            .select(
                 'VERSION_NUM AS "version_num"',
                 'VERSION_TYPE AS "version_type"',
                 'EXECUTION_SEQUENCE AS "execution_sequence"',
