@@ -1,30 +1,22 @@
 """Psqlpy database configuration."""
 
-import logging
-from collections.abc import AsyncGenerator
-from contextlib import asynccontextmanager
-from typing import TYPE_CHECKING, Any, ClassVar, TypedDict, cast
+from typing import TYPE_CHECKING, Any, ClassVar, TypedDict
 
+from mypy_extensions import mypyc_attr
 from psqlpy import ConnectionPool
 from typing_extensions import NotRequired
 
-from sqlspec.adapters.psqlpy._types import PsqlpyConnection
-from sqlspec.adapters.psqlpy.driver import (
-    PsqlpyCursor,
-    PsqlpyDriver,
-    PsqlpyExceptionHandler,
-    build_psqlpy_statement_config,
-)
+from sqlspec.adapters.psqlpy._typing import PsqlpyConnection
+from sqlspec.adapters.psqlpy.core import apply_driver_features, build_connection_config, default_statement_config
+from sqlspec.adapters.psqlpy.driver import PsqlpyCursor, PsqlpyDriver, PsqlpyExceptionHandler, PsqlpySessionContext
 from sqlspec.config import AsyncDatabaseConfig, ExtensionConfigs
-from sqlspec.core import StatementConfig
-from sqlspec.typing import PGVECTOR_INSTALLED
-from sqlspec.utils.serializers import to_json
+from sqlspec.extensions.events import EventRuntimeHints
+from sqlspec.utils.config_tools import normalize_connection_config, reject_pool_aliases
 
 if TYPE_CHECKING:
     from collections.abc import Callable
 
-
-logger = logging.getLogger("sqlspec.adapters.psqlpy")
+    from sqlspec.core import StatementConfig
 
 
 class PsqlpyConnectionParams(TypedDict):
@@ -77,7 +69,7 @@ class PsqlpyPoolParams(PsqlpyConnectionParams):
     conn_recycling_method: NotRequired[str]
     max_db_pool_size: NotRequired[int]
     configure: NotRequired["Callable[..., Any]"]
-    extra: NotRequired[dict[str, Any]]
+    extra: NotRequired["dict[str, Any]"]
 
 
 class PsqlpyDriverFeatures(TypedDict):
@@ -89,16 +81,77 @@ class PsqlpyDriverFeatures(TypedDict):
         Provides automatic conversion between NumPy arrays and PostgreSQL vector types.
     json_serializer: Custom JSON serializer applied to the statement configuration.
     json_deserializer: Custom JSON deserializer retained alongside the serializer for parity with asyncpg.
+    enable_events: Enable database event channel support.
+        Defaults to True when extension_config["events"] is configured.
+        Provides pub/sub capabilities via LISTEN/NOTIFY or table-backed fallback.
+        Requires extension_config["events"] for migration setup when using table_queue backend.
+    events_backend: Event channel backend selection.
+        Options: "listen_notify", "table_queue", "listen_notify_durable"
+        - "listen_notify": Zero-copy PostgreSQL LISTEN/NOTIFY (ephemeral, real-time) - coming soon
+        - "table_queue": Durable table-backed queue with retries and exactly-once delivery (current default)
+        - "listen_notify_durable": Hybrid - real-time + durable (available when native support lands)
+        Defaults to "table_queue" until native LISTEN/NOTIFY support is implemented.
     """
 
     enable_pgvector: NotRequired[bool]
     json_serializer: NotRequired["Callable[[Any], str]"]
     json_deserializer: NotRequired["Callable[[str], Any]"]
+    enable_events: NotRequired[bool]
+    events_backend: NotRequired[str]
+
+
+class _PsqlpySessionFactory:
+    __slots__ = ("_config", "_ctx")
+
+    def __init__(self, config: "PsqlpyConfig") -> None:
+        self._config = config
+        self._ctx: Any | None = None
+
+    async def acquire_connection(self) -> "PsqlpyConnection":
+        pool = self._config.connection_instance
+        if pool is None:
+            pool = await self._config.create_pool()
+            self._config.connection_instance = pool
+        ctx = pool.acquire()
+        self._ctx = ctx
+        return await ctx.__aenter__()
+
+    async def release_connection(self, _conn: "PsqlpyConnection") -> None:
+        if self._ctx is not None:
+            await self._ctx.__aexit__(None, None, None)
+            self._ctx = None
 
 
 __all__ = ("PsqlpyConfig", "PsqlpyConnectionParams", "PsqlpyCursor", "PsqlpyDriverFeatures", "PsqlpyPoolParams")
 
 
+class PsqlpyConnectionContext:
+    """Async context manager for Psqlpy connections."""
+
+    __slots__ = ("_config", "_ctx")
+
+    def __init__(self, config: "PsqlpyConfig") -> None:
+        self._config = config
+        self._ctx: Any = None
+
+    async def __aenter__(self) -> PsqlpyConnection:
+        pool = self._config.connection_instance
+        if pool is None:
+            pool = await self._config.create_pool()
+            self._config.connection_instance = pool
+
+        self._ctx = pool.acquire()
+        return await self._ctx.__aenter__()  # type: ignore[no-any-return]
+
+    async def __aexit__(
+        self, exc_type: "type[BaseException] | None", exc_val: "BaseException | None", exc_tb: Any
+    ) -> bool | None:
+        if self._ctx:
+            return await self._ctx.__aexit__(exc_type, exc_val, exc_tb)  # type: ignore[no-any-return]
+        return None
+
+
+@mypyc_attr(native_class=False)
 class PsqlpyConfig(AsyncDatabaseConfig[PsqlpyConnection, ConnectionPool, PsqlpyDriver]):
     """Configuration for Psqlpy asynchronous database connections."""
 
@@ -113,81 +166,56 @@ class PsqlpyConfig(AsyncDatabaseConfig[PsqlpyConnection, ConnectionPool, PsqlpyD
     def __init__(
         self,
         *,
-        pool_config: PsqlpyPoolParams | dict[str, Any] | None = None,
-        pool_instance: ConnectionPool | None = None,
-        migration_config: dict[str, Any] | None = None,
-        statement_config: StatementConfig | None = None,
+        connection_config: "PsqlpyPoolParams | dict[str, Any] | None" = None,
+        connection_instance: ConnectionPool | None = None,
+        migration_config: "dict[str, Any] | None" = None,
+        statement_config: "StatementConfig | None" = None,
         driver_features: "PsqlpyDriverFeatures | dict[str, Any] | None" = None,
         bind_key: str | None = None,
         extension_config: "ExtensionConfigs | None" = None,
+        **kwargs: Any,
     ) -> None:
         """Initialize Psqlpy configuration.
 
         Args:
-            pool_config: Pool configuration parameters.
-            pool_instance: Existing connection pool instance to use.
+            connection_config: Connection and pool configuration parameters.
+            connection_instance: Existing connection pool instance to use.
             migration_config: Migration configuration.
             statement_config: SQL statement configuration.
             driver_features: Driver feature configuration (TypedDict or dict).
             bind_key: Optional unique identifier for this configuration.
             extension_config: Extension-specific configuration (e.g., Litestar plugin settings).
+            **kwargs: Additional keyword arguments.
         """
-        processed_pool_config: dict[str, Any] = dict(pool_config) if pool_config else {}
-        if "extra" in processed_pool_config:
-            extras = processed_pool_config.pop("extra")
-            processed_pool_config.update(extras)
+        reject_pool_aliases(kwargs)
 
-        processed_driver_features: dict[str, Any] = dict(driver_features) if driver_features else {}
-        serializer = processed_driver_features.get("json_serializer")
-        serializer_callable = to_json if serializer is None else cast("Callable[[Any], str]", serializer)
-        processed_driver_features.setdefault("json_serializer", serializer_callable)
-        processed_driver_features.setdefault("enable_pgvector", PGVECTOR_INSTALLED)
+        connection_config = normalize_connection_config(connection_config)
+
+        statement_config = statement_config or default_statement_config
+        statement_config, driver_features = apply_driver_features(statement_config, driver_features)
 
         super().__init__(
-            pool_config=processed_pool_config,
-            pool_instance=pool_instance,
+            connection_config=connection_config,
+            connection_instance=connection_instance,
             migration_config=migration_config,
-            statement_config=statement_config or build_psqlpy_statement_config(json_serializer=serializer_callable),
-            driver_features=processed_driver_features,
+            statement_config=statement_config,
+            driver_features=driver_features,
             bind_key=bind_key,
             extension_config=extension_config,
+            **kwargs,
         )
 
-    def _get_pool_config_dict(self) -> dict[str, Any]:
-        """Get pool configuration as plain dict for external library.
-
-        Returns:
-            Dictionary with pool parameters, filtering out None values.
-        """
-        return {k: v for k, v in self.pool_config.items() if v is not None}
-
     async def _create_pool(self) -> "ConnectionPool":
         """Create the actual async connection pool."""
-        logger.info("Creating psqlpy connection pool", extra={"adapter": "psqlpy"})
-
-        try:
-            config = self._get_pool_config_dict()
-
-            pool = ConnectionPool(**config)
-            logger.info("Psqlpy connection pool created successfully", extra={"adapter": "psqlpy"})
-        except Exception as e:
-            logger.exception("Failed to create psqlpy connection pool", extra={"adapter": "psqlpy", "error": str(e)})
-            raise
-        return pool
+        return ConnectionPool(**build_connection_config(self.connection_config))
 
     async def _close_pool(self) -> None:
         """Close the actual async connection pool."""
-        if not self.pool_instance:
+        if not self.connection_instance:
             return
 
-        logger.info("Closing psqlpy connection pool", extra={"adapter": "psqlpy"})
-
-        try:
-            self.pool_instance.close()
-            logger.info("Psqlpy connection pool closed successfully", extra={"adapter": "psqlpy"})
-        except Exception as e:
-            logger.exception("Failed to close psqlpy connection pool", extra={"adapter": "psqlpy", "error": str(e)})
-            raise
+        self.connection_instance.close()
+        self.connection_instance = None
 
     async def close_pool(self) -> None:
         """Close the connection pool."""
@@ -199,49 +227,46 @@ class PsqlpyConfig(AsyncDatabaseConfig[PsqlpyConnection, ConnectionPool, PsqlpyD
         Returns:
             A psqlpy Connection instance.
         """
-        if not self.pool_instance:
-            self.pool_instance = await self._create_pool()
+        pool = self.connection_instance
+        if pool is None:
+            pool = await self.create_pool()
+            self.connection_instance = pool
 
-        return await self.pool_instance.connection()
+        return await pool.connection()
 
-    @asynccontextmanager
-    async def provide_connection(self, *args: Any, **kwargs: Any) -> AsyncGenerator[PsqlpyConnection, None]:
+    def provide_connection(self, *args: Any, **kwargs: Any) -> "PsqlpyConnectionContext":
         """Provide an async connection context manager.
 
         Args:
             *args: Additional arguments.
             **kwargs: Additional keyword arguments.
 
-        Yields:
-            A psqlpy Connection instance.
+        Returns:
+            A psqlpy Connection context manager.
         """
-        if not self.pool_instance:
-            self.pool_instance = await self._create_pool()
+        return PsqlpyConnectionContext(self)
 
-        async with self.pool_instance.acquire() as conn:
-            yield conn
-
-    @asynccontextmanager
-    async def provide_session(
-        self, *args: Any, statement_config: "StatementConfig | None" = None, **kwargs: Any
-    ) -> AsyncGenerator[PsqlpyDriver, None]:
+    def provide_session(
+        self, *_args: Any, statement_config: "StatementConfig | None" = None, **_kwargs: Any
+    ) -> "PsqlpySessionContext":
         """Provide an async driver session context manager.
 
         Args:
-            *args: Additional arguments.
+            *_args: Additional arguments.
             statement_config: Optional statement configuration override.
-            **kwargs: Additional keyword arguments.
+            **_kwargs: Additional keyword arguments.
 
-        Yields:
-            A PsqlpyDriver instance.
+        Returns:
+            A PsqlpyDriver session context manager.
         """
-        async with self.provide_connection(*args, **kwargs) as conn:
-            driver = self.driver_type(
-                connection=conn,
-                statement_config=statement_config or self.statement_config,
-                driver_features=self.driver_features,
-            )
-            yield self._prepare_driver(driver)
+        factory = _PsqlpySessionFactory(self)
+        return PsqlpySessionContext(
+            acquire_connection=factory.acquire_connection,
+            release_connection=factory.release_connection,
+            statement_config=statement_config or self.statement_config or default_statement_config,
+            driver_features=self.driver_features,
+            prepare_driver=self._prepare_driver,
+        )
 
     async def provide_pool(self, *args: Any, **kwargs: Any) -> ConnectionPool:
         """Provide async pool instance.
@@ -249,9 +274,9 @@ class PsqlpyConfig(AsyncDatabaseConfig[PsqlpyConnection, ConnectionPool, PsqlpyD
         Returns:
             The async connection pool.
         """
-        if not self.pool_instance:
-            self.pool_instance = await self.create_pool()
-        return self.pool_instance
+        if not self.connection_instance:
+            self.connection_instance = await self.create_pool()
+        return self.connection_instance
 
     def get_signature_namespace(self) -> "dict[str, Any]":
         """Get the signature namespace for Psqlpy types.
@@ -261,11 +286,19 @@ class PsqlpyConfig(AsyncDatabaseConfig[PsqlpyConnection, ConnectionPool, PsqlpyD
         """
         namespace = super().get_signature_namespace()
         namespace.update({
+            "PsqlpyConnectionContext": PsqlpyConnectionContext,
             "PsqlpyConnection": PsqlpyConnection,
             "PsqlpyConnectionParams": PsqlpyConnectionParams,
             "PsqlpyCursor": PsqlpyCursor,
             "PsqlpyDriver": PsqlpyDriver,
+            "PsqlpyDriverFeatures": PsqlpyDriverFeatures,
             "PsqlpyExceptionHandler": PsqlpyExceptionHandler,
             "PsqlpyPoolParams": PsqlpyPoolParams,
+            "PsqlpySessionContext": PsqlpySessionContext,
         })
         return namespace
+
+    def get_event_runtime_hints(self) -> "EventRuntimeHints":
+        """Return LISTEN/NOTIFY defaults for Psqlpy adapters."""
+
+        return EventRuntimeHints(poll_interval=0.5, select_for_update=True, skip_locked=True, json_passthrough=True)
