@@ -1,23 +1,22 @@
 """SQLite database configuration with thread-local connections."""
 
-import logging
 import uuid
-from contextlib import contextmanager
 from typing import TYPE_CHECKING, Any, ClassVar, TypedDict, cast
 
 from typing_extensions import NotRequired
 
-from sqlspec.adapters.sqlite._type_handlers import register_type_handlers
-from sqlspec.adapters.sqlite._types import SqliteConnection
-from sqlspec.adapters.sqlite.driver import SqliteCursor, SqliteDriver, SqliteExceptionHandler, sqlite_statement_config
+from sqlspec.adapters.sqlite._typing import SqliteConnection
+from sqlspec.adapters.sqlite.core import apply_driver_features, build_connection_config, default_statement_config
+from sqlspec.adapters.sqlite.driver import SqliteCursor, SqliteDriver, SqliteExceptionHandler, SqliteSessionContext
 from sqlspec.adapters.sqlite.pool import SqliteConnectionPool
+from sqlspec.adapters.sqlite.type_converter import register_type_handlers
 from sqlspec.config import ExtensionConfigs, SyncDatabaseConfig
-from sqlspec.utils.serializers import from_json, to_json
+from sqlspec.utils.logging import get_logger
 
-logger = logging.getLogger(__name__)
+logger = get_logger("sqlspec.adapters.sqlite")
 
 if TYPE_CHECKING:
-    from collections.abc import Callable, Generator
+    from collections.abc import Callable
 
     from sqlspec.core import StatementConfig
     from sqlspec.observability import ObservabilityConfig
@@ -48,16 +47,67 @@ class SqliteDriverFeatures(TypedDict):
         Defaults to sqlspec.utils.serializers.to_json.
     json_deserializer: Custom JSON deserializer function.
         Defaults to sqlspec.utils.serializers.from_json.
+    enable_events: Enable database event channel support.
+        Defaults to True when extension_config["events"] is configured.
+        Provides pub/sub capabilities via table-backed queue (SQLite has no native pub/sub).
+        Requires extension_config["events"] for migration setup.
+    events_backend: Event channel backend selection.
+        Only option: "table_queue" (durable table-backed queue with retries and exactly-once delivery).
+        SQLite does not have native pub/sub, so table_queue is the only backend.
+        Defaults to "table_queue".
     """
 
     enable_custom_adapters: NotRequired[bool]
     json_serializer: "NotRequired[Callable[[Any], str]]"
     json_deserializer: "NotRequired[Callable[[str], Any]]"
+    enable_events: NotRequired[bool]
+    events_backend: NotRequired[str]
 
 
 __all__ = ("SqliteConfig", "SqliteConnectionParams", "SqliteDriverFeatures")
 
 
+class SqliteConnectionContext:
+    """Context manager for Sqlite connections."""
+
+    __slots__ = ("_config", "_ctx")
+
+    def __init__(self, config: "SqliteConfig") -> None:
+        self._config = config
+        self._ctx: Any = None
+
+    def __enter__(self) -> SqliteConnection:
+        pool = self._config.provide_pool()
+        self._ctx = pool.get_connection()
+        return cast("SqliteConnection", self._ctx.__enter__())
+
+    def __exit__(
+        self, exc_type: "type[BaseException] | None", exc_val: "BaseException | None", exc_tb: Any
+    ) -> bool | None:
+        if self._ctx:
+            return cast("bool | None", self._ctx.__exit__(exc_type, exc_val, exc_tb))
+        return None
+
+
+class _SqliteSessionConnectionHandler:
+    __slots__ = ("_config", "_ctx")
+
+    def __init__(self, config: "SqliteConfig") -> None:
+        self._config = config
+        self._ctx: Any = None
+
+    def acquire_connection(self) -> "SqliteConnection":
+        pool = self._config.provide_pool()
+        self._ctx = pool.get_connection()
+        return cast("SqliteConnection", self._ctx.__enter__())
+
+    def release_connection(self, _conn: "SqliteConnection") -> None:
+        if self._ctx is None:
+            return
+        self._ctx.__exit__(None, None, None)
+        self._ctx = None
+
+
 class SqliteConfig(SyncDatabaseConfig[SqliteConnection, SqliteConnectionPool, SqliteDriver]):
     """SQLite configuration with thread-local connections."""
 
@@ -72,76 +122,76 @@ class SqliteConfig(SyncDatabaseConfig[SqliteConnection, SqliteConnectionPool, Sq
     def __init__(
         self,
         *,
-        pool_config: "SqliteConnectionParams | dict[str, Any] | None" = None,
-        pool_instance: "SqliteConnectionPool | None" = None,
+        connection_config: "SqliteConnectionParams | dict[str, Any] | None" = None,
+        connection_instance: "SqliteConnectionPool | None" = None,
         migration_config: "dict[str, Any] | None" = None,
         statement_config: "StatementConfig | None" = None,
         driver_features: "SqliteDriverFeatures | dict[str, Any] | None" = None,
         bind_key: "str | None" = None,
         extension_config: "ExtensionConfigs | None" = None,
         observability_config: "ObservabilityConfig | None" = None,
+        **kwargs: Any,
     ) -> None:
         """Initialize SQLite configuration.
 
         Args:
-            pool_config: Configuration parameters including connection settings
-            pool_instance: Pre-created pool instance
+            connection_config: Configuration parameters including connection settings
+            connection_instance: Pre-created pool instance
             migration_config: Migration configuration
             statement_config: Default SQL statement configuration
             driver_features: Optional driver feature configuration
             bind_key: Optional bind key for the configuration
             extension_config: Extension-specific configuration (e.g., Litestar plugin settings)
             observability_config: Adapter-level observability overrides for lifecycle hooks and observers
+            **kwargs: Additional keyword arguments passed to the base configuration.
         """
-        if pool_config is None:
-            pool_config = {}
-        if "database" not in pool_config or pool_config["database"] == ":memory:":
-            pool_config["database"] = f"file:memory_{uuid.uuid4().hex}?mode=memory&cache=private"
-            pool_config["uri"] = True
-        elif "database" in pool_config:
-            database_path = str(pool_config["database"])
-            if database_path.startswith("file:") and not pool_config.get("uri"):
+        config_dict: dict[str, Any] = dict(connection_config) if connection_config else {}
+        if "database" not in config_dict or config_dict["database"] == ":memory:":
+            config_dict["database"] = f"file:memory_{uuid.uuid4().hex}?mode=memory&cache=private"
+            config_dict["uri"] = True
+        elif "database" in config_dict:
+            database_path = str(config_dict["database"])
+            if database_path.startswith("file:") and not config_dict.get("uri"):
                 logger.debug(
                     "Database URI detected (%s) but uri=True not set. "
                     "Auto-enabling URI mode to prevent physical file creation.",
                     database_path,
                 )
-                pool_config["uri"] = True
+                config_dict["uri"] = True
 
-        processed_driver_features: dict[str, Any] = dict(driver_features) if driver_features else {}
-        processed_driver_features.setdefault("enable_custom_adapters", True)
-        json_serializer = processed_driver_features.setdefault("json_serializer", to_json)
-        json_deserializer = processed_driver_features.setdefault("json_deserializer", from_json)
-
-        base_statement_config = statement_config or sqlite_statement_config
-        if json_serializer is not None:
-            parameter_config = base_statement_config.parameter_config.with_json_serializers(
-                json_serializer, deserializer=json_deserializer
-            )
-            base_statement_config = base_statement_config.replace(parameter_config=parameter_config)
+        statement_config = statement_config or default_statement_config
+        statement_config, driver_features = apply_driver_features(statement_config, driver_features)
 
         super().__init__(
             bind_key=bind_key,
-            pool_instance=pool_instance,
-            pool_config=cast("dict[str, Any]", pool_config),
+            connection_instance=connection_instance,
+            connection_config=config_dict,
             migration_config=migration_config,
-            statement_config=base_statement_config,
-            driver_features=processed_driver_features,
+            statement_config=statement_config,
+            driver_features=driver_features,
             extension_config=extension_config,
             observability_config=observability_config,
+            **kwargs,
         )
 
-    def _get_connection_config_dict(self) -> "dict[str, Any]":
-        """Get connection configuration as plain dict for pool creation."""
-
-        excluded_keys = {"pool_min_size", "pool_max_size", "pool_timeout", "pool_recycle_seconds", "extra"}
-        return {k: v for k, v in self.pool_config.items() if v is not None and k not in excluded_keys}
-
     def _create_pool(self) -> SqliteConnectionPool:
         """Create connection pool from configuration."""
-        config_dict = self._get_connection_config_dict()
+        config_dict = build_connection_config(self.connection_config)
 
-        pool = SqliteConnectionPool(connection_parameters=config_dict, **self.pool_config)
+        pool_kwargs: dict[str, Any] = {}
+        recycle_seconds = self.connection_config.get("pool_recycle_seconds")
+        if recycle_seconds is not None:
+            pool_kwargs["recycle_seconds"] = recycle_seconds
+
+        health_check_interval = self.connection_config.get("health_check_interval")
+        if health_check_interval is not None:
+            pool_kwargs["health_check_interval"] = health_check_interval
+
+        enable_optimizations = self.connection_config.get("enable_optimizations")
+        if enable_optimizations is not None:
+            pool_kwargs["enable_optimizations"] = enable_optimizations
+
+        pool = SqliteConnectionPool(connection_parameters=config_dict, **pool_kwargs)
 
         if self.driver_features.get("enable_custom_adapters", False):
             self._register_type_adapters()
@@ -162,8 +212,8 @@ class SqliteConfig(SyncDatabaseConfig[SqliteConnection, SqliteConnectionPool, Sq
 
     def _close_pool(self) -> None:
         """Close the connection pool."""
-        if self.pool_instance:
-            self.pool_instance.close()
+        if self.connection_instance:
+            self.connection_instance.close()
 
     def create_connection(self) -> SqliteConnection:
         """Get a SQLite connection from the pool.
@@ -174,33 +224,40 @@ class SqliteConfig(SyncDatabaseConfig[SqliteConnection, SqliteConnectionPool, Sq
         pool = self.provide_pool()
         return pool.acquire()
 
-    @contextmanager
-    def provide_connection(self, *args: "Any", **kwargs: "Any") -> "Generator[SqliteConnection, None, None]":
+    def provide_connection(self, *args: "Any", **kwargs: "Any") -> "SqliteConnectionContext":
         """Provide a SQLite connection context manager.
 
-        Yields:
-            SqliteConnection: A thread-local connection
+        Args:
+            *args: Additional arguments.
+            **kwargs: Additional keyword arguments.
+
+        Returns:
+            A Sqlite connection context manager.
         """
-        pool = self.provide_pool()
-        with pool.get_connection() as connection:
-            yield connection
+        return SqliteConnectionContext(self)
 
-    @contextmanager
     def provide_session(
-        self, *args: "Any", statement_config: "StatementConfig | None" = None, **kwargs: "Any"
-    ) -> "Generator[SqliteDriver, None, None]":
+        self, *_args: "Any", statement_config: "StatementConfig | None" = None, **_kwargs: "Any"
+    ) -> "SqliteSessionContext":
         """Provide a SQLite driver session.
 
-        Yields:
-            SqliteDriver: A driver instance with thread-local connection
+        Args:
+            *_args: Additional arguments.
+            statement_config: Optional statement configuration override.
+            **_kwargs: Additional keyword arguments.
+
+        Returns:
+            A Sqlite driver session context manager.
         """
-        with self.provide_connection(*args, **kwargs) as connection:
-            driver = self.driver_type(
-                connection=connection,
-                statement_config=statement_config or self.statement_config,
-                driver_features=self.driver_features,
-            )
-            yield self._prepare_driver(driver)
+        handler = _SqliteSessionConnectionHandler(self)
+
+        return SqliteSessionContext(
+            acquire_connection=handler.acquire_connection,
+            release_connection=handler.release_connection,
+            statement_config=statement_config or self.statement_config or default_statement_config,
+            driver_features=self.driver_features,
+            prepare_driver=self._prepare_driver,
+        )
 
     def get_signature_namespace(self) -> "dict[str, Any]":
         """Get the signature namespace for SQLite types.
@@ -210,6 +267,7 @@ class SqliteConfig(SyncDatabaseConfig[SqliteConnection, SqliteConnectionPool, Sq
         """
         namespace = super().get_signature_namespace()
         namespace.update({
+            "SqliteConnectionContext": SqliteConnectionContext,
             "SqliteConnection": SqliteConnection,
             "SqliteConnectionParams": SqliteConnectionParams,
             "SqliteConnectionPool": SqliteConnectionPool,
@@ -217,5 +275,6 @@ class SqliteConfig(SyncDatabaseConfig[SqliteConnection, SqliteConnectionPool, Sq
             "SqliteDriver": SqliteDriver,
             "SqliteDriverFeatures": SqliteDriverFeatures,
             "SqliteExceptionHandler": SqliteExceptionHandler,
+            "SqliteSessionContext": SqliteSessionContext,
         })
         return namespace
