@@ -3,12 +3,14 @@
 import contextlib
 import sqlite3
 import threading
+import time
 from contextlib import contextmanager
 from typing import TYPE_CHECKING, Any, TypedDict, cast
 
 from typing_extensions import NotRequired
 
-from sqlspec.adapters.sqlite._types import SqliteConnection
+from sqlspec.adapters.sqlite._typing import SqliteConnection
+from sqlspec.utils.logging import get_logger
 
 if TYPE_CHECKING:
     from collections.abc import Generator
@@ -29,6 +31,8 @@ class SqliteConnectionParams(TypedDict):
 
 __all__ = ("SqliteConnectionPool",)
 
+logger = get_logger(__name__)
+
 
 class SqliteConnectionPool:
     """Thread-local connection manager for SQLite.
@@ -38,23 +42,36 @@ class SqliteConnectionPool:
     efficient than a traditional pool for SQLite's constraints.
     """
 
-    __slots__ = ("_connection_parameters", "_enable_optimizations", "_thread_local")
+    __slots__ = (
+        "_connection_parameters",
+        "_enable_optimizations",
+        "_health_check_interval",
+        "_recycle_seconds",
+        "_thread_local",
+    )
 
     def __init__(
-        self, connection_parameters: "dict[str, Any]", enable_optimizations: bool = True, **kwargs: Any
+        self,
+        connection_parameters: "dict[str, Any]",
+        enable_optimizations: bool = True,
+        recycle_seconds: int = 86400,
+        health_check_interval: float = 30.0,
     ) -> None:
         """Initialize the thread-local connection manager.
 
         Args:
             connection_parameters: SQLite connection parameters
             enable_optimizations: Whether to apply performance PRAGMAs
-            **kwargs: Ignored pool parameters for compatibility
+            recycle_seconds: Connection recycle time in seconds (default 24h)
+            health_check_interval: Seconds of idle time before running health check
         """
         if "check_same_thread" not in connection_parameters:
             connection_parameters = {**connection_parameters, "check_same_thread": False}
         self._connection_parameters = connection_parameters
         self._thread_local = threading.local()
         self._enable_optimizations = enable_optimizations
+        self._recycle_seconds = recycle_seconds
+        self._health_check_interval = health_check_interval
 
     def _create_connection(self) -> SqliteConnection:
         """Create a new SQLite connection with optimizations."""
@@ -62,35 +79,76 @@ class SqliteConnectionPool:
 
         if self._enable_optimizations:
             database = self._connection_parameters.get("database", ":memory:")
-            is_memory = database == ":memory:" or "mode=memory" in database
+            is_memory = database == ":memory:" or "mode=memory" in str(database)
 
-            if not is_memory:
-                connection.execute("PRAGMA journal_mode = DELETE")
+            if is_memory:
+                connection.execute("PRAGMA journal_mode = MEMORY")
+                connection.execute("PRAGMA synchronous = OFF")
+                connection.execute("PRAGMA temp_store = MEMORY")
+            else:
+                connection.execute("PRAGMA journal_mode = WAL")
+                connection.execute("PRAGMA synchronous = NORMAL")
                 connection.execute("PRAGMA busy_timeout = 5000")
-                connection.execute("PRAGMA optimize")
 
             connection.execute("PRAGMA foreign_keys = ON")
-            connection.execute("PRAGMA synchronous = NORMAL")
 
         return connection  # type: ignore[no-any-return]
 
+    def _is_connection_alive(self, connection: SqliteConnection) -> bool:
+        """Check if a connection is still alive and usable.
+
+        Args:
+            connection: Connection to check
+
+        Returns:
+            True if connection is alive, False otherwise
+        """
+        try:
+            connection.execute("SELECT 1")
+        except Exception:
+            return False
+        return True
+
     def _get_thread_connection(self) -> SqliteConnection:
         """Get or create a connection for the current thread."""
-        try:
+        thread_state = self._thread_local.__dict__
+        if "connection" not in thread_state:
+            self._thread_local.connection = self._create_connection()
+            self._thread_local.created_at = time.time()
+            self._thread_local.last_used = time.time()
             return cast("SqliteConnection", self._thread_local.connection)
-        except AttributeError:
-            connection = self._create_connection()
-            self._thread_local.connection = connection
-            return connection
+
+        if self._recycle_seconds > 0 and time.time() - self._thread_local.created_at > self._recycle_seconds:
+            logger.debug("SQLite connection exceeded recycle time, recreating")
+            with contextlib.suppress(Exception):
+                self._thread_local.connection.close()
+            self._thread_local.connection = self._create_connection()
+            self._thread_local.created_at = time.time()
+            self._thread_local.last_used = time.time()
+            return cast("SqliteConnection", self._thread_local.connection)
+
+        idle_time = time.time() - thread_state.get("last_used", 0)
+        if idle_time > self._health_check_interval and not self._is_connection_alive(self._thread_local.connection):
+            logger.debug("SQLite connection failed health check after %.1fs idle, recreating", idle_time)
+            with contextlib.suppress(Exception):
+                self._thread_local.connection.close()
+            self._thread_local.connection = self._create_connection()
+            self._thread_local.created_at = time.time()
+
+        self._thread_local.last_used = time.time()
+        return cast("SqliteConnection", self._thread_local.connection)
 
     def _close_thread_connection(self) -> None:
         """Close the connection for the current thread."""
-        try:
-            connection = self._thread_local.connection
-            connection.close()
+        thread_state = self._thread_local.__dict__
+        if "connection" in thread_state:
+            with contextlib.suppress(Exception):
+                self._thread_local.connection.close()
             del self._thread_local.connection
-        except AttributeError:
-            pass
+            if "created_at" in thread_state:
+                del self._thread_local.created_at
+            if "last_used" in thread_state:
+                del self._thread_local.last_used
 
     @contextmanager
     def get_connection(self) -> "Generator[SqliteConnection, None, None]":
