@@ -9,19 +9,22 @@ from collections.abc import Mapping, Sequence
 from datetime import datetime
 from decimal import Decimal
 from itertools import starmap
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, cast
 
+import sqlglot as sg
 from mypy_extensions import trait
 from sqlglot import exp
 from sqlglot.errors import ParseError
 from typing_extensions import Self
 
 from sqlspec.builder._base import QueryBuilder
+from sqlspec.builder._explain import ExplainMixin
 from sqlspec.builder._parsing_utils import extract_sql_object_expression
+from sqlspec.builder._select import is_explicitly_quoted
 from sqlspec.core import SQLResult
 from sqlspec.exceptions import DialectNotSupportedError, SQLBuilderError
 from sqlspec.utils.serializers import to_json
-from sqlspec.utils.type_guards import has_query_builder_parameters
+from sqlspec.utils.type_guards import has_expression_and_sql
 
 if TYPE_CHECKING:
     from sqlglot.dialects.dialect import DialectType
@@ -31,19 +34,12 @@ __all__ = ("Merge",)
 MERGE_UNSUPPORTED_DIALECTS = frozenset({"mysql", "sqlite", "duckdb"})
 
 
+@trait
 class _MergeAssignmentMixin:
     """Shared assignment helpers for MERGE clause mixins."""
 
     __slots__ = ()
 
-    def add_parameter(self, value: Any, name: str | None = None) -> tuple[Any, str]:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
-    def _generate_unique_parameter_name(self, base_name: str) -> str:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
     def _is_column_reference(self, value: str) -> bool:
         """Check if value is a SQL expression rather than a literal string.
 
@@ -53,8 +49,9 @@ class _MergeAssignmentMixin:
         if not isinstance(value, str):
             return False
 
+        builder = cast("QueryBuilder", self)
         with contextlib.suppress(ParseError):
-            parsed: exp.Expression | None = exp.maybe_parse(value.strip())
+            parsed: exp.Expression | None = exp.maybe_parse(value.strip(), dialect=builder.dialect)
             if parsed is None:
                 return False
 
@@ -85,13 +82,14 @@ class _MergeAssignmentMixin:
     def _process_assignment(self, target_column: str, value: Any) -> exp.Expression:
         column_identifier = exp.column(target_column) if isinstance(target_column, str) else target_column
 
-        if hasattr(value, "expression") and hasattr(value, "sql"):
+        if has_expression_and_sql(value):
             value_expr = extract_sql_object_expression(value, builder=self)
             return exp.EQ(this=column_identifier, expression=value_expr)
         if isinstance(value, exp.Expression):
             return exp.EQ(this=column_identifier, expression=value)
         if isinstance(value, str) and self._is_column_reference(value):
-            parsed_expression: exp.Expression | None = exp.maybe_parse(value)
+            builder = cast("QueryBuilder", self)
+            parsed_expression: exp.Expression | None = exp.maybe_parse(value, dialect=builder.dialect)
             if parsed_expression is None:
                 msg = f"Could not parse assignment expression: {value}"
                 raise SQLBuilderError(msg)
@@ -99,9 +97,7 @@ class _MergeAssignmentMixin:
 
         column_name = target_column if isinstance(target_column, str) else str(target_column)
         column_leaf = column_name.split(".")[-1]
-        param_name = self._generate_unique_parameter_name(column_leaf)
-        _, param_name = self.add_parameter(value, name=param_name)
-        placeholder = exp.Placeholder(this=param_name)
+        placeholder, _ = cast("QueryBuilder", self).create_placeholder(value, column_leaf)
         return exp.EQ(this=column_identifier, expression=placeholder)
 
 
@@ -125,6 +121,13 @@ class MergeIntoClauseMixin:
         table_expr: exp.Expression
         if isinstance(table, str):
             table_expr = exp.to_table(table)
+            if is_explicitly_quoted(table):
+                stripped = table.strip('"`')
+                table_expr.set("quoted", True)
+                table_expr.set("this", exp.to_identifier(stripped, quoted=True))
+                cast("QueryBuilder", self)._merge_target_quoted = True  # pyright: ignore[reportPrivateUsage]
+            else:
+                cast("QueryBuilder", self)._merge_target_quoted = False  # pyright: ignore[reportPrivateUsage]
             if alias:
                 table_expr = exp.alias_(table_expr, alias, table=True)
         else:
@@ -143,14 +146,6 @@ class MergeUsingClauseMixin(_MergeAssignmentMixin):
     def get_expression(self) -> exp.Expression | None: ...
     def set_expression(self, expression: exp.Expression) -> None: ...
 
-    def add_parameter(self, value: Any, name: str | None = None) -> tuple[Any, str]:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
-    def _generate_unique_parameter_name(self, base_name: str) -> str:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
     def _create_dict_source_expression(
         self, source: "dict[str, Any] | list[dict[str, Any]]", alias: "str | None"
     ) -> "exp.Expression":
@@ -182,7 +177,8 @@ class MergeUsingClauseMixin(_MergeAssignmentMixin):
             raise SQLBuilderError(msg)
 
         columns = list(data[0].keys())
-        dialect = getattr(self, "dialect_name", None)
+        builder = cast("QueryBuilder", self)
+        dialect = builder.dialect_name
 
         if dialect == "postgres":
             return self._create_postgres_json_source(data, columns, is_list, alias)
@@ -203,9 +199,8 @@ class MergeUsingClauseMixin(_MergeAssignmentMixin):
         mechanisms (AsyncPG json_serializer, Psycopg/Psqlpy JSON codecs).
         """
 
-        json_param_name = self._generate_unique_parameter_name("json_data")
         json_value = data if is_list else [data[0]]
-        _, json_param_name = self.add_parameter(json_value, name=json_param_name)
+        _, json_param_name = cast("QueryBuilder", self).create_placeholder(json_value, "json_data")
 
         sample_values: dict[str, Any] = {}
         for record in data:
@@ -214,25 +209,29 @@ class MergeUsingClauseMixin(_MergeAssignmentMixin):
                     sample_values[column] = value
 
         alias_name = alias or "src"
+        recordset_alias = f"{alias_name}_data"
 
         column_type_spec = ", ".join([f"{col} {self._infer_postgres_type(sample_values.get(col))}" for col in columns])
         column_selects = ", ".join(columns)
-        from_sql = f"SELECT {column_selects} FROM jsonb_to_recordset(:{json_param_name}::jsonb) AS {alias_name}({column_type_spec})"
-
-        import sqlglot as sg
+        from_sql = (
+            f"SELECT {column_selects} FROM jsonb_to_recordset(:{json_param_name}::jsonb) AS "
+            f"{recordset_alias}({column_type_spec})"
+        )
 
         parsed = sg.parse_one(from_sql, dialect="postgres")
-        paren_expr = exp.paren(parsed)
-        paren_expr.set("alias", exp.TableAlias(this=exp.to_identifier(alias_name)))
-        return paren_expr
+        return exp.Subquery(
+            this=parsed,
+            alias=exp.TableAlias(
+                this=exp.to_identifier(alias_name), columns=[exp.to_identifier(col) for col in columns]
+            ),
+        )
 
     def _create_oracle_json_source(
         self, data: "list[dict[str, Any]]", columns: "list[str]", alias: "str | None"
     ) -> "exp.Expression":
         """Create Oracle JSON_TABLE source (production-proven pattern from oracledb-vertexai-demo)."""
-        json_param_name = self._generate_unique_parameter_name("json_payload")
         json_value = to_json(data)
-        _, json_param_name = self.add_parameter(json_value, name=json_param_name)
+        _, json_param_name = cast("QueryBuilder", self).create_placeholder(json_value, "json_payload")
 
         sample_values: dict[str, Any] = {}
         for record in data:
@@ -250,12 +249,8 @@ class MergeUsingClauseMixin(_MergeAssignmentMixin):
 
         from_sql = f"SELECT {column_selects} FROM JSON_TABLE(:{json_param_name}, '$[*]' COLUMNS ({columns_clause}))"
 
-        import sqlglot as sg
-
         parsed = sg.parse_one(from_sql, dialect="oracle")
-        paren_expr = exp.paren(parsed)
-        paren_expr.set("alias", exp.TableAlias(this=exp.to_identifier(alias_name)))
-        return paren_expr
+        return exp.Subquery(this=parsed, alias=exp.TableAlias(this=exp.to_identifier(alias_name)))
 
     def _infer_postgres_type(self, value: "Any") -> str:
         """Infer PostgreSQL column type from Python value.
@@ -310,9 +305,8 @@ class MergeUsingClauseMixin(_MergeAssignmentMixin):
                 column_name = column if isinstance(column, str) else str(column)
                 if "." in column_name:
                     column_name = column_name.split(".")[-1]
-                param_name = self._generate_unique_parameter_name(column_name)
-                _, param_name = self.add_parameter(value, name=param_name)
-                row_params.append(exp.Placeholder(this=param_name))
+                placeholder, _ = cast("QueryBuilder", self).create_placeholder(value, column_name)
+                row_params.append(placeholder)
             parameterized_values.append(row_params)
 
         if is_list:
@@ -358,17 +352,11 @@ class MergeUsingClauseMixin(_MergeAssignmentMixin):
                 source_expr = exp.Subquery(this=paren_expr.this, alias=exp.to_identifier(alias))
             else:
                 source_expr = paren_expr
-        elif has_query_builder_parameters(source) and hasattr(source, "_expression"):
-            parameters_obj = getattr(source, "parameters", None)
-            if isinstance(parameters_obj, dict):
-                for param_name, param_value in parameters_obj.items():
-                    self.add_parameter(param_value, name=param_name)
-            elif isinstance(parameters_obj, (list, tuple)):
-                for param_value in parameters_obj:
-                    self.add_parameter(param_value)
-            elif parameters_obj is not None:
-                self.add_parameter(parameters_obj)
-            subquery_expression_source = getattr(source, "_expression", None)
+        elif isinstance(source, QueryBuilder):
+            builder = cast("QueryBuilder", self)
+            for param_name, param_value in source.parameters.items():
+                builder.add_parameter(param_value, name=param_name)
+            subquery_expression_source = source.get_expression()
             if not isinstance(subquery_expression_source, exp.Expression):
                 subquery_expression_source = exp.select()
 
@@ -417,7 +405,8 @@ class MergeOnClauseMixin:
 
         assert current_expr is not None
         if isinstance(condition, str):
-            parsed_condition: exp.Expression | None = exp.maybe_parse(condition, dialect=getattr(self, "dialect", None))
+            builder = cast("QueryBuilder", self)
+            parsed_condition: exp.Expression | None = exp.maybe_parse(condition, dialect=builder.dialect)
             if parsed_condition is None:
                 msg = f"Could not parse ON condition: {condition}"
                 raise SQLBuilderError(msg)
@@ -441,14 +430,6 @@ class MergeMatchedClauseMixin(_MergeAssignmentMixin):
     def get_expression(self) -> exp.Expression | None: ...
     def set_expression(self, expression: exp.Expression) -> None: ...
 
-    def add_parameter(self, value: Any, name: str | None = None) -> tuple[Any, str]:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
-    def _generate_unique_parameter_name(self, base_name: str) -> str:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
     def when_matched_then_update(
         self,
         set_values: dict[str, Any] | None = None,
@@ -477,19 +458,25 @@ class MergeMatchedClauseMixin(_MergeAssignmentMixin):
         when_kwargs: dict[str, Any] = {"matched": True, "then": update_expression}
         if condition is not None:
             if isinstance(condition, str):
-                parsed_condition: exp.Expression | None = exp.maybe_parse(
-                    condition, dialect=getattr(self, "dialect", None)
-                )
+                builder = cast("QueryBuilder", self)
+                parsed_condition: exp.Expression | None = exp.maybe_parse(condition, dialect=builder.dialect)
                 if parsed_condition is None:
                     msg = f"Could not parse WHEN clause condition: {condition}"
                     raise SQLBuilderError(msg)
-                when_kwargs["this"] = parsed_condition
+                condition_expr = parsed_condition
             elif isinstance(condition, exp.Expression):
-                when_kwargs["this"] = condition
+                condition_expr = condition
             else:
                 msg = f"Unsupported condition type for WHEN clause: {type(condition)}"
                 raise SQLBuilderError(msg)
 
+            builder = cast("QueryBuilder", self)
+            dialect_name = builder.dialect_name
+            if dialect_name == "oracle":
+                update_expression.set("where", condition_expr)
+            else:
+                when_kwargs["condition"] = condition_expr
+
         whens = current_expr.args.get("whens")
         if not isinstance(whens, exp.Whens):
             whens = exp.Whens(expressions=[])
@@ -507,9 +494,8 @@ class MergeMatchedClauseMixin(_MergeAssignmentMixin):
         when_kwargs: dict[str, Any] = {"matched": True, "then": exp.Var(this="DELETE")}
         if condition is not None:
             if isinstance(condition, str):
-                parsed_condition: exp.Expression | None = exp.maybe_parse(
-                    condition, dialect=getattr(self, "dialect", None)
-                )
+                builder = cast("QueryBuilder", self)
+                parsed_condition: exp.Expression | None = exp.maybe_parse(condition, dialect=builder.dialect)
                 if parsed_condition is None:
                     msg = f"Could not parse WHEN clause condition: {condition}"
                     raise SQLBuilderError(msg)
@@ -537,14 +523,6 @@ class MergeNotMatchedClauseMixin(_MergeAssignmentMixin):
     def get_expression(self) -> exp.Expression | None: ...
     def set_expression(self, expression: exp.Expression) -> None: ...
 
-    def add_parameter(self, value: Any, name: str | None = None) -> tuple[Any, str]:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
-    def _generate_unique_parameter_name(self, base_name: str) -> str:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
     def when_not_matched_then_insert(
         self,
         columns: Mapping[str, Any] | Sequence[str] | None = None,
@@ -578,10 +556,13 @@ class MergeNotMatchedClauseMixin(_MergeAssignmentMixin):
             if values is None:
                 using_alias = None
                 using_expr = current_expr.args.get("using")
-                if using_expr is not None and (
-                    isinstance(using_expr, (exp.Subquery, exp.Table)) or hasattr(using_expr, "alias")
-                ):
+                if using_expr is not None and isinstance(using_expr, (exp.Subquery, exp.Table)):
                     using_alias = using_expr.alias
+                elif using_expr is not None:
+                    try:
+                        using_alias = using_expr.alias
+                    except AttributeError:
+                        using_alias = None
                 column_values = [f"{using_alias}.{col}" for col in column_names] if using_alias else column_names
             else:
                 column_values = list(values)
@@ -593,25 +574,24 @@ class MergeNotMatchedClauseMixin(_MergeAssignmentMixin):
 
         insert_values: list[exp.Expression] = []
         for column_name, value in zip(column_names, column_values, strict=True):
-            if hasattr(value, "expression") and hasattr(value, "sql"):
+            if has_expression_and_sql(value):
                 insert_values.append(extract_sql_object_expression(value, builder=self))
             elif isinstance(value, exp.Expression):
                 insert_values.append(value)
             elif isinstance(value, str):
                 if self._is_column_reference(value):
-                    parsed_value: exp.Expression | None = exp.maybe_parse(value, dialect=getattr(self, "dialect", None))
+                    builder = cast("QueryBuilder", self)
+                    parsed_value: exp.Expression | None = exp.maybe_parse(value, dialect=builder.dialect)
                     if parsed_value is None:
                         msg = f"Could not parse column reference: {value}"
                         raise SQLBuilderError(msg)
                     insert_values.append(parsed_value)
                 else:
-                    param_name = self._generate_unique_parameter_name(column_name.split(".")[-1])
-                    _, param_name = self.add_parameter(value, name=param_name)
-                    insert_values.append(exp.Placeholder(this=param_name))
+                    placeholder, _ = cast("QueryBuilder", self).create_placeholder(value, column_name.split(".")[-1])
+                    insert_values.append(placeholder)
             else:
-                param_name = self._generate_unique_parameter_name(column_name.split(".")[-1])
-                _, param_name = self.add_parameter(value, name=param_name)
-                insert_values.append(exp.Placeholder(this=param_name))
+                placeholder, _ = cast("QueryBuilder", self).create_placeholder(value, column_name.split(".")[-1])
+                insert_values.append(placeholder)
 
         insert_expr.set("this", exp.Tuple(expressions=insert_columns))
         insert_expr.set("expression", exp.Tuple(expressions=insert_values))
@@ -632,14 +612,6 @@ class MergeNotMatchedBySourceClauseMixin(_MergeAssignmentMixin):
     def get_expression(self) -> exp.Expression | None: ...
     def set_expression(self, expression: exp.Expression) -> None: ...
 
-    def add_parameter(self, value: Any, name: str | None = None) -> tuple[Any, str]:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
-    def _generate_unique_parameter_name(self, base_name: str) -> str:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
     def when_not_matched_by_source_then_update(
         self, set_values: dict[str, Any] | None = None, **assignments: Any
     ) -> Self:
@@ -662,20 +634,20 @@ class MergeNotMatchedBySourceClauseMixin(_MergeAssignmentMixin):
         set_expressions: list[exp.Expression] = []
         for column_name, value in combined_assignments.items():
             column_identifier = exp.column(column_name)
-            if hasattr(value, "expression") and hasattr(value, "sql"):
+            if has_expression_and_sql(value):
                 value_expr = extract_sql_object_expression(value, builder=self)
             elif isinstance(value, exp.Expression):
                 value_expr = value
             elif isinstance(value, str) and self._is_column_reference(value):
-                parsed_value: exp.Expression | None = exp.maybe_parse(value)
+                builder = cast("QueryBuilder", self)
+                parsed_value: exp.Expression | None = exp.maybe_parse(value, dialect=builder.dialect)
                 if parsed_value is None:
                     msg = f"Could not parse assignment expression: {value}"
                     raise SQLBuilderError(msg)
                 value_expr = parsed_value
             else:
-                param_name = self._generate_unique_parameter_name(column_name)
-                _, param_name = self.add_parameter(value, name=param_name)
-                value_expr = exp.Placeholder(this=param_name)
+                placeholder, _ = cast("QueryBuilder", self).create_placeholder(value, column_name)
+                value_expr = placeholder
             set_expressions.append(exp.EQ(this=column_identifier, expression=value_expr))
 
         update_expr = exp.Update(expressions=set_expressions)
@@ -709,6 +681,7 @@ class Merge(
     MergeNotMatchedClauseMixin,
     MergeIntoClauseMixin,
     MergeNotMatchedBySourceClauseMixin,
+    ExplainMixin,
 ):
     """Builder for MERGE statements.
 
@@ -718,6 +691,8 @@ class Merge(
 
     __slots__ = ()
     _expression: exp.Expression | None
+    _merge_target_quoted: bool
+    _lock_targets_quoted: bool
 
     def __init__(self, target_table: str | None = None, **kwargs: Any) -> None:
         """Initialize MERGE with optional target table.
@@ -728,7 +703,19 @@ class Merge(
         """
         if "enable_optimization" not in kwargs:
             kwargs["enable_optimization"] = False
-        super().__init__(**kwargs)
+        (dialect, schema, enable_optimization, optimize_joins, optimize_predicates, simplify_expressions) = (
+            self._parse_query_builder_kwargs(kwargs)
+        )
+        super().__init__(
+            dialect=dialect,
+            schema=schema,
+            enable_optimization=enable_optimization,
+            optimize_joins=optimize_joins,
+            optimize_predicates=optimize_predicates,
+            simplify_expressions=simplify_expressions,
+        )
+        self._merge_target_quoted = False
+        self._lock_targets_quoted = False
         self._initialize_expression()
 
         if target_table:
@@ -792,9 +779,43 @@ class Merge(
 
         Returns:
             Built statement object.
-
-        Raises:
-            DialectNotSupportedError: If dialect does not support MERGE.
         """
         self._validate_dialect_support()
+        target_dialect = dialect or self.dialect
+        if isinstance(target_dialect, str):
+            dialect_name = target_dialect
+        elif isinstance(target_dialect, type):
+            dialect_name = target_dialect.__name__
+        else:
+            dialect_name = None
+        if dialect_name:
+            dialect_name = dialect_name.lower()
+        self._normalize_merge_conditions_for_dialect(dialect_name)
         return super().build(dialect=dialect)
+
+    def _normalize_merge_conditions_for_dialect(self, dialect_name: str | None) -> None:
+        """Normalize WHEN clause conditions for dialect quirks (e.g., Oracle).
+
+        Oracle requires conditional logic on UPDATE/DELETE branches to live in the
+        clause-specific WHERE, not on the WHEN predicate. Move the condition down
+        when present so generated SQL matches Oracle syntax.
+        """
+        if dialect_name != "oracle":
+            return
+
+        merge_expr = self.get_expression()
+        if not isinstance(merge_expr, exp.Merge):
+            return
+
+        whens = merge_expr.args.get("whens")
+        if not isinstance(whens, exp.Whens):
+            return
+
+        for when_expr in list(whens.expressions):
+            condition_expr = when_expr.args.get("condition")
+            if condition_expr is None:
+                continue
+            then_expr = when_expr.args.get("then")
+            if isinstance(then_expr, exp.Update):
+                then_expr.set("where", condition_expr)
+                when_expr.set("condition", None)
