@@ -13,7 +13,8 @@ from mypy_extensions import trait
 from sqlglot import exp
 from typing_extensions import Self
 
-from sqlspec.builder._base import QueryBuilder, SafeQuery
+from sqlspec.builder._base import BuiltQuery, QueryBuilder
+from sqlspec.builder._explain import ExplainMixin
 from sqlspec.builder._join import JoinClauseMixin
 from sqlspec.builder._parsing_utils import (
     extract_column_name,
@@ -27,9 +28,8 @@ from sqlspec.builder._parsing_utils import (
 from sqlspec.core import SQL, ParameterStyle, ParameterValidator, SQLResult
 from sqlspec.exceptions import SQLBuilderError
 from sqlspec.utils.type_guards import (
-    has_expression_and_parameters,
     has_expression_and_sql,
-    has_query_builder_parameters,
+    has_parameter_builder,
     has_sqlglot_expression,
     is_expression,
     is_iterable_parameters,
@@ -39,6 +39,7 @@ BETWEEN_BOUND_COUNT = 2
 PAIR_LENGTH = 2
 TRIPLE_LENGTH = 3
 
+
 if TYPE_CHECKING:
     from collections.abc import Callable
 
@@ -67,6 +68,74 @@ __all__ = (
 )
 
 
+def is_explicitly_quoted(identifier: Any) -> bool:
+    """Detect if identifier was provided with explicit quotes."""
+    if not isinstance(identifier, str):
+        return False
+    stripped = identifier.strip()
+    return (stripped.startswith('"') and stripped.endswith('"')) or (
+        stripped.startswith("`") and stripped.endswith("`")
+    )
+
+
+def _expr_eq(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return exp.EQ(this=col, expression=placeholder)
+
+
+def _expr_neq(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return exp.NEQ(this=col, expression=placeholder)
+
+
+def _expr_gt(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return exp.GT(this=col, expression=placeholder)
+
+
+def _expr_gte(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return exp.GTE(this=col, expression=placeholder)
+
+
+def _expr_lt(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return exp.LT(this=col, expression=placeholder)
+
+
+def _expr_lte(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return exp.LTE(this=col, expression=placeholder)
+
+
+def _expr_like_exp(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return exp.Like(this=col, expression=placeholder)
+
+
+def _expr_like_method(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return cast("exp.Expression", col.like(placeholder))
+
+
+def _expr_not_like(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return exp.Not(this=exp.Like(this=col, expression=placeholder))
+
+
+def _expr_like_not(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return exp.Not(this=cast("exp.Expression", col.like(placeholder)))
+
+
+def _expr_ilike(col: "exp.Expression", placeholder: "exp.Placeholder") -> "exp.Expression":
+    return cast("exp.Expression", col.ilike(placeholder))
+
+
+_SIMPLE_OPERATOR_MAP: dict[str, Any] = {
+    "=": _expr_eq,
+    "==": _expr_eq,
+    "!=": _expr_neq,
+    "<>": _expr_neq,
+    ">": _expr_gt,
+    ">=": _expr_gte,
+    "<": _expr_lt,
+    "<=": _expr_lte,
+    "LIKE": _expr_like_exp,
+    "NOT LIKE": _expr_not_like,
+}
+
+
 class Case:
     """Represent a SQL CASE expression with structured components."""
 
@@ -119,16 +188,18 @@ class SubqueryBuilder:
     def __call__(self, subquery: Any) -> exp.Expression:
         if isinstance(subquery, exp.Expression):
             subquery_expr = subquery
-        elif hasattr(subquery, "build") and callable(getattr(subquery, "build", None)):
+        elif has_parameter_builder(subquery):
             built_query = subquery.build()
-            sql_text = built_query.sql if hasattr(built_query, "sql") else str(built_query)
-            parsed_expr: exp.Expression | None = exp.maybe_parse(sql_text)
+            sql_text = built_query.sql if isinstance(built_query, BuiltQuery) else str(built_query)
+            dialect = subquery.dialect if isinstance(subquery, QueryBuilder) else None
+            parsed_expr: exp.Expression | None = exp.maybe_parse(sql_text, dialect=dialect)
             if parsed_expr is None:
                 msg = f"Could not parse subquery SQL: {sql_text}"
                 raise SQLBuilderError(msg)
             subquery_expr = parsed_expr
         else:
-            parsed_expr = exp.maybe_parse(str(subquery))
+            dialect = subquery.dialect if isinstance(subquery, (QueryBuilder, BuiltQuery)) else None
+            parsed_expr = exp.maybe_parse(str(subquery), dialect=dialect)
             if parsed_expr is None:
                 msg = f"Could not convert subquery to expression: {subquery}"
                 raise SQLBuilderError(msg)
@@ -236,20 +307,60 @@ class SelectClauseMixin:
         builder.set_expression(select_expr)
         return cast("Self", builder)
 
-    def from_(self, table: str | exp.Expression | Any, alias: str | None = None) -> Self:
+    def from_(
+        self,
+        table: str | exp.Expression | Any,
+        alias: str | None = None,
+        as_of: Any | None = None,
+        as_of_type: str | None = None,
+    ) -> Self:
+        """Set the FROM clause and optionally attach temporal versioning.
+
+        ``as_of`` copies the resolved table expression, normalizes aliases, and adds an ``exp.Version`` so sqlglot's generator emits dialect-specific time-travel SQL.
+        """
         builder = cast("SQLBuilderProtocol", self)
         select_expr = _ensure_select_expression(builder, error_message="FROM clause only valid for SELECT.")
+        from_expr: exp.Expression
+
         if isinstance(table, str):
             from_expr = parse_table_expression(table, alias)
         elif is_expression(table):
             from_expr = exp.alias_(table, alias) if alias else table
-        elif has_query_builder_parameters(table):
-            subquery = table.build()
-            sql_text = subquery.sql if hasattr(subquery, "sql") and not callable(subquery.sql) else str(subquery)
-            subquery_exp = exp.paren(exp.maybe_parse(sql_text, dialect=getattr(builder, "dialect", None)))
-            from_expr = exp.alias_(subquery_exp, alias) if alias else subquery_exp
+        elif has_parameter_builder(table):
+            subquery_expression = table.get_expression()
+            if subquery_expression is None:
+                msg = "Subquery builder has no expression to include in FROM clause."
+                raise SQLBuilderError(msg)
+
+            subquery_copy = subquery_expression.copy()
+            base_builder = cast("QueryBuilder", builder)
+            param_mapping = base_builder._merge_cte_parameters(alias or "subquery", table.parameters)
+            if param_mapping:
+                subquery_copy = base_builder._update_placeholders_in_expression(subquery_copy, param_mapping)
+
+            wrapped_subquery = exp.paren(subquery_copy)
+            from_expr = exp.alias_(wrapped_subquery, alias) if alias else wrapped_subquery
         else:
             from_expr = table
+
+        if as_of is not None:
+            inner_expr = from_expr.copy()
+            target_alias = alias
+
+            if isinstance(inner_expr, exp.Alias):
+                target_alias = inner_expr.alias
+                inner_expr = inner_expr.this
+
+            if target_alias is None and isinstance(inner_expr, exp.Table):
+                alias_expr = inner_expr.args.get("alias")
+                if alias_expr is not None:
+                    target_alias = alias_expr.this
+                    inner_expr.set("alias", None)
+
+            version = exp.Version(this=as_of_type or "TIMESTAMP", kind="AS OF", expression=exp.convert(as_of))
+            inner_expr.set("version", version)
+            from_expr = exp.alias_(inner_expr, target_alias) if target_alias else inner_expr
+
         builder.set_expression(select_expr.from_(from_expr, copy=False))
         return cast("Self", builder)
 
@@ -349,6 +460,10 @@ class ReturningClauseMixin:
 class WhereClauseMixin:
     __slots__ = ()
 
+    def _merge_sql_object_parameters(self, sql_obj: Any) -> None:
+        builder = cast("SQLBuilderProtocol", self)
+        builder._merge_sql_object_parameters(sql_obj)
+
     def get_expression(self) -> exp.Expression | None: ...
     def set_expression(self, expression: exp.Expression) -> None: ...
 
@@ -366,16 +481,6 @@ class WhereClauseMixin:
         placeholder = exp.Placeholder(this=param_name)
         return condition_factory(col_expr, placeholder)
 
-    def _merge_sql_object_parameters(self, sql_obj: Any) -> None:
-        if not has_expression_and_parameters(sql_obj):
-            return
-
-        builder = cast("SQLBuilderProtocol", self)
-        sql_parameters = getattr(sql_obj, "parameters", {})
-        for param_name, param_value in sql_parameters.items():
-            unique_name = builder._generate_unique_parameter_name(param_name)
-            builder.add_parameter(param_value, name=unique_name)
-
     def _get_existing_where_clause(self) -> exp.Where | None:
         builder = cast("SQLBuilderProtocol", self)
         expression = builder.get_expression()
@@ -406,7 +511,7 @@ class WhereClauseMixin:
         self, column_exp: exp.Expression, value: Any, column_name: str = "column"
     ) -> exp.Expression:
         builder = cast("SQLBuilderProtocol", self)
-        if has_query_builder_parameters(value) or isinstance(value, exp.Expression):
+        if has_parameter_builder(value) or isinstance(value, exp.Expression):
             subquery_expr = self._normalize_subquery_expression(value, builder)
             return exp.In(this=column_exp, expressions=[subquery_expr])
         if is_iterable_parameters(value):
@@ -426,7 +531,7 @@ class WhereClauseMixin:
         self, column_exp: exp.Expression, value: Any, column_name: str = "column"
     ) -> exp.Expression:
         builder = cast("SQLBuilderProtocol", self)
-        if has_query_builder_parameters(value) or isinstance(value, exp.Expression):
+        if has_parameter_builder(value) or isinstance(value, exp.Expression):
             subquery_expr = self._normalize_subquery_expression(value, builder)
             return exp.Not(this=exp.In(this=column_exp, expressions=[subquery_expr]))
         if is_iterable_parameters(value):
@@ -486,13 +591,13 @@ class WhereClauseMixin:
 
     def _create_any_condition(self, column_expr: exp.Expression, values: Any, column_name: str) -> exp.Expression:
         builder = cast("SQLBuilderProtocol", self)
-        if has_query_builder_parameters(values):
+        if has_parameter_builder(values):
             subquery_expr = self._normalize_subquery_expression(values, builder)
             return exp.EQ(this=column_expr, expression=exp.Any(this=subquery_expr))
         if isinstance(values, exp.Expression):
             return exp.EQ(this=column_expr, expression=exp.Any(this=values))
         if has_sqlglot_expression(values):
-            raw_expr = getattr(values, "sqlglot_expression", None)
+            raw_expr = values.sqlglot_expression
             if isinstance(raw_expr, exp.Expression):
                 return exp.EQ(this=column_expr, expression=exp.Any(this=raw_expr))
             parsed_expr: exp.Expression | None = exp.maybe_parse(str(values), dialect=builder.dialect)
@@ -500,10 +605,10 @@ class WhereClauseMixin:
                 return exp.EQ(this=column_expr, expression=exp.Any(this=parsed_expr))
         if has_expression_and_sql(values):
             self._merge_sql_object_parameters(values)
-            expression_attr = getattr(values, "expression", None)
+            expression_attr = values.expression
             if isinstance(expression_attr, exp.Expression):
                 return exp.EQ(this=column_expr, expression=exp.Any(this=expression_attr))
-            sql_text = getattr(values, "sql", "")
+            sql_text = values.sql
             parsed_expr = exp.maybe_parse(sql_text, dialect=builder.dialect)
             if parsed_expr is not None:
                 return exp.EQ(this=column_expr, expression=exp.Any(this=parsed_expr))
@@ -530,13 +635,13 @@ class WhereClauseMixin:
 
     def _create_not_any_condition(self, column_expr: exp.Expression, values: Any, column_name: str) -> exp.Expression:
         builder = cast("SQLBuilderProtocol", self)
-        if has_query_builder_parameters(values):
+        if has_parameter_builder(values):
             subquery_expr = self._normalize_subquery_expression(values, builder)
             return exp.NEQ(this=column_expr, expression=exp.Any(this=subquery_expr))
         if isinstance(values, exp.Expression):
             return exp.NEQ(this=column_expr, expression=exp.Any(this=values))
         if has_sqlglot_expression(values):
-            raw_expr = getattr(values, "sqlglot_expression", None)
+            raw_expr = values.sqlglot_expression
             if isinstance(raw_expr, exp.Expression):
                 return exp.NEQ(this=column_expr, expression=exp.Any(this=raw_expr))
             parsed_expr: exp.Expression | None = exp.maybe_parse(str(values), dialect=builder.dialect)
@@ -544,10 +649,10 @@ class WhereClauseMixin:
                 return exp.NEQ(this=column_expr, expression=exp.Any(this=parsed_expr))
         if has_expression_and_sql(values):
             self._merge_sql_object_parameters(values)
-            expression_attr = getattr(values, "expression", None)
+            expression_attr = values.expression
             if isinstance(expression_attr, exp.Expression):
                 return exp.NEQ(this=column_expr, expression=exp.Any(this=expression_attr))
-            sql_text = getattr(values, "sql", "")
+            sql_text = values.sql
             parsed_expr = exp.maybe_parse(sql_text, dialect=builder.dialect)
             if parsed_expr is not None:
                 return exp.NEQ(this=column_expr, expression=exp.Any(this=parsed_expr))
@@ -573,9 +678,9 @@ class WhereClauseMixin:
         return exp.NEQ(this=column_expr, expression=exp.Any(this=tuple_expr))
 
     def _normalize_subquery_expression(self, subquery: Any, builder: "SQLBuilderProtocol") -> exp.Expression:
-        if has_query_builder_parameters(subquery):
+        if has_parameter_builder(subquery):
             subquery_builder = cast("QueryBuilder", subquery)
-            safe_query: SafeQuery = subquery_builder.build()
+            safe_query: BuiltQuery = subquery_builder.build()
             parsed_subquery: exp.Expression | None = exp.maybe_parse(safe_query.sql, dialect=builder.dialect)
             if parsed_subquery is None:
                 msg = f"Could not parse subquery SQL: {safe_query.sql}"
@@ -601,10 +706,10 @@ class WhereClauseMixin:
 
         if has_expression_and_sql(subquery):
             self._merge_sql_object_parameters(subquery)
-            expression_attr = getattr(subquery, "expression", None)
+            expression_attr = subquery.expression
             if isinstance(expression_attr, exp.Expression):
                 return expression_attr
-            sql_text = getattr(subquery, "sql", "")
+            sql_text = subquery.sql
             parsed_from_sql: exp.Expression | None = exp.maybe_parse(sql_text, dialect=builder.dialect)
             if parsed_from_sql is None:
                 msg = f"Could not parse subquery SQL: {sql_text}"
@@ -634,9 +739,7 @@ class WhereClauseMixin:
     def _process_tuple_condition(self, condition: "tuple[Any, ...]") -> exp.Expression:
         if len(condition) == PAIR_LENGTH:
             column, value = condition
-            return self._create_parameterized_condition(
-                column, value, lambda col, placeholder: exp.EQ(this=col, expression=placeholder)
-            )
+            return self._create_parameterized_condition(column, value, _expr_eq)
 
         if len(condition) != TRIPLE_LENGTH:
             msg = f"Condition tuple must have 2 or 3 elements, got {len(condition)}"
@@ -647,21 +750,8 @@ class WhereClauseMixin:
         column_expr = parse_column_expression(column_raw)
         column_name = extract_column_name(column_raw)
 
-        simple_operator_map: dict[str, Callable[[exp.Expression, exp.Placeholder], exp.Expression]] = {
-            "=": lambda col, placeholder: exp.EQ(this=col, expression=placeholder),
-            "==": lambda col, placeholder: exp.EQ(this=col, expression=placeholder),
-            "!=": lambda col, placeholder: exp.NEQ(this=col, expression=placeholder),
-            "<>": lambda col, placeholder: exp.NEQ(this=col, expression=placeholder),
-            ">": lambda col, placeholder: exp.GT(this=col, expression=placeholder),
-            ">=": lambda col, placeholder: exp.GTE(this=col, expression=placeholder),
-            "<": lambda col, placeholder: exp.LT(this=col, expression=placeholder),
-            "<=": lambda col, placeholder: exp.LTE(this=col, expression=placeholder),
-            "LIKE": lambda col, placeholder: exp.Like(this=col, expression=placeholder),
-            "NOT LIKE": lambda col, placeholder: exp.Not(this=exp.Like(this=col, expression=placeholder)),
-        }
-
-        if operator_upper in simple_operator_map:
-            return self._create_parameterized_condition(column_raw, value, simple_operator_map[operator_upper])
+        if operator_upper in _SIMPLE_OPERATOR_MAP:
+            return self._create_parameterized_condition(column_raw, value, _SIMPLE_OPERATOR_MAP[operator_upper])
 
         if operator_upper == "IN":
             return self._handle_in_operator(column_expr, value, column_name)
@@ -731,24 +821,24 @@ class WhereClauseMixin:
             return condition
         if isinstance(condition, tuple):
             return self._process_tuple_condition(condition)
-        if has_query_builder_parameters(condition):
+        if has_parameter_builder(condition):
             column_expr_obj = cast("ColumnExpression", condition)
-            expression_attr = cast("exp.Expression | None", getattr(column_expr_obj, "_expression", None))
+            expression_attr = cast("exp.Expression | None", column_expr_obj._expression)
             if expression_attr is None:
                 msg = "Column expression is missing underlying sqlglot expression."
                 raise SQLBuilderError(msg)
             return expression_attr
         if has_sqlglot_expression(condition):
-            raw_expr = getattr(condition, "sqlglot_expression", None)
+            raw_expr = condition.sqlglot_expression
             if isinstance(raw_expr, exp.Expression):
                 return builder._parameterize_expression(raw_expr)
             return parse_condition_expression(str(condition))
         if has_expression_and_sql(condition):
-            expression_attr = getattr(condition, "expression", None)
+            expression_attr = condition.expression
             if isinstance(expression_attr, exp.Expression):
                 self._merge_sql_object_parameters(condition)
                 return expression_attr
-            sql_text = getattr(condition, "sql", "")
+            sql_text = condition.sql
             self._merge_sql_object_parameters(condition)
             return parse_condition_expression(sql_text)
 
@@ -784,39 +874,27 @@ class WhereClauseMixin:
         raise SQLBuilderError(msg)
 
     def where_eq(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.EQ(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_eq)
         return self.where(condition)
 
     def where_neq(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.NEQ(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_neq)
         return self.where(condition)
 
     def where_lt(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.LT(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_lt)
         return self.where(condition)
 
     def where_lte(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.LTE(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_lte)
         return self.where(condition)
 
     def where_gt(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.GT(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_gt)
         return self.where(condition)
 
     def where_gte(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.GTE(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_gte)
         return self.where(condition)
 
     def where_between(self, column: str | exp.Column, low: Any, high: Any) -> Self:
@@ -845,15 +923,11 @@ class WhereClauseMixin:
         return self.where(condition)
 
     def where_not_like(self, column: str | exp.Column, pattern: str) -> Self:
-        condition = self._create_parameterized_condition(
-            column, pattern, lambda col, placeholder: exp.Not(this=exp.Like(this=col, expression=placeholder))
-        )
+        condition = self._create_parameterized_condition(column, pattern, _expr_not_like)
         return self.where(condition)
 
     def where_ilike(self, column: str | exp.Column, pattern: str) -> Self:
-        condition = self._create_parameterized_condition(
-            column, pattern, lambda col, placeholder: col.ilike(placeholder)
-        )
+        condition = self._create_parameterized_condition(column, pattern, _expr_ilike)
         return self.where(condition)
 
     def where_is_null(self, column: str | exp.Column) -> Self:
@@ -869,7 +943,7 @@ class WhereClauseMixin:
     def where_in(self, column: str | exp.Column, values: Any) -> Self:
         builder = cast("SQLBuilderProtocol", self)
         col_expr = parse_column_expression(column) if not isinstance(column, exp.Column) else column
-        if has_query_builder_parameters(values) or isinstance(values, (exp.Expression, str)):
+        if has_parameter_builder(values) or isinstance(values, (exp.Expression, str)):
             subquery_exp = self._normalize_subquery_expression(values, builder)
             return self.where(exp.In(this=col_expr, expressions=[subquery_exp]))
 
@@ -904,47 +978,32 @@ class WhereClauseMixin:
         return self.where(exp.Not(this=exp.Exists(this=subquery_expr)))
 
     def where_like_any(self, column: str | exp.Column, patterns: list[str]) -> Self:
-        conditions = [
-            self._create_parameterized_condition(column, pattern, lambda col, placeholder: col.like(placeholder))
-            for pattern in patterns
-        ]
+        conditions = [self._create_parameterized_condition(column, pattern, _expr_like_method) for pattern in patterns]
         or_condition = self._create_or_expression(conditions)
         return self.where(or_condition)
 
     def or_where_eq(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.EQ(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_eq)
         return self._combine_with_or(condition)
 
     def or_where_neq(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.NEQ(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_neq)
         return self._combine_with_or(condition)
 
     def or_where_lt(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.LT(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_lt)
         return self._combine_with_or(condition)
 
     def or_where_lte(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.LTE(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_lte)
         return self._combine_with_or(condition)
 
     def or_where_gt(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.GT(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_gt)
         return self._combine_with_or(condition)
 
     def or_where_gte(self, column: str | exp.Column, value: Any) -> Self:
-        condition = self._create_parameterized_condition(
-            column, value, lambda col, placeholder: exp.GTE(this=col, expression=placeholder)
-        )
+        condition = self._create_parameterized_condition(column, value, _expr_gte)
         return self._combine_with_or(condition)
 
     def or_where_between(self, column: str | exp.Column, low: Any, high: Any) -> Self:
@@ -966,15 +1025,11 @@ class WhereClauseMixin:
         return self._combine_with_or(cast("exp.Expression", condition))
 
     def or_where_not_like(self, column: str | exp.Column, pattern: str) -> Self:
-        condition = self._create_parameterized_condition(
-            column, pattern, lambda col, placeholder: col.like(placeholder).not_()
-        )
+        condition = self._create_parameterized_condition(column, pattern, _expr_like_not)
         return self._combine_with_or(condition)
 
     def or_where_ilike(self, column: str | exp.Column, pattern: str) -> Self:
-        condition = self._create_parameterized_condition(
-            column, pattern, lambda col, placeholder: col.ilike(placeholder)
-        )
+        condition = self._create_parameterized_condition(column, pattern, _expr_ilike)
         return self._combine_with_or(condition)
 
     def or_where_is_null(self, column: str | exp.Column) -> Self:
@@ -1178,21 +1233,11 @@ class CommonTableExpressionMixin:
     _with_ctes: Any
     dialect: Any
 
-    def add_parameter(self, value: Any, name: str | None = None) -> tuple[Any, str]:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
-    def _generate_unique_parameter_name(self, base_name: str) -> str:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
-    def _update_placeholders_in_expression(
-        self, expression: exp.Expression, param_mapping: dict[str, str]
-    ) -> exp.Expression:
-        msg = "Method must be provided by QueryBuilder subclass"
-        raise NotImplementedError(msg)
-
     def with_(self, name: str, query: Any | str, recursive: bool = False, columns: list[str] | None = None) -> Self:
+        """Add a CTE via the WITH clause.
+
+        When ``query`` is another builder we reuse its expression, merge parameters with unique names, and let sqlglot handle the actual CTE wrapping to avoid duplicating ``_with_ctes`` state.
+        """
         builder = cast("QueryBuilder", self)
         expression = builder.get_expression()
         if expression is None:
@@ -1209,48 +1254,33 @@ class CommonTableExpressionMixin:
         elif isinstance(query, exp.Expression):
             cte_select = query
         else:
-            built_query = query.to_statement()
-            cte_sql = built_query.sql
-            cte_select = exp.maybe_parse(cte_sql, dialect=self.dialect)
+            cte_select = query.get_expression()
+            if cte_select is None:
+                msg = f"Could not get expression from builder: {query}"
+                raise SQLBuilderError(msg)
 
+            built_query = query.to_statement()
             parameters = built_query.parameters
             if isinstance(parameters, dict):
                 param_mapping: dict[str, str] = {}
                 for param_name, param_value in parameters.items():
-                    unique_name = self._generate_unique_parameter_name(f"{name}_{param_name}")
+                    unique_name = builder._generate_unique_parameter_name(f"{name}_{param_name}")
                     param_mapping[param_name] = unique_name
-                    self.add_parameter(param_value, name=unique_name)
-                if cte_select is not None:
-                    cte_select = self._update_placeholders_in_expression(cte_select, param_mapping)
+                    builder.add_parameter(param_value, name=unique_name)
+                cte_select = builder._update_placeholders_in_expression(cte_select, param_mapping)
             elif isinstance(parameters, (list, tuple)):
                 for param_value in parameters:
-                    self.add_parameter(param_value)
+                    builder.add_parameter(param_value)
             elif parameters is not None:
-                self.add_parameter(parameters)
+                builder.add_parameter(parameters)
 
         if cte_select is None:
             msg = f"Could not parse CTE query: {query}"
             raise SQLBuilderError(msg)
 
-        if columns:
-            cte_alias_expr = exp.alias_(cte_select, name, table=[exp.to_identifier(col) for col in columns])
-        else:
-            cte_alias_expr = exp.alias_(cte_select, name)
-
-        existing_with = expression.args.get("with")
-        if existing_with:
-            existing_with.expressions.append(cte_alias_expr)
-            if recursive:
-                existing_with.set("recursive", recursive)
-        else:
-            if isinstance(expression, (exp.Select, exp.Insert, exp.Update)):
-                updated = expression.with_(cte_alias_expr, as_=name, copy=False)
-                builder.set_expression(updated)
-                if recursive:
-                    with_clause = updated.find(exp.With)
-                    if with_clause:
-                        with_clause.set("recursive", recursive)
-            builder._with_ctes[name] = exp.CTE(this=cte_select, alias=exp.to_table(name))
+        if isinstance(expression, (exp.Select, exp.Insert, exp.Update)):
+            updated = expression.with_(name, as_=cte_select.copy(), recursive=recursive, copy=True)
+            builder.set_expression(updated)
 
         return cast("Self", builder)
 
@@ -1277,11 +1307,11 @@ class SetOperationMixin:
     def _combine_with_other(self, other: Any, *, operator: str, distinct: bool) -> Self:
         builder = cast("QueryBuilder", self)
 
-        if not hasattr(other, "_build_final_expression") or not hasattr(other, "parameters"):
+        if not isinstance(other, QueryBuilder):
             msg = "Set operations require another SQLSpec query builder."
             raise SQLBuilderError(msg)
 
-        other_builder = cast("QueryBuilder", other)
+        other_builder = other
         left_expr = builder._build_final_expression(copy=True)
         right_expr = other_builder._build_final_expression(copy=True)
 
@@ -1323,6 +1353,27 @@ class SetOperationMixin:
 TABLE_HINT_PATTERN: Final[str] = r"\b{}\b(\s+AS\s+\w+)?"
 
 
+def _parse_hint_expression(hint: Any, dialect: "DialectType | str | None") -> exp.Expression:
+    try:
+        hint_str = str(hint)
+        hint_expr: exp.Expression | None = exp.maybe_parse(hint_str, dialect=dialect)
+        return hint_expr or exp.Anonymous(this=hint_str)
+    except Exception:
+        return exp.Anonymous(this=str(hint))
+
+
+class _TableHintReplacer:
+    __slots__ = ("_hint", "_table")
+
+    def __init__(self, hint: str, table: str) -> None:
+        self._hint = hint
+        self._table = table
+
+    def __call__(self, match: "re.Match[str]") -> str:
+        alias_part = match.group(1) or ""
+        return f"/*+ {self._hint} */ {self._table}{alias_part}"
+
+
 class Select(
     QueryBuilder,
     WhereClauseMixin,
@@ -1335,6 +1386,7 @@ class Select(
     CommonTableExpressionMixin,
     PivotClauseMixin,
     UnpivotClauseMixin,
+    ExplainMixin,
 ):
     """Builder for SELECT queries.
 
@@ -1363,7 +1415,17 @@ class Select(
             Select("id", "name")  # Shorthand for Select().select("id", "name")
             Select()              # Same as Select() - start empty
         """
-        super().__init__(**kwargs)
+        (dialect, schema, enable_optimization, optimize_joins, optimize_predicates, simplify_expressions) = (
+            self._parse_query_builder_kwargs(kwargs)
+        )
+        super().__init__(
+            dialect=dialect,
+            schema=schema,
+            enable_optimization=enable_optimization,
+            optimize_joins=optimize_joins,
+            optimize_predicates=optimize_predicates,
+            simplify_expressions=simplify_expressions,
+        )
 
         self._hints: list[dict[str, object]] = []
 
@@ -1404,14 +1466,14 @@ class Select(
         self._hints.append({"hint": hint, "location": location, "table": table, "dialect": dialect})
         return self
 
-    def build(self, dialect: "DialectType" = None) -> "SafeQuery":
+    def build(self, dialect: "DialectType" = None) -> "BuiltQuery":
         """Builds the SQL query string and parameters with hint injection.
 
         Args:
             dialect: Optional dialect override for SQL generation.
 
         Returns:
-            SafeQuery: A dataclass containing the SQL string and parameters.
+            BuiltQuery: A dataclass containing the SQL string and parameters.
         """
         safe_query = super().build(dialect=dialect)
 
@@ -1425,16 +1487,9 @@ class Select(
         if isinstance(modified_expr, exp.Select):
             statement_hints = [h["hint"] for h in self._hints if h.get("location") == "statement"]
             if statement_hints:
-
-                def parse_hint_safely(hint: Any) -> exp.Expression:
-                    try:
-                        hint_str = str(hint)
-                        hint_expr: exp.Expression | None = exp.maybe_parse(hint_str, dialect=target_dialect)
-                        return hint_expr or exp.Anonymous(this=hint_str)
-                    except Exception:
-                        return exp.Anonymous(this=str(hint))
-
-                hint_expressions: list[exp.Expression] = [parse_hint_safely(hint) for hint in statement_hints]
+                hint_expressions: list[exp.Expression] = [
+                    _parse_hint_expression(hint, target_dialect) for hint in statement_hints
+                ]
 
                 if hint_expressions:
                     modified_expr.set("hint", exp.Hint(expressions=hint_expressions))
@@ -1447,18 +1502,11 @@ class Select(
                 hint = str(hint_dict["hint"])
                 pattern = TABLE_HINT_PATTERN.format(re.escape(table))
 
-                def make_replacement(hint_val: str, table_val: str) -> "Callable[[re.Match[str]], str]":
-                    def replacement_func(match: re.Match[str]) -> str:
-                        alias_part = match.group(1) or ""
-                        return f"/*+ {hint_val} */ {table_val}{alias_part}"
-
-                    return replacement_func
-
                 modified_sql = re.sub(
-                    pattern, make_replacement(hint, table), modified_sql, count=1, flags=re.IGNORECASE
+                    pattern, _TableHintReplacer(hint, table), modified_sql, count=1, flags=re.IGNORECASE
                 )
 
-        return SafeQuery(sql=modified_sql, parameters=safe_query.parameters, dialect=safe_query.dialect)
+        return BuiltQuery(sql=modified_sql, parameters=safe_query.parameters, dialect=safe_query.dialect)
 
     def _validate_select_expression(self) -> None:
         """Validate that current expression is a valid SELECT statement.
@@ -1503,7 +1551,7 @@ class Select(
         assert self._expression is not None
         select_expr = cast("exp.Select", self._expression)
 
-        lock_args = {"update": True}
+        lock_args: dict[str, Any] = {"update": True}
 
         if skip_locked:
             lock_args["wait"] = False
@@ -1512,7 +1560,10 @@ class Select(
 
         if of:
             tables = [of] if isinstance(of, str) else of
-            lock_args["expressions"] = [exp.table_(t) for t in tables]  # type: ignore[assignment]
+            lock_args["expressions"] = [exp.to_identifier(str(t), quoted=is_explicitly_quoted(t)) for t in tables]
+            self._lock_targets_quoted = any(is_explicitly_quoted(t) for t in tables)
+        else:
+            self._lock_targets_quoted = False
 
         lock = exp.Lock(**lock_args)
 
@@ -1541,7 +1592,7 @@ class Select(
         assert self._expression is not None
         select_expr = cast("exp.Select", self._expression)
 
-        lock_args = {"update": False}
+        lock_args: dict[str, Any] = {"update": False}
 
         if skip_locked:
             lock_args["wait"] = False
@@ -1550,7 +1601,10 @@ class Select(
 
         if of:
             tables = [of] if isinstance(of, str) else of
-            lock_args["expressions"] = [exp.table_(t) for t in tables]  # type: ignore[assignment]
+            lock_args["expressions"] = [exp.to_identifier(str(t), quoted=is_explicitly_quoted(t)) for t in tables]
+            self._lock_targets_quoted = any(is_explicitly_quoted(t) for t in tables)
+        else:
+            self._lock_targets_quoted = False
 
         lock = exp.Lock(**lock_args)
 
