@@ -13,6 +13,7 @@ Supported dialects include Oracle PL/SQL, T-SQL, PostgreSQL,
 MySQL, SQLite, DuckDB, and BigQuery.
 """
 
+import logging
 import re
 import threading
 from abc import ABC, abstractmethod
@@ -23,7 +24,7 @@ from typing import Any, Final, TypeAlias, cast
 
 from mypy_extensions import mypyc_attr
 
-from sqlspec.core.cache import CacheKey, UnifiedCache
+from sqlspec.core.cache import CacheKey, LRUCache
 from sqlspec.utils.logging import get_logger
 
 __all__ = (
@@ -39,6 +40,9 @@ __all__ = (
 
 logger = get_logger("sqlspec.core.splitter")
 
+_TOKENIZE_DEBUG_SAMPLE_LIMIT: Final[int] = 3
+_TOKENIZE_SNIPPET_LENGTH: Final[int] = 20
+
 DEFAULT_PATTERN_CACHE_SIZE: Final = 1000
 DEFAULT_RESULT_CACHE_SIZE: Final = 5000
 DEFAULT_CACHE_TTL: Final = 3600
@@ -66,6 +70,10 @@ SPLITTER_SLOTS: Final = (
 )
 
 
+def _special_terminator_true(_tokens: "list[Token]", _pos: int) -> bool:
+    return True
+
+
 class TokenType(Enum):
     """Types of tokens recognized by the SQL lexer."""
 
@@ -125,28 +133,28 @@ class DialectConfig(ABC):
 
     @property
     @abstractmethod
-    def block_starters(self) -> set[str]:
+    def block_starters(self) -> "set[str]":
         """Keywords that start a block (e.g., BEGIN, DECLARE)."""
 
     @property
     @abstractmethod
-    def block_enders(self) -> set[str]:
+    def block_enders(self) -> "set[str]":
         """Keywords that end a block (e.g., END)."""
 
     @property
     @abstractmethod
-    def statement_terminators(self) -> set[str]:
+    def statement_terminators(self) -> "set[str]":
         """Characters that terminate statements (e.g., ;)."""
 
     @property
-    def batch_separators(self) -> set[str]:
+    def batch_separators(self) -> "set[str]":
         """Keywords that separate batches (e.g., GO for T-SQL)."""
         if self._batch_separators is None:
             self._batch_separators = set()
         return self._batch_separators
 
     @property
-    def special_terminators(self) -> dict[str, Callable[[list[Token], int], bool]]:
+    def special_terminators(self) -> "dict[str, Callable[[list[Token], int], bool]]":
         """Special terminators that need custom handling."""
         if self._special_terminators is None:
             self._special_terminators = {}
@@ -159,7 +167,7 @@ class DialectConfig(ABC):
             self._max_nesting_depth = 256
         return self._max_nesting_depth
 
-    def get_all_token_patterns(self) -> list[tuple[TokenType, TokenPattern]]:
+    def get_all_token_patterns(self) -> "list[tuple[TokenType, TokenPattern]]":
         """Get the complete ordered list of token patterns for this dialect.
 
         Returns:
@@ -187,7 +195,7 @@ class DialectConfig(ABC):
 
         return patterns
 
-    def _get_dialect_specific_patterns(self) -> list[tuple[TokenType, TokenPattern]]:
+    def _get_dialect_specific_patterns(self) -> "list[tuple[TokenType, TokenPattern]]":
         """Get dialect-specific token patterns.
 
         Returns:
@@ -196,7 +204,7 @@ class DialectConfig(ABC):
         return []
 
     @staticmethod
-    def is_real_block_ender(tokens: list[Token], current_pos: int) -> bool:  # noqa: ARG004
+    def is_real_block_ender(tokens: "list[Token]", current_pos: int) -> bool:  # noqa: ARG004
         """Check if END keyword represents an actual block terminator.
 
         Args:
@@ -208,7 +216,7 @@ class DialectConfig(ABC):
         """
         return True
 
-    def should_delay_semicolon_termination(self, tokens: list[Token], current_pos: int) -> bool:
+    def should_delay_semicolon_termination(self, tokens: "list[Token]", current_pos: int) -> bool:
         """Check if semicolon termination should be delayed.
 
         Args:
@@ -231,30 +239,30 @@ class OracleDialectConfig(DialectConfig):
         return self._name
 
     @property
-    def block_starters(self) -> set[str]:
+    def block_starters(self) -> "set[str]":
         if self._block_starters is None:
             self._block_starters = {"BEGIN", "DECLARE", "CASE"}
         return self._block_starters
 
     @property
-    def block_enders(self) -> set[str]:
+    def block_enders(self) -> "set[str]":
         if self._block_enders is None:
             self._block_enders = {"END"}
         return self._block_enders
 
     @property
-    def statement_terminators(self) -> set[str]:
+    def statement_terminators(self) -> "set[str]":
         if self._statement_terminators is None:
             self._statement_terminators = {";"}
         return self._statement_terminators
 
     @property
-    def special_terminators(self) -> dict[str, Callable[[list[Token], int], bool]]:
+    def special_terminators(self) -> "dict[str, Callable[[list[Token], int], bool]]":
         if self._special_terminators is None:
             self._special_terminators = {"/": self._handle_slash_terminator}
         return self._special_terminators
 
-    def should_delay_semicolon_termination(self, tokens: list[Token], current_pos: int) -> bool:
+    def should_delay_semicolon_termination(self, tokens: "list[Token]", current_pos: int) -> bool:
         """Check if semicolon termination should be delayed for Oracle slash terminators.
 
         Args:
@@ -276,7 +284,7 @@ class OracleDialectConfig(DialectConfig):
 
         return False
 
-    def _has_upcoming_slash(self, tokens: list[Token], current_pos: int) -> bool:
+    def _has_upcoming_slash(self, tokens: "list[Token]", current_pos: int) -> bool:
         """Check if there's a slash terminator on its own line ahead.
 
         Args:
@@ -306,7 +314,7 @@ class OracleDialectConfig(DialectConfig):
         return False
 
     @staticmethod
-    def is_real_block_ender(tokens: list[Token], current_pos: int) -> bool:
+    def is_real_block_ender(tokens: "list[Token]", current_pos: int) -> bool:
         """Check if END keyword represents a block terminator in Oracle PL/SQL.
 
         Args:
@@ -337,7 +345,7 @@ class OracleDialectConfig(DialectConfig):
         return True
 
     @staticmethod
-    def _handle_slash_terminator(tokens: list[Token], current_pos: int) -> bool:
+    def _handle_slash_terminator(tokens: "list[Token]", current_pos: int) -> bool:
         """Check if Oracle slash terminator is properly positioned.
 
         Oracle slash terminators must be on their own line with only
@@ -375,25 +383,25 @@ class TSQLDialectConfig(DialectConfig):
         return self._name
 
     @property
-    def block_starters(self) -> set[str]:
+    def block_starters(self) -> "set[str]":
         if self._block_starters is None:
             self._block_starters = {"BEGIN", "TRY"}
         return self._block_starters
 
     @property
-    def block_enders(self) -> set[str]:
+    def block_enders(self) -> "set[str]":
         if self._block_enders is None:
             self._block_enders = {"END", "CATCH"}
         return self._block_enders
 
     @property
-    def statement_terminators(self) -> set[str]:
+    def statement_terminators(self) -> "set[str]":
         if self._statement_terminators is None:
             self._statement_terminators = {";"}
         return self._statement_terminators
 
     @property
-    def batch_separators(self) -> set[str]:
+    def batch_separators(self) -> "set[str]":
         if self._batch_separators is None:
             self._batch_separators = {"GO"}
         return self._batch_separators
@@ -409,24 +417,24 @@ class PostgreSQLDialectConfig(DialectConfig):
         return self._name
 
     @property
-    def block_starters(self) -> set[str]:
+    def block_starters(self) -> "set[str]":
         if self._block_starters is None:
-            self._block_starters = {"BEGIN", "DECLARE", "CASE", "DO"}
+            self._block_starters = {"DECLARE", "CASE", "DO"}
         return self._block_starters
 
     @property
-    def block_enders(self) -> set[str]:
+    def block_enders(self) -> "set[str]":
         if self._block_enders is None:
             self._block_enders = {"END"}
         return self._block_enders
 
     @property
-    def statement_terminators(self) -> set[str]:
+    def statement_terminators(self) -> "set[str]":
         if self._statement_terminators is None:
             self._statement_terminators = {";"}
         return self._statement_terminators
 
-    def _get_dialect_specific_patterns(self) -> list[tuple[TokenType, TokenPattern]]:
+    def _get_dialect_specific_patterns(self) -> "list[tuple[TokenType, TokenPattern]]":
         """Get PostgreSQL-specific token patterns.
 
         Returns:
@@ -476,19 +484,19 @@ class GenericDialectConfig(DialectConfig):
         return self._name
 
     @property
-    def block_starters(self) -> set[str]:
+    def block_starters(self) -> "set[str]":
         if self._block_starters is None:
             self._block_starters = {"BEGIN", "DECLARE", "CASE"}
         return self._block_starters
 
     @property
-    def block_enders(self) -> set[str]:
+    def block_enders(self) -> "set[str]":
         if self._block_enders is None:
             self._block_enders = {"END"}
         return self._block_enders
 
     @property
-    def statement_terminators(self) -> set[str]:
+    def statement_terminators(self) -> "set[str]":
         if self._statement_terminators is None:
             self._statement_terminators = {";"}
         return self._statement_terminators
@@ -504,27 +512,27 @@ class MySQLDialectConfig(DialectConfig):
         return self._name
 
     @property
-    def block_starters(self) -> set[str]:
+    def block_starters(self) -> "set[str]":
         if self._block_starters is None:
             self._block_starters = {"BEGIN", "DECLARE", "CASE"}
         return self._block_starters
 
     @property
-    def block_enders(self) -> set[str]:
+    def block_enders(self) -> "set[str]":
         if self._block_enders is None:
             self._block_enders = {"END"}
         return self._block_enders
 
     @property
-    def statement_terminators(self) -> set[str]:
+    def statement_terminators(self) -> "set[str]":
         if self._statement_terminators is None:
             self._statement_terminators = {";"}
         return self._statement_terminators
 
     @property
-    def special_terminators(self) -> dict[str, Callable[[list[Token], int], bool]]:
+    def special_terminators(self) -> "dict[str, Callable[[list[Token], int], bool]]":
         if self._special_terminators is None:
-            self._special_terminators = {"\\g": lambda _tokens, _pos: True, "\\G": lambda _tokens, _pos: True}
+            self._special_terminators = {"\\g": _special_terminator_true, "\\G": _special_terminator_true}
         return self._special_terminators
 
 
@@ -538,19 +546,19 @@ class SQLiteDialectConfig(DialectConfig):
         return self._name
 
     @property
-    def block_starters(self) -> set[str]:
+    def block_starters(self) -> "set[str]":
         if self._block_starters is None:
             self._block_starters = {"BEGIN", "CASE"}
         return self._block_starters
 
     @property
-    def block_enders(self) -> set[str]:
+    def block_enders(self) -> "set[str]":
         if self._block_enders is None:
             self._block_enders = {"END"}
         return self._block_enders
 
     @property
-    def statement_terminators(self) -> set[str]:
+    def statement_terminators(self) -> "set[str]":
         if self._statement_terminators is None:
             self._statement_terminators = {";"}
         return self._statement_terminators
@@ -566,19 +574,19 @@ class DuckDBDialectConfig(DialectConfig):
         return self._name
 
     @property
-    def block_starters(self) -> set[str]:
+    def block_starters(self) -> "set[str]":
         if self._block_starters is None:
             self._block_starters = {"BEGIN", "CASE"}
         return self._block_starters
 
     @property
-    def block_enders(self) -> set[str]:
+    def block_enders(self) -> "set[str]":
         if self._block_enders is None:
             self._block_enders = {"END"}
         return self._block_enders
 
     @property
-    def statement_terminators(self) -> set[str]:
+    def statement_terminators(self) -> "set[str]":
         if self._statement_terminators is None:
             self._statement_terminators = {";"}
         return self._statement_terminators
@@ -594,30 +602,30 @@ class BigQueryDialectConfig(DialectConfig):
         return self._name
 
     @property
-    def block_starters(self) -> set[str]:
+    def block_starters(self) -> "set[str]":
         if self._block_starters is None:
             self._block_starters = {"BEGIN", "CASE"}
         return self._block_starters
 
     @property
-    def block_enders(self) -> set[str]:
+    def block_enders(self) -> "set[str]":
         if self._block_enders is None:
             self._block_enders = {"END"}
         return self._block_enders
 
     @property
-    def statement_terminators(self) -> set[str]:
+    def statement_terminators(self) -> "set[str]":
         if self._statement_terminators is None:
             self._statement_terminators = {";"}
         return self._statement_terminators
 
 
-_pattern_cache: UnifiedCache | None = None
-_result_cache: UnifiedCache | None = None
+_pattern_cache: LRUCache | None = None
+_result_cache: LRUCache | None = None
 _cache_lock = threading.Lock()
 
 
-def _get_pattern_cache() -> UnifiedCache:
+def _get_pattern_cache() -> LRUCache:
     """Get or create the global pattern compilation cache.
 
     Returns:
@@ -627,11 +635,11 @@ def _get_pattern_cache() -> UnifiedCache:
     if _pattern_cache is None:
         with _cache_lock:
             if _pattern_cache is None:
-                _pattern_cache = UnifiedCache(max_size=DEFAULT_PATTERN_CACHE_SIZE, ttl_seconds=DEFAULT_CACHE_TTL)
+                _pattern_cache = LRUCache(max_size=DEFAULT_PATTERN_CACHE_SIZE, ttl_seconds=DEFAULT_CACHE_TTL)
     return _pattern_cache
 
 
-def _get_result_cache() -> UnifiedCache:
+def _get_result_cache() -> LRUCache:
     """Get or create the global result cache.
 
     Returns:
@@ -641,7 +649,7 @@ def _get_result_cache() -> UnifiedCache:
     if _result_cache is None:
         with _cache_lock:
             if _result_cache is None:
-                _result_cache = UnifiedCache(max_size=DEFAULT_RESULT_CACHE_SIZE, ttl_seconds=DEFAULT_CACHE_TTL)
+                _result_cache = LRUCache(max_size=DEFAULT_RESULT_CACHE_SIZE, ttl_seconds=DEFAULT_CACHE_TTL)
     return _result_cache
 
 
@@ -669,7 +677,7 @@ class StatementSplitter:
 
         self._compiled_patterns = self._get_or_compile_patterns()
 
-    def _get_or_compile_patterns(self) -> list[tuple[TokenType, CompiledTokenPattern]]:
+    def _get_or_compile_patterns(self) -> "list[tuple[TokenType, CompiledTokenPattern]]":
         """Get compiled regex patterns from cache or compile and cache them.
 
         Returns:
@@ -703,6 +711,9 @@ class StatementSplitter:
         pos = 0
         line = 1
         line_start = 0
+        unmatched_count = 0
+        first_unmatched_pos: int | None = None
+        first_unmatched_snippet: str | None = None
 
         while pos < len(sql):
             matched = False
@@ -740,10 +751,25 @@ class StatementSplitter:
                         break
 
             if not matched:
-                logger.error("Failed to tokenize at position %d: %s", pos, sql[pos : pos + 20])
+                if unmatched_count == 0:
+                    first_unmatched_pos = pos
+                    first_unmatched_snippet = sql[pos : pos + _TOKENIZE_SNIPPET_LENGTH]
+                unmatched_count += 1
+                if unmatched_count <= _TOKENIZE_DEBUG_SAMPLE_LIMIT and logger.isEnabledFor(logging.DEBUG):
+                    logger.debug(
+                        "Failed to tokenize at position %d: %s", pos, sql[pos : pos + _TOKENIZE_SNIPPET_LENGTH]
+                    )
                 pos += 1
 
-    def split(self, sql: str) -> list[str]:
+        if unmatched_count and logger.isEnabledFor(logging.DEBUG):
+            logger.debug(
+                "Tokenization skipped %d unmatched characters (first at %s: %s)",
+                unmatched_count,
+                first_unmatched_pos,
+                first_unmatched_snippet,
+            )
+
+    def split(self, sql: str) -> "list[str]":
         """Split SQL script into individual statements.
 
         Args:
@@ -764,7 +790,7 @@ class StatementSplitter:
         self._result_cache.put(cache_key, statements)
         return statements
 
-    def _do_split(self, sql: str) -> list[str]:
+    def _do_split(self, sql: str) -> "list[str]":
         """Perform the actual SQL script splitting logic.
 
         Args:
@@ -845,7 +871,7 @@ class StatementSplitter:
         return statements
 
     @staticmethod
-    def _is_plsql_block(tokens: list[Token]) -> bool:
+    def _is_plsql_block(tokens: "list[Token]") -> bool:
         """Check if the token list represents a PL/SQL block.
 
         Args:
@@ -877,7 +903,7 @@ class StatementSplitter:
         return False
 
 
-def split_sql_script(script: str, dialect: str | None = None, strip_trailing_terminator: bool = False) -> list[str]:
+def split_sql_script(script: str, dialect: str | None = None, strip_trailing_terminator: bool = False) -> "list[str]":
     """Split SQL script into individual statements.
 
     Args:
@@ -925,7 +951,7 @@ def clear_splitter_caches() -> None:
     result_cache.clear()
 
 
-def get_splitter_cache_stats() -> dict[str, Any]:
+def get_splitter_cache_stats() -> "dict[str, Any]":
     """Get statistics from splitter caches.
 
     Returns:
