@@ -1,28 +1,32 @@
 """Driver protocols and base classes for database adapters."""
 
-from sqlspec.driver import mixins
 from sqlspec.driver._async import AsyncDataDictionaryBase, AsyncDriverAdapterBase
 from sqlspec.driver._common import (
     CommonDriverAttributesMixin,
+    DataDictionaryDialectMixin,
+    DataDictionaryMixin,
     ExecutionResult,
     StackExecutionObserver,
-    VersionInfo,
     describe_stack_statement,
+    hash_stack_operations,
 )
+from sqlspec.driver._sql_helpers import convert_to_dialect
 from sqlspec.driver._sync import SyncDataDictionaryBase, SyncDriverAdapterBase
 
 __all__ = (
     "AsyncDataDictionaryBase",
     "AsyncDriverAdapterBase",
     "CommonDriverAttributesMixin",
+    "DataDictionaryDialectMixin",
+    "DataDictionaryMixin",
     "DriverAdapterProtocol",
     "ExecutionResult",
     "StackExecutionObserver",
     "SyncDataDictionaryBase",
     "SyncDriverAdapterBase",
-    "VersionInfo",
+    "convert_to_dialect",
     "describe_stack_statement",
-    "mixins",
+    "hash_stack_operations",
 )
 
 DriverAdapterProtocol = SyncDriverAdapterBase | AsyncDriverAdapterBase
