@@ -1,19 +1,22 @@
 """Base store classes for ADK session backend (sync and async)."""
 
+import logging
 import re
 from abc import ABC, abstractmethod
 from typing import TYPE_CHECKING, Any, Final, Generic, TypeVar, cast
 
-from sqlspec.utils.logging import get_logger
+from sqlspec.observability import resolve_db_system
+from sqlspec.utils.logging import get_logger, log_with_context
 
 if TYPE_CHECKING:
     from datetime import datetime
 
+    from sqlspec.config import ADKConfig, DatabaseConfigProtocol
     from sqlspec.extensions.adk._types import EventRecord, SessionRecord
 
-ConfigT = TypeVar("ConfigT")
+ConfigT = TypeVar("ConfigT", bound="DatabaseConfigProtocol[Any, Any, Any]")
 
-logger = get_logger("extensions.adk.store")
+logger = get_logger("sqlspec.extensions.adk.store")
 
 __all__ = ("BaseAsyncADKStore", "BaseSyncADKStore")
 
@@ -138,20 +141,18 @@ class BaseAsyncADKStore(ABC, Generic[ConfigT]):
         Returns:
             Dict with session_table, events_table, and optionally owner_id_column.
         """
-        if hasattr(self._config, "extension_config"):
-            extension_config = cast("dict[str, dict[str, Any]]", self._config.extension_config)  # pyright: ignore
-            adk_config: dict[str, Any] = extension_config.get("adk", {})
-            session_table = adk_config.get("session_table")
-            events_table = adk_config.get("events_table")
-            result: dict[str, Any] = {
-                "session_table": session_table if session_table is not None else "adk_sessions",
-                "events_table": events_table if events_table is not None else "adk_events",
-            }
-            owner_id = adk_config.get("owner_id_column")
-            if owner_id is not None:
-                result["owner_id_column"] = owner_id
-            return result
-        return {"session_table": "adk_sessions", "events_table": "adk_events"}
+        extension_config = self._config.extension_config
+        adk_config = cast("ADKConfig", extension_config.get("adk", {}))
+        session_table = adk_config.get("session_table")
+        events_table = adk_config.get("events_table")
+        result: dict[str, Any] = {
+            "session_table": session_table if session_table is not None else "adk_sessions",
+            "events_table": events_table if events_table is not None else "adk_events",
+        }
+        owner_id = adk_config.get("owner_id_column")
+        if owner_id is not None:
+            result["owner_id_column"] = owner_id
+        return result
 
     @property
     def config(self) -> ConfigT:
@@ -270,6 +271,12 @@ class BaseAsyncADKStore(ABC, Generic[ConfigT]):
         """Create the sessions and events tables if they don't exist."""
         raise NotImplementedError
 
+    async def ensure_tables(self) -> None:
+        """Create tables and emit a standardized log entry."""
+
+        await self.create_tables()
+        self._log_tables_created()
+
     @abstractmethod
     async def _get_create_sessions_table_sql(self) -> str:
         """Get the CREATE TABLE SQL for the sessions table.
@@ -302,6 +309,16 @@ class BaseAsyncADKStore(ABC, Generic[ConfigT]):
         """
         raise NotImplementedError
 
+    def _log_tables_created(self) -> None:
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "adk.tables.ready",
+            db_system=resolve_db_system(type(self).__name__),
+            session_table=self._session_table,
+            events_table=self._events_table,
+        )
+
 
 class BaseSyncADKStore(ABC, Generic[ConfigT]):
     """Base class for sync SQLSpec-backed ADK session stores.
@@ -359,20 +376,18 @@ class BaseSyncADKStore(ABC, Generic[ConfigT]):
         Returns:
             Dict with session_table, events_table, and optionally owner_id_column.
         """
-        if hasattr(self._config, "extension_config"):
-            extension_config = cast("dict[str, dict[str, Any]]", self._config.extension_config)  # pyright: ignore
-            adk_config: dict[str, Any] = extension_config.get("adk", {})
-            session_table = adk_config.get("session_table")
-            events_table = adk_config.get("events_table")
-            result: dict[str, Any] = {
-                "session_table": session_table if session_table is not None else "adk_sessions",
-                "events_table": events_table if events_table is not None else "adk_events",
-            }
-            owner_id = adk_config.get("owner_id_column")
-            if owner_id is not None:
-                result["owner_id_column"] = owner_id
-            return result
-        return {"session_table": "adk_sessions", "events_table": "adk_events"}
+        extension_config = self._config.extension_config
+        adk_config = cast("ADKConfig", extension_config.get("adk", {}))
+        session_table = adk_config.get("session_table")
+        events_table = adk_config.get("events_table")
+        result: dict[str, Any] = {
+            "session_table": session_table if session_table is not None else "adk_sessions",
+            "events_table": events_table if events_table is not None else "adk_events",
+        }
+        owner_id = adk_config.get("owner_id_column")
+        if owner_id is not None:
+            result["owner_id_column"] = owner_id
+        return result
 
     @property
     def config(self) -> ConfigT:
@@ -507,6 +522,12 @@ class BaseSyncADKStore(ABC, Generic[ConfigT]):
         """Create both sessions and events tables if they don't exist."""
         raise NotImplementedError
 
+    def ensure_tables(self) -> None:
+        """Create tables and emit a standardized log entry."""
+
+        self.create_tables()
+        self._log_tables_created()
+
     @abstractmethod
     def _get_create_sessions_table_sql(self) -> str:
         """Get SQL to create sessions table.
@@ -525,6 +546,16 @@ class BaseSyncADKStore(ABC, Generic[ConfigT]):
         """
         raise NotImplementedError
 
+    def _log_tables_created(self) -> None:
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "adk.tables.ready",
+            db_system=resolve_db_system(type(self).__name__),
+            session_table=self._session_table,
+            events_table=self._events_table,
+        )
+
     @abstractmethod
     def _get_drop_tables_sql(self) -> "list[str]":
         """Get SQL to drop tables.
