@@ -15,6 +15,16 @@ if TYPE_CHECKING:
     from sqlspec.driver import AsyncDriverAdapterBase, SyncDriverAdapterBase
     from sqlspec.extensions.fastapi.providers import DependencyDefaults, FilterConfig
 
+    # Type aliases for static analysis - IDEs see the real types
+    _AsyncSession = AsyncDriverAdapterBase
+    _SyncSession = SyncDriverAdapterBase
+    _Session = AsyncDriverAdapterBase | SyncDriverAdapterBase
+else:
+    # Runtime fallback - FastAPI sees Any (avoids NameError)
+    _AsyncSession = Any
+    _SyncSession = Any
+    _Session = Any
+
 __all__ = ("SQLSpecPlugin",)
 
 
@@ -32,7 +42,7 @@ class SQLSpecPlugin(_StarlettePlugin):
 
         sqlspec = SQLSpec()
         config = AsyncpgConfig(
-            pool_config={"dsn": "postgresql://localhost/mydb"},
+            connection_config={"dsn": "postgresql://localhost/mydb"},
             extension_config={
                 "starlette": {
                     "commit_mode": "autocommit",
@@ -122,7 +132,7 @@ class SQLSpecPlugin(_StarlettePlugin):
         # Extract string key if provided, ignore config types/instances (used only for type narrowing)
         session_key = key if isinstance(key, str) or key is None else None
 
-        def dependency(request: Request) -> "AsyncDriverAdapterBase | SyncDriverAdapterBase":
+        def dependency(request: Request) -> _Session:
             return self.get_session(request, session_key)  # type: ignore[no-any-return]
 
         return dependency
@@ -151,7 +161,7 @@ class SQLSpecPlugin(_StarlettePlugin):
                 return await db.execute("SELECT * FROM products")
         """
 
-        def dependency(request: Request) -> "AsyncDriverAdapterBase":
+        def dependency(request: Request) -> _AsyncSession:
             return self.get_session(request, key)  # type: ignore[no-any-return]
 
         return dependency
@@ -175,7 +185,7 @@ class SQLSpecPlugin(_StarlettePlugin):
                 return db.execute("SELECT * FROM users")
         """
 
-        def dependency(request: Request) -> "SyncDriverAdapterBase":
+        def dependency(request: Request) -> _SyncSession:
             return self.get_session(request, key)  # type: ignore[no-any-return]
 
         return dependency
