@@ -11,7 +11,7 @@ Example:
 
     sqlspec = SQLSpec()
     config = SqliteConfig(
-        pool_config={"database": "app.db"},
+        connection_config={"database": "app.db"},
         extension_config={
             "flask": {
                 "commit_mode": "autocommit",
@@ -31,6 +31,7 @@ Example:
         return {"users": result.all()}
 """
 
+from sqlspec.extensions.flask._state import FlaskConfigState
 from sqlspec.extensions.flask.extension import SQLSpecPlugin
 
-__all__ = ("SQLSpecPlugin",)
+__all__ = ("FlaskConfigState", "SQLSpecPlugin")
