@@ -1,3 +1,5 @@
+from sqlspec.extensions.litestar._utils import get_sqlspec_scope_state, set_sqlspec_scope_state
+from sqlspec.extensions.litestar.channels import SQLSpecChannelsBackend
 from sqlspec.extensions.litestar.cli import database_group
 from sqlspec.extensions.litestar.config import LitestarConfig
 from sqlspec.extensions.litestar.plugin import (
@@ -18,6 +20,9 @@ __all__ = (
     "BaseSQLSpecStore",
     "CommitMode",
     "LitestarConfig",
+    "SQLSpecChannelsBackend",
     "SQLSpecPlugin",
     "database_group",
+    "get_sqlspec_scope_state",
+    "set_sqlspec_scope_state",
 )
