@@ -6,15 +6,17 @@ from abc import ABC, abstractmethod
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Generic, TypeVar, cast
 
-from sqlspec.builder import Delete, Insert, Select, Update, sql
-from sqlspec.builder._ddl import CreateTable
+from rich.console import Console
+
+from sqlspec.builder import CreateTable, Delete, Insert, Select, Update, sql
 from sqlspec.loader import SQLFileLoader
+from sqlspec.migrations.context import MigrationContext
 from sqlspec.migrations.loaders import get_migration_loader
 from sqlspec.migrations.templates import MigrationTemplateSettings, TemplateDescriptionHints, build_template_settings
+from sqlspec.migrations.version import parse_version
 from sqlspec.utils.logging import get_logger
 from sqlspec.utils.module_loader import module_to_os_path
 from sqlspec.utils.sync_tools import await_
-from sqlspec.utils.version import parse_version
 
 if TYPE_CHECKING:
     from sqlspec.config import DatabaseConfigProtocol
@@ -25,7 +27,7 @@ __all__ = ("BaseMigrationCommands", "BaseMigrationRunner", "BaseMigrationTracker
 DriverT = TypeVar("DriverT")
 ConfigT = TypeVar("ConfigT", bound="DatabaseConfigProtocol[Any, Any, Any]")
 
-logger = get_logger("migrations.base")
+logger = get_logger("sqlspec.migrations.base")
 
 
 class BaseMigrationTracker(ABC, Generic[DriverT]):
@@ -58,7 +60,8 @@ class BaseMigrationTracker(ABC, Generic[DriverT]):
             SQL builder object for table creation.
         """
         return (
-            sql.create_table(self.version_table)
+            sql
+            .create_table(self.version_table)
             .if_not_exists()
             .column("version_num", "VARCHAR(32)", primary_key=True)
             .column("version_type", "VARCHAR(16)")
@@ -125,7 +128,8 @@ class BaseMigrationTracker(ABC, Generic[DriverT]):
             SQL builder object for insert.
         """
         return (
-            sql.insert(self.version_table)
+            sql
+            .insert(self.version_table)
             .columns(
                 "version_num",
                 "version_type",
@@ -165,7 +169,8 @@ class BaseMigrationTracker(ABC, Generic[DriverT]):
             SQL builder object for update.
         """
         return (
-            sql.update(self.version_table)
+            sql
+            .update(self.version_table)
             .set("version_num", new_version)
             .set("version_type", new_version_type)
             .where(sql.version_num == old_version)
@@ -298,8 +303,6 @@ class BaseMigrationRunner(ABC, Generic[DriverT]):
         Returns:
             The extracted version string or None.
         """
-        from pathlib import Path
-
         stem = Path(filename).stem
 
         if stem.startswith("ext_"):
@@ -355,7 +358,7 @@ class BaseMigrationRunner(ABC, Generic[DriverT]):
                             prefixed_version = f"ext_{ext_name}_{version}"
                             migrations.append((prefixed_version, file_path))
 
-        return sorted(migrations, key=lambda m: parse_version(m[0]))
+        return sorted(migrations, key=_migration_sort_key)
 
     def _load_migration_metadata(self, file_path: Path, version: "str | None" = None) -> "dict[str, Any]":
         """Load migration metadata from file.
@@ -375,8 +378,6 @@ class BaseMigrationRunner(ABC, Generic[DriverT]):
         for ext_name, ext_path in self.extension_migrations.items():
             if file_path.parent == ext_path:
                 if ext_name in self.extension_configs and self.context:
-                    from sqlspec.migrations.context import MigrationContext
-
                     context_to_use = MigrationContext(
                         dialect=self.context.dialect,
                         config=self.context.config,
@@ -520,6 +521,10 @@ class BaseMigrationRunner(ABC, Generic[DriverT]):
         ...
 
 
+def _migration_sort_key(item: "tuple[str, Path]") -> Any:
+    return parse_version(item[0])
+
+
 class BaseMigrationCommands(ABC, Generic[ConfigT, DriverT]):
     """Base class for migration commands."""
 
@@ -532,7 +537,7 @@ class BaseMigrationCommands(ABC, Generic[ConfigT, DriverT]):
             config: The SQLSpec configuration.
         """
         self.config = config
-        migration_config = getattr(self.config, "migration_config", {}) or {}
+        migration_config = cast("dict[str, Any]", self.config.migration_config) or {}
 
         self.version_table = migration_config.get("version_table_name", "ddl_migrations")
         self.migrations_path = Path(migration_config.get("script_location", "migrations"))
@@ -561,7 +566,7 @@ class BaseMigrationCommands(ABC, Generic[ConfigT, DriverT]):
                 continue
 
             ext_name = ext_config
-            ext_options = getattr(self.config, "extension_config", {}).get(ext_name, {})
+            ext_options = cast("dict[str, Any]", self.config.extension_config).get(ext_name, {})
             configs[ext_name] = ext_options
 
         return configs
@@ -674,8 +679,6 @@ out-of-order migrations gracefully (e.g., from late-merging branches).
             directory: Directory to initialize migrations in.
             package: Whether to create __init__.py file.
         """
-        from rich.console import Console
-
         console = Console()
 
         migrations_dir = Path(directory)
