@@ -5,16 +5,32 @@ import inspect
 from dataclasses import dataclass, field
 from typing import TYPE_CHECKING, Any
 
+from sqlglot.dialects.dialect import Dialect
+
+from sqlspec.protocols import HasStatementConfigProtocol
 from sqlspec.utils.logging import get_logger
+from sqlspec.utils.type_guards import has_statement_config_factory
 
 if TYPE_CHECKING:
     from sqlspec.driver import AsyncDriverAdapterBase, SyncDriverAdapterBase
 
-logger = get_logger("migrations.context")
+logger = get_logger("sqlspec.migrations.context")
 
 __all__ = ("MigrationContext",)
 
 
+def _normalize_dialect_name(dialect: Any | None) -> "str | None":
+    if dialect is None:
+        return None
+    if isinstance(dialect, str):
+        return dialect
+    if isinstance(dialect, type):
+        return dialect.__name__
+    if isinstance(dialect, Dialect):
+        return dialect.__class__.__name__
+    return None
+
+
 @dataclass
 class MigrationContext:
     """Context object passed to migration functions.
@@ -55,17 +71,17 @@ class MigrationContext:
         Returns:
             Migration context with dialect information.
         """
-        dialect = None
+        dialect: Any | None = None
         try:
-            if hasattr(config, "statement_config") and config.statement_config:
-                dialect = getattr(config.statement_config, "dialect", None)
-            elif hasattr(config, "_create_statement_config") and callable(config._create_statement_config):
-                stmt_config = config._create_statement_config()
-                dialect = getattr(stmt_config, "dialect", None)
+            if isinstance(config, HasStatementConfigProtocol) and config.statement_config:
+                dialect = config.statement_config.dialect
+            elif has_statement_config_factory(config):
+                stmt_config = config._create_statement_config()  # pyright: ignore[reportPrivateUsage]
+                dialect = stmt_config.dialect
         except Exception:
             logger.debug("Unable to extract dialect from config")
 
-        return cls(dialect=dialect, config=config)
+        return cls(dialect=_normalize_dialect_name(dialect), config=config)
 
     @property
     def is_async_execution(self) -> bool:
@@ -90,9 +106,8 @@ class MigrationContext:
         """
         if self.driver is None:
             return False
-
-        execute_method = getattr(self.driver, "execute_script", None)
-        return execute_method is not None and inspect.iscoroutinefunction(execute_method)
+        execute_method = self.driver.execute_script
+        return inspect.iscoroutinefunction(execute_method)
 
     @property
     def execution_mode(self) -> str:
