@@ -3,6 +3,7 @@
 import ast
 import hashlib
 import inspect
+import logging
 import re
 import time
 from abc import ABC, abstractmethod
@@ -10,22 +11,25 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Any, Literal, Union, cast, overload
 
 from sqlspec.core import SQL
+from sqlspec.loader import SQLFileLoader
 from sqlspec.migrations.context import MigrationContext
 from sqlspec.migrations.loaders import get_migration_loader
 from sqlspec.migrations.templates import TemplateDescriptionHints
-from sqlspec.utils.logging import get_logger
+from sqlspec.migrations.version import parse_version
+from sqlspec.observability import resolve_db_system
+from sqlspec.utils.logging import get_logger, log_with_context
 from sqlspec.utils.sync_tools import async_, await_
-from sqlspec.utils.version import parse_version
 
 if TYPE_CHECKING:
     from collections.abc import Awaitable, Callable, Coroutine
 
+    from sqlspec.config import DatabaseConfigProtocol
     from sqlspec.driver import AsyncDriverAdapterBase, SyncDriverAdapterBase
     from sqlspec.observability import ObservabilityRuntime
 
 __all__ = ("AsyncMigrationRunner", "SyncMigrationRunner", "create_migration_runner")
 
-logger = get_logger("migrations.runner")
+logger = get_logger("sqlspec.migrations.runner")
 
 
 class _CachedMigrationMetadata:
@@ -76,8 +80,6 @@ class BaseMigrationRunner(ABC):
         """
         self.migrations_path = migrations_path
         self.extension_migrations = extension_migrations or {}
-        from sqlspec.loader import SQLFileLoader
-
         self.runtime = runtime
         self.loader = SQLFileLoader(runtime=runtime)
         self.project_root: Path | None = None
@@ -336,7 +338,7 @@ class BaseMigrationRunner(ABC):
             metadata=dict(metadata), mtime_ns=stat_result.st_mtime_ns, size=stat_result.st_size
         )
         if cached_metadata:
-            logger.info("Migration metadata cache invalidated: %s", cache_key)
+            logger.debug("Migration metadata cache invalidated: %s", cache_key)
         else:
             logger.debug("Cached migration metadata: %s", cache_key)
         return metadata
@@ -424,7 +426,9 @@ class BaseMigrationRunner(ABC):
 
         return context_to_use
 
-    def should_use_transaction(self, migration: "dict[str, Any]", config: Any) -> bool:
+    def should_use_transaction(
+        self, migration: "dict[str, Any]", config: "DatabaseConfigProtocol[Any, Any, Any]"
+    ) -> bool:
         """Determine if migration should run in a transaction.
 
         Args:
@@ -440,7 +444,7 @@ class BaseMigrationRunner(ABC):
         if migration.get("transactional") is not None:
             return bool(migration["transactional"])
 
-        migration_config = getattr(config, "migration_config", {}) or {}
+        migration_config = cast("dict[str, Any]", config.migration_config) or {}
         return bool(migration_config.get("transactional", True))
 
 
@@ -508,6 +512,14 @@ class SyncMigrationRunner(BaseMigrationRunner):
         upgrade_sql_list = self._get_migration_sql(migration, "up")
         if upgrade_sql_list is None:
             self._metric("migrations.upgrade.skipped")
+            log_with_context(
+                logger,
+                logging.WARNING,
+                "migration.apply",
+                db_system=resolve_db_system(type(driver).__name__),
+                version=migration.get("version"),
+                status="missing",
+            )
             return None, 0
 
         if use_transaction is None:
@@ -520,6 +532,15 @@ class SyncMigrationRunner(BaseMigrationRunner):
             version = cast("str | None", migration.get("version"))
             span = runtime.start_migration_span("upgrade", version=version)
             runtime.increment_metric("migrations.upgrade.invocations")
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.apply",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=migration.get("version"),
+            use_transaction=use_transaction,
+            status="start",
+        )
 
         start_time = time.perf_counter()
         execution_time = 0
@@ -548,12 +569,31 @@ class SyncMigrationRunner(BaseMigrationRunner):
                 duration_ms = int((time.perf_counter() - start_time) * 1000)
                 runtime.increment_metric("migrations.upgrade.errors")
                 runtime.end_migration_span(span, duration_ms=duration_ms, error=exc)
+            log_with_context(
+                logger,
+                logging.ERROR,
+                "migration.apply",
+                db_system=resolve_db_system(type(driver).__name__),
+                version=migration.get("version"),
+                duration_ms=int((time.perf_counter() - start_time) * 1000),
+                error_type=type(exc).__name__,
+                status="failed",
+            )
             raise
 
         if runtime is not None:
             runtime.increment_metric("migrations.upgrade.applied")
             runtime.increment_metric("migrations.upgrade.duration_ms", float(execution_time))
             runtime.end_migration_span(span, duration_ms=execution_time)
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.apply",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=migration.get("version"),
+            duration_ms=execution_time,
+            status="complete",
+        )
 
         return None, execution_time
 
@@ -579,6 +619,14 @@ class SyncMigrationRunner(BaseMigrationRunner):
         downgrade_sql_list = self._get_migration_sql(migration, "down")
         if downgrade_sql_list is None:
             self._metric("migrations.downgrade.skipped")
+            log_with_context(
+                logger,
+                logging.WARNING,
+                "migration.rollback",
+                db_system=resolve_db_system(type(driver).__name__),
+                version=migration.get("version"),
+                status="missing",
+            )
             return None, 0
 
         if use_transaction is None:
@@ -591,6 +639,15 @@ class SyncMigrationRunner(BaseMigrationRunner):
             version = cast("str | None", migration.get("version"))
             span = runtime.start_migration_span("downgrade", version=version)
             runtime.increment_metric("migrations.downgrade.invocations")
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.rollback",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=migration.get("version"),
+            use_transaction=use_transaction,
+            status="start",
+        )
 
         start_time = time.perf_counter()
         execution_time = 0
@@ -619,12 +676,31 @@ class SyncMigrationRunner(BaseMigrationRunner):
                 duration_ms = int((time.perf_counter() - start_time) * 1000)
                 runtime.increment_metric("migrations.downgrade.errors")
                 runtime.end_migration_span(span, duration_ms=duration_ms, error=exc)
+            log_with_context(
+                logger,
+                logging.ERROR,
+                "migration.rollback",
+                db_system=resolve_db_system(type(driver).__name__),
+                version=migration.get("version"),
+                duration_ms=int((time.perf_counter() - start_time) * 1000),
+                error_type=type(exc).__name__,
+                status="failed",
+            )
             raise
 
         if runtime is not None:
             runtime.increment_metric("migrations.downgrade.applied")
             runtime.increment_metric("migrations.downgrade.duration_ms", float(execution_time))
             runtime.end_migration_span(span, duration_ms=execution_time)
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.rollback",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=migration.get("version"),
+            duration_ms=execution_time,
+            status="complete",
+        )
 
         return None, execution_time
 
@@ -768,6 +844,14 @@ class AsyncMigrationRunner(BaseMigrationRunner):
         upgrade_sql_list = await self._get_migration_sql_async(migration, "up")
         if upgrade_sql_list is None:
             self._metric("migrations.upgrade.skipped")
+            log_with_context(
+                logger,
+                logging.WARNING,
+                "migration.apply",
+                db_system=resolve_db_system(type(driver).__name__),
+                version=migration.get("version"),
+                status="missing",
+            )
             return None, 0
 
         if use_transaction is None:
@@ -780,6 +864,15 @@ class AsyncMigrationRunner(BaseMigrationRunner):
             version = cast("str | None", migration.get("version"))
             span = runtime.start_migration_span("upgrade", version=version)
             runtime.increment_metric("migrations.upgrade.invocations")
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.apply",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=migration.get("version"),
+            use_transaction=use_transaction,
+            status="start",
+        )
 
         start_time = time.perf_counter()
         execution_time = 0
@@ -808,12 +901,31 @@ class AsyncMigrationRunner(BaseMigrationRunner):
                 duration_ms = int((time.perf_counter() - start_time) * 1000)
                 runtime.increment_metric("migrations.upgrade.errors")
                 runtime.end_migration_span(span, duration_ms=duration_ms, error=exc)
+            log_with_context(
+                logger,
+                logging.ERROR,
+                "migration.apply",
+                db_system=resolve_db_system(type(driver).__name__),
+                version=migration.get("version"),
+                duration_ms=int((time.perf_counter() - start_time) * 1000),
+                error_type=type(exc).__name__,
+                status="failed",
+            )
             raise
 
         if runtime is not None:
             runtime.increment_metric("migrations.upgrade.applied")
             runtime.increment_metric("migrations.upgrade.duration_ms", float(execution_time))
             runtime.end_migration_span(span, duration_ms=execution_time)
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.apply",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=migration.get("version"),
+            duration_ms=execution_time,
+            status="complete",
+        )
 
         return None, execution_time
 
@@ -839,6 +951,14 @@ class AsyncMigrationRunner(BaseMigrationRunner):
         downgrade_sql_list = await self._get_migration_sql_async(migration, "down")
         if downgrade_sql_list is None:
             self._metric("migrations.downgrade.skipped")
+            log_with_context(
+                logger,
+                logging.WARNING,
+                "migration.rollback",
+                db_system=resolve_db_system(type(driver).__name__),
+                version=migration.get("version"),
+                status="missing",
+            )
             return None, 0
 
         if use_transaction is None:
@@ -851,6 +971,15 @@ class AsyncMigrationRunner(BaseMigrationRunner):
             version = cast("str | None", migration.get("version"))
             span = runtime.start_migration_span("downgrade", version=version)
             runtime.increment_metric("migrations.downgrade.invocations")
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.rollback",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=migration.get("version"),
+            use_transaction=use_transaction,
+            status="start",
+        )
 
         start_time = time.perf_counter()
         execution_time = 0
@@ -879,12 +1008,31 @@ class AsyncMigrationRunner(BaseMigrationRunner):
                 duration_ms = int((time.perf_counter() - start_time) * 1000)
                 runtime.increment_metric("migrations.downgrade.errors")
                 runtime.end_migration_span(span, duration_ms=duration_ms, error=exc)
+            log_with_context(
+                logger,
+                logging.ERROR,
+                "migration.rollback",
+                db_system=resolve_db_system(type(driver).__name__),
+                version=migration.get("version"),
+                duration_ms=int((time.perf_counter() - start_time) * 1000),
+                error_type=type(exc).__name__,
+                status="failed",
+            )
             raise
 
         if runtime is not None:
             runtime.increment_metric("migrations.downgrade.applied")
             runtime.increment_metric("migrations.downgrade.duration_ms", float(execution_time))
             runtime.end_migration_span(span, duration_ms=execution_time)
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.rollback",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=migration.get("version"),
+            duration_ms=execution_time,
+            status="complete",
+        )
 
         return None, execution_time
 
