@@ -3,22 +3,38 @@
 This module provides functionality to track applied migrations in the database.
 """
 
+import logging
 import os
+from collections.abc import Mapping
 from typing import TYPE_CHECKING, Any
 
 from rich.console import Console
 
 from sqlspec.builder import sql
 from sqlspec.migrations.base import BaseMigrationTracker
-from sqlspec.utils.logging import get_logger
-from sqlspec.utils.version import parse_version
+from sqlspec.migrations.version import parse_version
+from sqlspec.observability import resolve_db_system
+from sqlspec.utils.logging import get_logger, log_with_context
 
 if TYPE_CHECKING:
     from sqlspec.driver import AsyncDriverAdapterBase, SyncDriverAdapterBase
 
 __all__ = ("AsyncMigrationTracker", "SyncMigrationTracker")
 
-logger = get_logger("migrations.tracker")
+logger = get_logger("sqlspec.migrations.tracker")
+
+
+def _extract_column_name(metadata: Any) -> "str | None":
+    """Extract column name from a metadata entry."""
+    if isinstance(metadata, Mapping):
+        value = metadata.get("column_name")
+        if value is None:
+            value = metadata.get("COLUMN_NAME")
+        return str(value).lower() if value is not None else None
+    value = getattr(metadata, "column_name", None)
+    if value is not None:
+        return str(value).lower()
+    return None
 
 
 class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
@@ -36,14 +52,30 @@ class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
         try:
             columns_data = driver.data_dictionary.get_columns(driver, self.version_table)
             if not columns_data:
-                logger.debug("Migration tracking table does not exist yet")
-                return
+                log_with_context(
+                    logger,
+                    logging.DEBUG,
+                    "migration.track",
+                    db_system=resolve_db_system(type(driver).__name__),
+                    table=self.version_table,
+                    operation="table_check",
+                    status="missing",
+                )
+                columns_data = []
 
-            existing_columns = {col["column_name"] for col in columns_data}
+            existing_columns = {name for col in columns_data if (name := _extract_column_name(col)) is not None}
             missing_columns = self._detect_missing_columns(existing_columns)
 
             if not missing_columns:
-                logger.debug("Migration tracking table schema is up-to-date")
+                log_with_context(
+                    logger,
+                    logging.DEBUG,
+                    "migration.track",
+                    db_system=resolve_db_system(type(driver).__name__),
+                    table=self.version_table,
+                    operation="schema_check",
+                    status="current",
+                )
                 return
 
             console = Console()
@@ -57,8 +89,17 @@ class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
             driver.commit()
             console.print("[green]Migration tracking table schema updated successfully[/]")
 
-        except Exception as e:
-            logger.warning("Could not check or migrate tracking table schema: %s", e)
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.ERROR,
+                "migration.track",
+                db_system=resolve_db_system(type(driver).__name__),
+                table=self.version_table,
+                operation="schema_check",
+                status="failed",
+                error_type=type(exc).__name__,
+            )
 
     def _add_column(self, driver: "SyncDriverAdapterBase", column_name: str) -> None:
         """Add a single column to the tracking table.
@@ -77,7 +118,16 @@ class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
             name=column_def.name, dtype=column_def.dtype, default=column_def.default, not_null=column_def.not_null
         )
         driver.execute(alter_sql)
-        logger.debug("Added column %s to tracking table", column_name)
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.track",
+            db_system=resolve_db_system(type(driver).__name__),
+            table=self.version_table,
+            column_name=column_name,
+            operation="schema_update",
+            status="column_added",
+        )
 
     def ensure_tracking_table(self, driver: "SyncDriverAdapterBase") -> None:
         """Create the migration tracking table if it doesn't exist.
@@ -102,7 +152,16 @@ class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
             The current version number or None if no migrations applied.
         """
         result = driver.execute(self._get_current_version_sql())
-        return result.data[0]["version_num"] if result.data else None
+        current = result.data[0]["version_num"] if result.data else None
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.history",
+            db_system=resolve_db_system(type(driver).__name__),
+            current_version=current,
+            status="current",
+        )
+        return current
 
     def get_applied_migrations(self, driver: "SyncDriverAdapterBase") -> "list[dict[str, Any]]":
         """Get all applied migrations in order.
@@ -114,7 +173,16 @@ class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
             List of migration records.
         """
         result = driver.execute(self._get_applied_migrations_sql())
-        return result.data or []
+        applied = result.data or []
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.history",
+            db_system=resolve_db_system(type(driver).__name__),
+            applied_count=len(applied),
+            status="listed",
+        )
+        return applied
 
     def record_migration(
         self, driver: "SyncDriverAdapterBase", version: str, description: str, execution_time_ms: int, checksum: str
@@ -149,6 +217,15 @@ class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
             )
         )
         self._safe_commit(driver)
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.track",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=version,
+            operation="record",
+            status="recorded",
+        )
 
     def remove_migration(self, driver: "SyncDriverAdapterBase", version: str) -> None:
         """Remove a migration record (used during downgrade).
@@ -159,6 +236,15 @@ class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
         """
         driver.execute(self._get_remove_migration_sql(version))
         self._safe_commit(driver)
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.track",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=version,
+            operation="remove",
+            status="removed",
+        )
 
     def update_version_record(self, driver: "SyncDriverAdapterBase", old_version: str, new_version: str) -> None:
         """Update migration version record from timestamp to sequential.
@@ -187,14 +273,32 @@ class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
             applied_versions = {row["version_num"] for row in check_result.data} if check_result.data else set()
 
             if new_version in applied_versions:
-                logger.debug("Version already updated: %s -> %s", old_version, new_version)
+                log_with_context(
+                    logger,
+                    logging.DEBUG,
+                    "migration.track",
+                    db_system=resolve_db_system(type(driver).__name__),
+                    old_version=old_version,
+                    new_version=new_version,
+                    operation="version_update",
+                    status="skipped",
+                )
                 return
 
             msg = f"Migration version {old_version} not found in database"
             raise ValueError(msg)
 
         self._safe_commit(driver)
-        logger.debug("Updated version record: %s -> %s", old_version, new_version)
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.track",
+            db_system=resolve_db_system(type(driver).__name__),
+            old_version=old_version,
+            new_version=new_version,
+            operation="version_update",
+            status="updated",
+        )
 
     def _safe_commit(self, driver: "SyncDriverAdapterBase") -> None:
         """Safely commit a transaction only if autocommit is disabled.
@@ -207,8 +311,17 @@ class SyncMigrationTracker(BaseMigrationTracker["SyncDriverAdapterBase"]):
 
         try:
             driver.commit()
-        except Exception:
-            logger.debug("Failed to commit transaction, likely due to autocommit being enabled")
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "migration.track",
+                db_system=resolve_db_system(type(driver).__name__),
+                operation="commit",
+                status="skipped",
+                reason="autocommit",
+                error_type=type(exc).__name__,
+            )
 
 
 class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
@@ -226,18 +339,32 @@ class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
         try:
             columns_data = await driver.data_dictionary.get_columns(driver, self.version_table)
             if not columns_data:
-                logger.debug("Migration tracking table does not exist yet")
-                return
+                log_with_context(
+                    logger,
+                    logging.DEBUG,
+                    "migration.track",
+                    db_system=resolve_db_system(type(driver).__name__),
+                    table=self.version_table,
+                    operation="table_check",
+                    status="missing",
+                )
+                columns_data = []
 
-            existing_columns = {col["column_name"] for col in columns_data}
+            existing_columns = {name for col in columns_data if (name := _extract_column_name(col)) is not None}
             missing_columns = self._detect_missing_columns(existing_columns)
 
             if not missing_columns:
-                logger.debug("Migration tracking table schema is up-to-date")
+                log_with_context(
+                    logger,
+                    logging.DEBUG,
+                    "migration.track",
+                    db_system=resolve_db_system(type(driver).__name__),
+                    table=self.version_table,
+                    operation="schema_check",
+                    status="current",
+                )
                 return
 
-            from rich.console import Console
-
             console = Console()
             console.print(
                 f"[cyan]Migrating tracking table schema, adding columns: {', '.join(sorted(missing_columns))}[/]"
@@ -249,8 +376,17 @@ class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
             await driver.commit()
             console.print("[green]Migration tracking table schema updated successfully[/]")
 
-        except Exception as e:
-            logger.warning("Could not check or migrate tracking table schema: %s", e)
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.ERROR,
+                "migration.track",
+                db_system=resolve_db_system(type(driver).__name__),
+                table=self.version_table,
+                operation="schema_check",
+                status="failed",
+                error_type=type(exc).__name__,
+            )
 
     async def _add_column(self, driver: "AsyncDriverAdapterBase", column_name: str) -> None:
         """Add a single column to the tracking table.
@@ -269,7 +405,16 @@ class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
             name=column_def.name, dtype=column_def.dtype, default=column_def.default, not_null=column_def.not_null
         )
         await driver.execute(alter_sql)
-        logger.debug("Added column %s to tracking table", column_name)
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.track",
+            db_system=resolve_db_system(type(driver).__name__),
+            table=self.version_table,
+            column_name=column_name,
+            operation="schema_update",
+            status="column_added",
+        )
 
     async def ensure_tracking_table(self, driver: "AsyncDriverAdapterBase") -> None:
         """Create the migration tracking table if it doesn't exist.
@@ -294,7 +439,16 @@ class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
             The current version number or None if no migrations applied.
         """
         result = await driver.execute(self._get_current_version_sql())
-        return result.data[0]["version_num"] if result.data else None
+        current = result.data[0]["version_num"] if result.data else None
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.history",
+            db_system=resolve_db_system(type(driver).__name__),
+            current_version=current,
+            status="current",
+        )
+        return current
 
     async def get_applied_migrations(self, driver: "AsyncDriverAdapterBase") -> "list[dict[str, Any]]":
         """Get all applied migrations in order.
@@ -306,7 +460,16 @@ class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
             List of migration records.
         """
         result = await driver.execute(self._get_applied_migrations_sql())
-        return result.data or []
+        applied = result.data or []
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.history",
+            db_system=resolve_db_system(type(driver).__name__),
+            applied_count=len(applied),
+            status="listed",
+        )
+        return applied
 
     async def record_migration(
         self, driver: "AsyncDriverAdapterBase", version: str, description: str, execution_time_ms: int, checksum: str
@@ -341,6 +504,15 @@ class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
             )
         )
         await self._safe_commit_async(driver)
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.track",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=version,
+            operation="record",
+            status="recorded",
+        )
 
     async def remove_migration(self, driver: "AsyncDriverAdapterBase", version: str) -> None:
         """Remove a migration record (used during downgrade).
@@ -351,6 +523,15 @@ class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
         """
         await driver.execute(self._get_remove_migration_sql(version))
         await self._safe_commit_async(driver)
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.track",
+            db_system=resolve_db_system(type(driver).__name__),
+            version=version,
+            operation="remove",
+            status="removed",
+        )
 
     async def update_version_record(self, driver: "AsyncDriverAdapterBase", old_version: str, new_version: str) -> None:
         """Update migration version record from timestamp to sequential.
@@ -379,14 +560,32 @@ class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
             applied_versions = {row["version_num"] for row in check_result.data} if check_result.data else set()
 
             if new_version in applied_versions:
-                logger.debug("Version already updated: %s -> %s", old_version, new_version)
+                log_with_context(
+                    logger,
+                    logging.DEBUG,
+                    "migration.track",
+                    db_system=resolve_db_system(type(driver).__name__),
+                    old_version=old_version,
+                    new_version=new_version,
+                    operation="version_update",
+                    status="skipped",
+                )
                 return
 
             msg = f"Migration version {old_version} not found in database"
             raise ValueError(msg)
 
         await self._safe_commit_async(driver)
-        logger.debug("Updated version record: %s -> %s", old_version, new_version)
+        log_with_context(
+            logger,
+            logging.INFO,
+            "migration.track",
+            db_system=resolve_db_system(type(driver).__name__),
+            old_version=old_version,
+            new_version=new_version,
+            operation="version_update",
+            status="updated",
+        )
 
     async def _safe_commit_async(self, driver: "AsyncDriverAdapterBase") -> None:
         """Safely commit a transaction only if autocommit is disabled.
@@ -399,5 +598,14 @@ class AsyncMigrationTracker(BaseMigrationTracker["AsyncDriverAdapterBase"]):
 
         try:
             await driver.commit()
-        except Exception:
-            logger.debug("Failed to commit transaction, likely due to autocommit being enabled")
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "migration.track",
+                db_system=resolve_db_system(type(driver).__name__),
+                operation="commit",
+                status="skipped",
+                reason="autocommit",
+                error_type=type(exc).__name__,
+            )
