@@ -16,8 +16,11 @@ from sqlspec.storage.errors import execute_async_storage_operation, execute_sync
 from sqlspec.storage.registry import StorageRegistry, storage_registry
 from sqlspec.utils.serializers import from_json, get_serializer_metrics, serialize_collection, to_json
 from sqlspec.utils.sync_tools import async_
+from sqlspec.utils.type_guards import supports_async_delete, supports_async_read_bytes, supports_async_write_bytes
 
 if TYPE_CHECKING:
+    from collections.abc import AsyncIterator, Iterator
+
     from sqlspec.protocols import ObjectStoreProtocol
     from sqlspec.typing import ArrowTable
 
@@ -100,7 +103,7 @@ class StorageTelemetry(TypedDict, total=False):
     partitions_created: int
     duration_s: float
     format: str
-    extra: "dict[str, Any]"
+    extra: "dict[str, object]"
     backend: str
     correlation_id: str
     config: str
@@ -219,6 +222,22 @@ def _encode_arrow_payload(table: "ArrowTable", format_choice: StorageFormat, *,
     return result_bytes
 
 
+def _delete_backend_sync(backend: "ObjectStoreProtocol", path: str, *, backend_name: str) -> None:
+    execute_sync_storage_operation(partial(backend.delete, path), backend=backend_name, operation="delete", path=path)
+
+
+def _write_backend_sync(backend: "ObjectStoreProtocol", path: str, payload: bytes, *, backend_name: str) -> None:
+    execute_sync_storage_operation(
+        partial(backend.write_bytes, path, payload), backend=backend_name, operation="write_bytes", path=path
+    )
+
+
+def _read_backend_sync(backend: "ObjectStoreProtocol", path: str, *, backend_name: str) -> bytes:
+    return execute_sync_storage_operation(
+        partial(backend.read_bytes, path), backend=backend_name, operation="read_bytes", path=path
+    )
+
+
 def _decode_arrow_payload(payload: bytes, format_choice: StorageFormat) -> "ArrowTable":
     pa = import_pyarrow()
     if format_choice == "parquet":
@@ -239,6 +258,47 @@ def _decode_arrow_payload(payload: bytes, format_choice: StorageFormat) -> "Arro
     raise ValueError(msg)
 
 
+def _resolve_alias_destination(
+    registry: StorageRegistry, destination: str, backend_options: "dict[str, Any]"
+) -> "tuple[ObjectStoreProtocol, str] | None":
+    if not destination.startswith("alias://"):
+        return None
+    payload = destination.removeprefix("alias://")
+    alias_name, _, relative_path = payload.partition("/")
+    alias = alias_name.strip()
+    if not alias:
+        msg = "Alias destinations must include a registry alias before the path component"
+        raise ImproperConfigurationError(msg)
+    path_segment = relative_path.strip()
+    if not path_segment:
+        msg = "Alias destinations must include an object path after the alias name"
+        raise ImproperConfigurationError(msg)
+    backend = registry.get(alias, **backend_options)
+    return backend, path_segment.lstrip("/")
+
+
+def _normalize_path_for_backend(destination: str) -> str:
+    if destination.startswith("file://"):
+        return destination.removeprefix("file://")
+    if "://" in destination:
+        _, remainder = destination.split("://", 1)
+        return remainder.lstrip("/")
+    return destination
+
+
+def _resolve_storage_backend(
+    registry: StorageRegistry, destination: StorageDestination, backend_options: "dict[str, Any] | None"
+) -> "tuple[ObjectStoreProtocol, str]":
+    destination_str = destination.as_posix() if isinstance(destination, Path) else str(destination)
+    options = backend_options or {}
+    alias_resolution = _resolve_alias_destination(registry, destination_str, options)
+    if alias_resolution is not None:
+        return alias_resolution
+    backend = registry.get(destination_str, **options)
+    normalized_path = _normalize_path_for_backend(destination_str)
+    return backend, normalized_path
+
+
 @mypyc_attr(allow_interpreted_subclasses=True)
 class SyncStoragePipeline:
     """Pipeline coordinating storage registry operations and telemetry."""
@@ -248,6 +308,12 @@ class SyncStoragePipeline:
     def __init__(self, *, registry: StorageRegistry | None = None) -> None:
         self.registry = registry or storage_registry
 
+    def _resolve_backend(
+        self, destination: StorageDestination, backend_options: "dict[str, Any] | None"
+    ) -> "tuple[ObjectStoreProtocol, str]":
+        """Resolve storage backend and normalized path for a destination."""
+        return _resolve_storage_backend(self.registry, destination, backend_options)
+
     def write_rows(
         self,
         rows: "list[dict[str, Any]]",
@@ -285,7 +351,7 @@ class SyncStoragePipeline:
         return self._write_bytes(
             payload,
             destination,
-            rows=int(getattr(table, "num_rows", 0)),
+            rows=int(table.num_rows),
             format_label=format_choice,
             storage_options=storage_options or {},
         )
@@ -295,21 +361,31 @@ class SyncStoragePipeline:
     ) -> "tuple[ArrowTable, StorageTelemetry]":
         """Read an artifact from storage and decode it into an Arrow table."""
 
-        backend, path = self._resolve_backend(source, **(storage_options or {}))
-        backend_name = getattr(backend, "backend_type", "storage")
-        payload = execute_sync_storage_operation(
-            partial(backend.read_bytes, path), backend=backend_name, operation="read_bytes", path=path
-        )
+        backend, path = self._resolve_backend(source, storage_options)
+        backend_name = backend.backend_type
+        payload = _read_backend_sync(backend, path, backend_name=backend_name)
         table = _decode_arrow_payload(payload, file_format)
+        rows_processed = int(table.num_rows)
         telemetry: StorageTelemetry = {
             "destination": path,
             "bytes_processed": len(payload),
-            "rows_processed": int(getattr(table, "num_rows", 0)),
+            "rows_processed": rows_processed,
             "format": file_format,
             "backend": backend_name,
         }
         return table, telemetry
 
+    def stream_read(
+        self,
+        source: StorageDestination,
+        *,
+        chunk_size: int | None = None,
+        storage_options: "dict[str, Any] | None" = None,
+    ) -> "Iterator[bytes]":
+        """Stream bytes from an artifact."""
+        backend, path = self._resolve_backend(source, storage_options)
+        return backend.stream_read(path, chunk_size=chunk_size)
+
     def allocate_staging_artifacts(self, requests: "list[StorageLoadRequest]") -> "list[StagedArtifact]":
         """Allocate staging metadata for upcoming loads."""
 
@@ -335,14 +411,9 @@ class SyncStoragePipeline:
         """Delete staged artifacts best-effort."""
 
         for artifact in artifacts:
-            backend, path = self._resolve_backend(artifact["uri"])
+            backend, path = self._resolve_backend(artifact["uri"], None)
             try:
-                execute_sync_storage_operation(
-                    partial(backend.delete, path),
-                    backend=getattr(backend, "backend_type", "storage"),
-                    operation="delete",
-                    path=path,
-                )
+                _delete_backend_sync(backend, path, backend_name=backend.backend_type)
             except Exception:
                 if not ignore_errors:
                     raise
@@ -356,12 +427,10 @@ class SyncStoragePipeline:
         format_label: str,
         storage_options: "dict[str, Any]",
     ) -> StorageTelemetry:
-        backend, path = self._resolve_backend(destination, **storage_options)
-        backend_name = getattr(backend, "backend_type", "storage")
+        backend, path = self._resolve_backend(destination, storage_options)
+        backend_name = backend.backend_type
         start = perf_counter()
-        execute_sync_storage_operation(
-            partial(backend.write_bytes, path, payload), backend=backend_name, operation="write_bytes", path=path
-        )
+        _write_backend_sync(backend, path, payload, backend_name=backend_name)
         elapsed = perf_counter() - start
         bytes_written = len(payload)
         _METRICS.record_bytes(bytes_written)
@@ -375,43 +444,6 @@ class SyncStoragePipeline:
         }
         return telemetry
 
-    def _resolve_backend(
-        self, destination: StorageDestination, **backend_options: Any
-    ) -> "tuple[ObjectStoreProtocol, str]":
-        destination_str = destination.as_posix() if isinstance(destination, Path) else str(destination)
-        alias_resolution = self._resolve_alias_destination(destination_str, backend_options)
-        if alias_resolution is not None:
-            return alias_resolution
-        backend = self.registry.get(destination_str, **backend_options)
-        normalized_path = self._normalize_path_for_backend(destination_str)
-        return backend, normalized_path
-
-    def _resolve_alias_destination(
-        self, destination: str, backend_options: "dict[str, Any]"
-    ) -> "tuple[ObjectStoreProtocol, str] | None":
-        if not destination.startswith("alias://"):
-            return None
-        payload = destination.removeprefix("alias://")
-        alias_name, _, relative_path = payload.partition("/")
-        alias = alias_name.strip()
-        if not alias:
-            msg = "Alias destinations must include a registry alias before the path component"
-            raise ImproperConfigurationError(msg)
-        path_segment = relative_path.strip()
-        if not path_segment:
-            msg = "Alias destinations must include an object path after the alias name"
-            raise ImproperConfigurationError(msg)
-        backend = self.registry.get(alias, **backend_options)
-        return backend, path_segment.lstrip("/")
-
-    def _normalize_path_for_backend(self, destination: str) -> str:
-        if destination.startswith("file://"):
-            return destination.removeprefix("file://")
-        if "://" in destination:
-            _, remainder = destination.split("://", 1)
-            return remainder.lstrip("/")
-        return destination
-
 
 @mypyc_attr(allow_interpreted_subclasses=True)
 class AsyncStoragePipeline:
@@ -455,35 +487,27 @@ class AsyncStoragePipeline:
         return await self._write_bytes_async(
             payload,
             destination,
-            rows=int(getattr(table, "num_rows", 0)),
+            rows=int(table.num_rows),
             format_label=format_choice,
             storage_options=storage_options or {},
         )
 
     async def cleanup_staging_artifacts(self, artifacts: "list[StagedArtifact]", *, ignore_errors: bool = True) -> None:
         for artifact in artifacts:
-            backend, path = self._resolve_backend(artifact["uri"])
-            backend_name = getattr(backend, "backend_type", "storage")
-            delete_async = getattr(backend, "delete_async", None)
-            if delete_async is not None:
+            backend, path = _resolve_storage_backend(self.registry, artifact["uri"], None)
+            backend_name = backend.backend_type
+            if supports_async_delete(backend):
                 try:
                     await execute_async_storage_operation(
-                        partial(delete_async, path), backend=backend_name, operation="delete", path=path
+                        partial(backend.delete_async, path), backend=backend_name, operation="delete", path=path
                     )
                 except Exception:
                     if not ignore_errors:
                         raise
                 continue
 
-            def _delete_sync(
-                backend: "ObjectStoreProtocol" = backend, path: str = path, backend_name: str = backend_name
-            ) -> None:
-                execute_sync_storage_operation(
-                    partial(backend.delete, path), backend=backend_name, operation="delete", path=path
-                )
-
             try:
-                await async_(_delete_sync)()
+                await async_(_delete_backend_sync)(backend=backend, path=path, backend_name=backend_name)
             except Exception:
                 if not ignore_errors:
                     raise
@@ -497,30 +521,18 @@ class AsyncStoragePipeline:
         format_label: str,
         storage_options: "dict[str, Any]",
     ) -> StorageTelemetry:
-        backend, path = self._resolve_backend(destination, **storage_options)
-        backend_name = getattr(backend, "backend_type", "storage")
-        writer = getattr(backend, "write_bytes_async", None)
+        backend, path = _resolve_storage_backend(self.registry, destination, storage_options)
+        backend_name = backend.backend_type
         start = perf_counter()
-        if writer is not None:
+        if supports_async_write_bytes(backend):
             await execute_async_storage_operation(
-                partial(writer, path, payload), backend=backend_name, operation="write_bytes", path=path
+                partial(backend.write_bytes_async, path, payload),
+                backend=backend_name,
+                operation="write_bytes",
+                path=path,
             )
         else:
-
-            def _write_sync(
-                backend: "ObjectStoreProtocol" = backend,
-                path: str = path,
-                payload: bytes = payload,
-                backend_name: str = backend_name,
-            ) -> None:
-                execute_sync_storage_operation(
-                    partial(backend.write_bytes, path, payload),
-                    backend=backend_name,
-                    operation="write_bytes",
-                    path=path,
-                )
-
-            await async_(_write_sync)()
+            await async_(_write_backend_sync)(backend=backend, path=path, payload=payload, backend_name=backend_name)
 
         elapsed = perf_counter() - start
         bytes_written = len(payload)
@@ -538,67 +550,33 @@ class AsyncStoragePipeline:
     async def read_arrow_async(
         self, source: StorageDestination, *, file_format: StorageFormat, storage_options: "dict[str, Any] | None" = None
     ) -> "tuple[ArrowTable, StorageTelemetry]":
-        backend, path = self._resolve_backend(source, **(storage_options or {}))
-        backend_name = getattr(backend, "backend_type", "storage")
-        reader = getattr(backend, "read_bytes_async", None)
-        if reader is not None:
+        backend, path = _resolve_storage_backend(self.registry, source, storage_options)
+        backend_name = backend.backend_type
+        if supports_async_read_bytes(backend):
             payload = await execute_async_storage_operation(
-                partial(reader, path), backend=backend_name, operation="read_bytes", path=path
+                partial(backend.read_bytes_async, path), backend=backend_name, operation="read_bytes", path=path
             )
         else:
-
-            def _read_sync(
-                backend: "ObjectStoreProtocol" = backend, path: str = path, backend_name: str = backend_name
-            ) -> bytes:
-                return execute_sync_storage_operation(
-                    partial(backend.read_bytes, path), backend=backend_name, operation="read_bytes", path=path
-                )
-
-            payload = await async_(_read_sync)()
+            payload = await async_(_read_backend_sync)(backend=backend, path=path, backend_name=backend_name)
 
         table = _decode_arrow_payload(payload, file_format)
+        rows_processed = int(table.num_rows)
         telemetry: StorageTelemetry = {
             "destination": path,
             "bytes_processed": len(payload),
-            "rows_processed": int(getattr(table, "num_rows", 0)),
+            "rows_processed": rows_processed,
             "format": file_format,
             "backend": backend_name,
         }
         return table, telemetry
 
-    def _resolve_backend(
-        self, destination: StorageDestination, **backend_options: Any
-    ) -> "tuple[ObjectStoreProtocol, str]":
-        destination_str = destination.as_posix() if isinstance(destination, Path) else str(destination)
-        alias_resolution = self._resolve_alias_destination(destination_str, backend_options)
-        if alias_resolution is not None:
-            return alias_resolution
-        backend = self.registry.get(destination_str, **backend_options)
-        normalized_path = self._normalize_path_for_backend(destination_str)
-        return backend, normalized_path
-
-    def _resolve_alias_destination(
-        self, destination: str, backend_options: "dict[str, Any]"
-    ) -> "tuple[ObjectStoreProtocol, str] | None":
-        if not destination.startswith("alias://"):
-            return None
-        payload = destination.removeprefix("alias://")
-        alias_name, _, relative_path = payload.partition("/")
-        alias = alias_name.strip()
-        if not alias:
-            msg = "Alias destinations must include a registry alias before the path component"
-            raise ImproperConfigurationError(msg)
-        path_segment = relative_path.strip()
-        if not path_segment:
-            msg = "Alias destinations must include an object path after the alias name"
-            raise ImproperConfigurationError(msg)
-        backend = self.registry.get(alias, **backend_options)
-        return backend, path_segment.lstrip("/")
-
-    def _normalize_path_for_backend(self, destination: str) -> str:
-        if destination.startswith("file://"):
-            return destination.removeprefix("file://")
-        if "://" in destination:
-            _, remainder = destination.split("://", 1)
-            return remainder.lstrip("/")
-        return destination
+    async def stream_read_async(
+        self,
+        source: StorageDestination,
+        *,
+        chunk_size: int | None = None,
+        storage_options: "dict[str, Any] | None" = None,
+    ) -> "AsyncIterator[bytes]":
+        """Stream bytes from an artifact asynchronously."""
+        backend, path = _resolve_storage_backend(self.registry, source, storage_options)
+        return await backend.stream_read_async(path, chunk_size=chunk_size)
