@@ -5,22 +5,22 @@ from sync frameworks like Flask. Based on the portal pattern from Advanced Alche
 """
 
 import asyncio
+import atexit
 import functools
 import os
 import queue
 import threading
-from typing import TYPE_CHECKING, Any, TypeVar, cast
+from typing import TYPE_CHECKING, Any, ClassVar, TypeVar, cast
 
 from sqlspec.exceptions import ImproperConfigurationError
 from sqlspec.utils.logging import get_logger
-from sqlspec.utils.singleton import SingletonMeta
 
 if TYPE_CHECKING:
     from collections.abc import Callable, Coroutine
 
 __all__ = ("Portal", "PortalManager", "PortalProvider", "get_global_portal")
 
-logger = get_logger("utils.portal")
+logger = get_logger("sqlspec.utils.portal")
 
 _R = TypeVar("_R")
 
@@ -112,6 +112,8 @@ class PortalProvider:
         """Stop the background thread and event loop.
 
         Gracefully shuts down the event loop and waits for thread to finish.
+        Only closes the loop after the thread has terminated to avoid
+        undefined behavior from closing a running loop.
         """
         if self._loop is None or self._loop_thread is None:
             logger.debug("Portal provider not running")
@@ -121,9 +123,10 @@ class PortalProvider:
         self._loop_thread.join(timeout=5)
 
         if self._loop_thread.is_alive():
-            logger.warning("Portal thread did not stop within 5 seconds")
+            logger.warning("Portal thread did not stop within 5 seconds, skipping loop.close()")
+        else:
+            self._loop.close()
 
-        self._loop.close()
         self._loop = None
         self._loop_thread = None
         self._ready_event.clear()
@@ -149,20 +152,21 @@ class PortalProvider:
         """Call an async function from synchronous context.
 
         Executes the async function in the background event loop and blocks
-        until the result is available.
+        until the result is available or timeout is reached.
 
         Args:
             func: The async function to call.
             *args: Positional arguments to the function.
-            **kwargs: Keyword arguments to the function.
+            **kwargs: Keyword arguments. Supports 'timeout' (float, default 300.0).
 
         Returns:
             Result of the async function.
 
         Raises:
-            ImproperConfigurationError: If portal provider not started.
+            ImproperConfigurationError: If portal provider not started or timeout reached.
 
         """
+        timeout: float = float(kwargs.pop("timeout", 300.0))
         if self._loop is None or not self.is_running:
             msg = "Portal provider not running. Call start() first."
             raise ImproperConfigurationError(msg)
@@ -173,7 +177,11 @@ class PortalProvider:
 
         self._loop.call_soon_threadsafe(self._process_request)
 
-        result, exception = local_result_queue.get()
+        try:
+            result, exception = local_result_queue.get(timeout=timeout)
+        except queue.Empty:
+            msg = f"Portal call timed out after {timeout} seconds"
+            raise ImproperConfigurationError(msg) from None
 
         if exception:
             raise exception
@@ -228,7 +236,7 @@ class Portal:
         Args:
             func: The async function to call.
             *args: Positional arguments to the function.
-            **kwargs: Keyword arguments to the function.
+            **kwargs: Keyword arguments. Supports 'timeout' (float, default 300.0).
 
         Returns:
             Result of the async function.
@@ -237,7 +245,7 @@ class Portal:
         return self._provider.call(func, *args, **kwargs)
 
 
-class PortalManager(metaclass=SingletonMeta):
+class PortalManager:
     """Singleton manager for global portal instance.
 
     Provides a global portal for use by sync_tools and other utilities
@@ -251,18 +259,35 @@ class PortalManager(metaclass=SingletonMeta):
 
     """
 
+    _instance: "ClassVar[PortalManager | None]" = None
+    _singleton_lock: "ClassVar[threading.Lock]" = threading.Lock()
+    _initialized: bool = False
+
+    def __new__(cls) -> "PortalManager":
+        """Get the singleton instance of PortalManager."""
+        if cls._instance is None:
+            with cls._singleton_lock:
+                if cls._instance is None:
+                    cls._instance = super().__new__(cls)
+        return cls._instance
+
     def __init__(self) -> None:
         """Initialize the PortalManager singleton."""
+        if hasattr(self, "_initialized") and self._initialized:
+            return
+
         self._provider: PortalProvider | None = None
         self._portal: Portal | None = None
         self._lock = threading.Lock()
         self._pid: int | None = None
+        self._atexit_registered: bool = False
+        self._initialized = True
 
     def get_or_create_portal(self) -> Portal:
         """Get or create the global portal instance.
 
         Lazily creates and starts the portal provider on first access.
-        Thread-safe via locking.
+        Thread-safe via locking. Registers an atexit handler for cleanup.
 
         Returns:
             Global portal instance.
@@ -278,10 +303,31 @@ class PortalManager(metaclass=SingletonMeta):
                     self._provider.start()
                     self._portal = Portal(self._provider)
                     self._pid = current_pid
+                    self._register_atexit()
                     logger.debug("Global portal provider created and started")
 
         return cast("Portal", self._portal)
 
+    def _register_atexit(self) -> None:
+        """Register atexit handler for graceful shutdown.
+
+        Only registers once per process to avoid duplicate cleanup.
+        """
+        if not self._atexit_registered:
+            atexit.register(self._atexit_cleanup)
+            self._atexit_registered = True
+            logger.debug("Portal atexit handler registered")
+
+    def _atexit_cleanup(self) -> None:
+        """Cleanup handler called at interpreter shutdown.
+
+        Gracefully stops the portal provider to ensure pending
+        async operations complete before the process exits.
+        """
+        if self._provider is not None and self._provider.is_running:
+            logger.debug("Portal atexit cleanup: stopping provider")
+            self.stop()
+
     @property
     def is_running(self) -> bool:
         """Check if global portal is running.
@@ -310,6 +356,11 @@ class PortalManager(metaclass=SingletonMeta):
         pid_changed = self._pid is not None and self._pid != current_pid
         return portal_missing or provider_missing or pid_changed
 
+    @classmethod
+    def get_instance(cls) -> "PortalManager":
+        """Get the singleton instance."""
+        return cls()
+
 
 def get_global_portal() -> Portal:
     """Get the global portal instance for async-to-sync bridging.
@@ -321,5 +372,4 @@ def get_global_portal() -> Portal:
         Global portal instance.
 
     """
-    manager = PortalManager()
-    return manager.get_or_create_portal()
+    return PortalManager().get_or_create_portal()
