@@ -9,24 +9,80 @@ from collections import defaultdict
 from collections.abc import Callable
 from contextlib import asynccontextmanager, contextmanager
 from decimal import Decimal
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, cast
 
 import pytest
 
-from sqlspec.core import SQL, ParameterStyle, ParameterStyleConfig, StatementConfig, TypedParameter, UnifiedCache
+from sqlspec.core import (
+    SQL,
+    LRUCache,
+    ParameterStyle,
+    ParameterStyleConfig,
+    StatementConfig,
+    TypedParameter,
+    get_default_cache,
+)
 from sqlspec.driver import (
     AsyncDataDictionaryBase,
     AsyncDriverAdapterBase,
     ExecutionResult,
     SyncDataDictionaryBase,
     SyncDriverAdapterBase,
-    VersionInfo,
 )
 from sqlspec.exceptions import SQLSpecError
+from sqlspec.typing import ColumnMetadata, ForeignKeyMetadata, IndexMetadata, TableMetadata, VersionInfo
+from tests.conftest import is_compiled
 
 if TYPE_CHECKING:
     from collections.abc import AsyncGenerator, Generator
 
+
+class MockSyncExceptionHandler:
+    """Mock sync exception handler for testing.
+
+    Implements the SyncExceptionHandler protocol with deferred exception pattern.
+    """
+
+    __slots__ = ("pending_exception",)
+
+    def __init__(self) -> None:
+        self.pending_exception: Exception | None = None
+
+    def __enter__(self) -> "MockSyncExceptionHandler":
+        return self
+
+    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> bool:
+        if exc_type is None:
+            return False
+        if isinstance(exc_val, Exception):
+            self.pending_exception = SQLSpecError(f"Mock database error: {exc_val}")
+            return True
+        return False
+
+
+class MockAsyncExceptionHandler:
+    """Mock async exception handler for testing.
+
+    Implements the AsyncExceptionHandler protocol with deferred exception pattern.
+    """
+
+    __slots__ = ("pending_exception",)
+
+    def __init__(self) -> None:
+        self.pending_exception: Exception | None = None
+
+    async def __aenter__(self) -> "MockAsyncExceptionHandler":
+        return self
+
+    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> bool:
+        if exc_type is None:
+            return False
+        if isinstance(exc_val, Exception):
+            self.pending_exception = SQLSpecError(f"Mock async database error: {exc_val}")
+            return True
+        return False
+
+
 __all__ = (
     "MockAsyncConnection",
     "MockAsyncCursor",
@@ -45,12 +101,12 @@ __all__ = (
     "mock_async_connection",
     "mock_async_driver",
     "mock_bigquery_connection",
+    "mock_lru_cache",
     "mock_mysql_connection",
     "mock_postgres_connection",
     "mock_sqlite_connection",
     "mock_sync_connection",
     "mock_sync_driver",
-    "mock_unified_cache",
     "parameter_style_config_advanced",
     "parameter_style_config_basic",
     "performance_timer",
@@ -184,10 +240,8 @@ def cache_config_disabled() -> dict[str, Any]:
 
 
 @pytest.fixture
-def mock_unified_cache() -> UnifiedCache:
-    """Mock unified cache for testing cache behavior."""
-
-    from sqlspec.core import get_default_cache
+def mock_lru_cache() -> LRUCache:
+    """Mock LRU cache for testing cache behavior."""
 
     return get_default_cache()
 
@@ -554,18 +608,44 @@ class MockAsyncCursor:
 class MockSyncDataDictionary(SyncDataDictionaryBase):
     """Mock sync data dictionary for testing."""
 
-    def get_version(self, driver: SyncDriverAdapterBase) -> "VersionInfo | None":
+    def get_version(self, driver: "MockSyncDriver") -> "VersionInfo | None":
         """Return mock version info."""
         return VersionInfo(3, 42, 0)
 
-    def get_feature_flag(self, driver: SyncDriverAdapterBase, feature: str) -> bool:
+    def get_feature_flag(self, driver: "MockSyncDriver", feature: str) -> bool:
         """Return mock feature flag."""
         return feature in {"supports_transactions", "supports_prepared_statements"}
 
-    def get_optimal_type(self, driver: SyncDriverAdapterBase, type_category: str) -> str:
+    def get_optimal_type(self, driver: "MockSyncDriver", type_category: str) -> str:
         """Return mock optimal type."""
         return {"text": "TEXT", "boolean": "INTEGER"}.get(type_category, "TEXT")
 
+    def get_tables(self, driver: "MockSyncDriver", schema: "str | None" = None) -> "list[TableMetadata]":
+        """Return mock table list."""
+        _ = (driver, schema)
+        return []
+
+    def get_columns(
+        self, driver: "MockSyncDriver", table: "str | None" = None, schema: "str | None" = None
+    ) -> "list[ColumnMetadata]":
+        """Return mock column metadata."""
+        _ = (driver, table, schema)
+        return []
+
+    def get_indexes(
+        self, driver: "MockSyncDriver", table: "str | None" = None, schema: "str | None" = None
+    ) -> "list[IndexMetadata]":
+        """Return mock index metadata."""
+        _ = (driver, table, schema)
+        return []
+
+    def get_foreign_keys(
+        self, driver: "MockSyncDriver", table: "str | None" = None, schema: "str | None" = None
+    ) -> "list[ForeignKeyMetadata]":
+        """Return mock foreign key metadata."""
+        _ = (driver, table, schema)
+        return []
+
     def list_available_features(self) -> "list[str]":
         """Return mock available features."""
         return ["supports_transactions", "supports_prepared_statements"]
@@ -574,18 +654,44 @@ class MockSyncDataDictionary(SyncDataDictionaryBase):
 class MockAsyncDataDictionary(AsyncDataDictionaryBase):
     """Mock async data dictionary for testing."""
 
-    async def get_version(self, driver: AsyncDriverAdapterBase) -> "VersionInfo | None":
+    async def get_version(self, driver: "MockAsyncDriver") -> "VersionInfo | None":
         """Return mock version info."""
         return VersionInfo(3, 42, 0)
 
-    async def get_feature_flag(self, driver: AsyncDriverAdapterBase, feature: str) -> bool:
+    async def get_feature_flag(self, driver: "MockAsyncDriver", feature: str) -> bool:
         """Return mock feature flag."""
         return feature in {"supports_transactions", "supports_prepared_statements"}
 
-    async def get_optimal_type(self, driver: AsyncDriverAdapterBase, type_category: str) -> str:
+    async def get_optimal_type(self, driver: "MockAsyncDriver", type_category: str) -> str:
         """Return mock optimal type."""
         return {"text": "TEXT", "boolean": "INTEGER"}.get(type_category, "TEXT")
 
+    async def get_tables(self, driver: "MockAsyncDriver", schema: "str | None" = None) -> "list[TableMetadata]":
+        """Return mock table list."""
+        _ = (driver, schema)
+        return []
+
+    async def get_columns(
+        self, driver: "MockAsyncDriver", table: "str | None" = None, schema: "str | None" = None
+    ) -> "list[ColumnMetadata]":
+        """Return mock column metadata."""
+        _ = (driver, table, schema)
+        return []
+
+    async def get_indexes(
+        self, driver: "MockAsyncDriver", table: "str | None" = None, schema: "str | None" = None
+    ) -> "list[IndexMetadata]":
+        """Return mock index metadata."""
+        _ = (driver, table, schema)
+        return []
+
+    async def get_foreign_keys(
+        self, driver: "MockAsyncDriver", table: "str | None" = None, schema: "str | None" = None
+    ) -> "list[ForeignKeyMetadata]":
+        """Return mock foreign key metadata."""
+        _ = (driver, table, schema)
+        return []
+
     def list_available_features(self) -> "list[str]":
         """Return mock available features."""
         return ["supports_transactions", "supports_prepared_statements"]
@@ -633,19 +739,15 @@ class MockSyncDriver(SyncDriverAdapterBase):
         finally:
             cursor.close()
 
-    @contextmanager
-    def handle_database_exceptions(self) -> "Generator[None, None, None]":
+    def handle_database_exceptions(self) -> "MockSyncExceptionHandler":
         """Handle database exceptions."""
-        try:
-            yield
-        except Exception as e:
-            raise SQLSpecError(f"Mock database error: {e}") from e
+        return MockSyncExceptionHandler()
 
-    def _try_special_handling(self, cursor: MockSyncCursor, statement: SQL) -> Any | None:
+    def dispatch_special_handling(self, cursor: MockSyncCursor, statement: SQL) -> Any | None:
         """Mock special handling - always return None."""
         return None
 
-    def _execute_statement(self, cursor: MockSyncCursor, statement: SQL) -> ExecutionResult:
+    def dispatch_execute(self, cursor: MockSyncCursor, statement: SQL) -> ExecutionResult:
         """Mock execute statement."""
         sql, prepared_parameters = self._get_compiled_sql(statement, self.statement_config)
         cursor.execute(sql, prepared_parameters)
@@ -661,7 +763,7 @@ class MockSyncDriver(SyncDriverAdapterBase):
 
         return self.create_execution_result(cursor, rowcount_override=cursor.rowcount)
 
-    def _execute_many(self, cursor: MockSyncCursor, statement: SQL) -> ExecutionResult:
+    def dispatch_execute_many(self, cursor: MockSyncCursor, statement: SQL) -> ExecutionResult:
         """Mock execute many."""
         sql, prepared_parameters = self._get_compiled_sql(statement, self.statement_config)
 
@@ -669,10 +771,11 @@ class MockSyncDriver(SyncDriverAdapterBase):
             msg = "execute_many requires parameters"
             raise ValueError(msg)
 
-        cursor.executemany(sql, prepared_parameters)
+        parameter_sets = cast("list[Any]", prepared_parameters)
+        cursor.executemany(sql, parameter_sets)
         return self.create_execution_result(cursor, rowcount_override=cursor.rowcount, is_many_result=True)
 
-    def _execute_script(self, cursor: MockSyncCursor, statement: SQL) -> ExecutionResult:
+    def dispatch_execute_script(self, cursor: MockSyncCursor, statement: SQL) -> ExecutionResult:
         """Mock execute script."""
         sql, prepared_parameters = self._get_compiled_sql(statement, self.statement_config)
         statements = self.split_script_statements(sql, self.statement_config, strip_trailing_semicolon=True)
@@ -698,6 +801,10 @@ class MockSyncDriver(SyncDriverAdapterBase):
         """Mock commit transaction."""
         self.connection.commit()
 
+    def _connection_in_transaction(self) -> bool:
+        """Check if connection is in transaction."""
+        return bool(self.connection.in_transaction)
+
 
 class MockAsyncDriver(AsyncDriverAdapterBase):
     """Mock async driver with adapter interface."""
@@ -741,19 +848,15 @@ class MockAsyncDriver(AsyncDriverAdapterBase):
         finally:
             await cursor.close()
 
-    @asynccontextmanager
-    async def handle_database_exceptions(self) -> "AsyncGenerator[None, None]":
+    def handle_database_exceptions(self) -> "MockAsyncExceptionHandler":
         """Handle database exceptions."""
-        try:
-            yield
-        except Exception as e:
-            raise SQLSpecError(f"Mock async database error: {e}") from e
+        return MockAsyncExceptionHandler()
 
-    async def _try_special_handling(self, cursor: MockAsyncCursor, statement: SQL) -> Any | None:
+    async def dispatch_special_handling(self, cursor: MockAsyncCursor, statement: SQL) -> Any | None:
         """Mock async special handling - always return None."""
         return None
 
-    async def _execute_statement(self, cursor: MockAsyncCursor, statement: SQL) -> ExecutionResult:
+    async def dispatch_execute(self, cursor: MockAsyncCursor, statement: SQL) -> ExecutionResult:
         """Mock async execute statement."""
         sql, prepared_parameters = self._get_compiled_sql(statement, self.statement_config)
         await cursor.execute(sql, prepared_parameters)
@@ -769,7 +872,7 @@ class MockAsyncDriver(AsyncDriverAdapterBase):
 
         return self.create_execution_result(cursor, rowcount_override=cursor.rowcount)
 
-    async def _execute_many(self, cursor: MockAsyncCursor, statement: SQL) -> ExecutionResult:
+    async def dispatch_execute_many(self, cursor: MockAsyncCursor, statement: SQL) -> ExecutionResult:
         """Mock async execute many."""
         sql, prepared_parameters = self._get_compiled_sql(statement, self.statement_config)
 
@@ -777,10 +880,11 @@ class MockAsyncDriver(AsyncDriverAdapterBase):
             msg = "execute_many requires parameters"
             raise ValueError(msg)
 
-        await cursor.executemany(sql, prepared_parameters)
+        parameter_sets = cast("list[Any]", prepared_parameters)
+        await cursor.executemany(sql, parameter_sets)
         return self.create_execution_result(cursor, rowcount_override=cursor.rowcount, is_many_result=True)
 
-    async def _execute_script(self, cursor: MockAsyncCursor, statement: SQL) -> ExecutionResult:
+    async def dispatch_execute_script(self, cursor: MockAsyncCursor, statement: SQL) -> ExecutionResult:
         """Mock async execute script."""
         sql, prepared_parameters = self._get_compiled_sql(statement, self.statement_config)
         statements = self.split_script_statements(sql, self.statement_config, strip_trailing_semicolon=True)
@@ -806,6 +910,10 @@ class MockAsyncDriver(AsyncDriverAdapterBase):
         """Mock async commit transaction."""
         await self.connection.commit()
 
+    def _connection_in_transaction(self) -> bool:
+        """Check if connection is in transaction."""
+        return bool(self.connection.in_transaction)
+
 
 @pytest.fixture
 def mock_sync_connection() -> MockSyncConnection:
@@ -822,12 +930,16 @@ def mock_async_connection() -> MockAsyncConnection:
 @pytest.fixture
 def mock_sync_driver(mock_sync_connection: MockSyncConnection) -> MockSyncDriver:
     """Fixture for mock sync driver."""
+    if is_compiled():
+        pytest.skip("Requires interpreted driver base")
     return MockSyncDriver(mock_sync_connection)
 
 
 @pytest.fixture
 def mock_async_driver(mock_async_connection: MockAsyncConnection) -> MockAsyncDriver:
     """Fixture for mock async driver."""
+    if is_compiled():
+        pytest.skip("Requires interpreted driver base")
     return MockAsyncDriver(mock_async_connection)
 
 
